/*
* ==============================================================================
*  Name        : BrowserControl_Interface.h
*  Part of     : Browser Control.
*  Description : Defines a browser control interface.
*  Version     :
*
*    Copyright (c) 2007,
*    All rights reserved.
* ==============================================================================
*/
#ifndef __BROWSERCONTROL_INTERFACE_H__
#define __BROWSERCONTROL_INTERFACE_H__


#include "BrowserControl_DataTypes.h" 
#ifdef SFB_VISUAL_SUPPORT
#include "WmGrpType.h"//tahseen
#endif
#ifdef __cplusplus
extern "C"
{
#endif

#ifdef __SHP__
typedef void BrowserWidget;
#else /*<TODO> Put GDK platform condition*/
#include <gtk/gtk.h>
typedef GtkWidget BrowserWidget;
#endif

/**
 * @defgroup Init Creation and Destruction
 */
/**
 * @defgroup Page Page related APIs
 */
/**
 * @defgroup Observer Observer related APIs
 */
/**
 * @defgroup Settings Setting related APIs
 */
/**
 * @defgroup Error Error handling related APIs
 */
/**
 * @defgroup General General
 */



//template
/*
* @function Function_Name
* @brief Purpose : Line 1
* Line 2
* @param type1 param_name1 description1
* @return type1 description
* @ingroup group_name
*/

//------------------------------------------------------------------------
// WebKit initialize and finalize APIs
//------------------------------------------------------------------------

/**
* BrowserControl_WebKit_Initialize
* @brief Purpose : Initializes the WebKit. This function initializes all
* the data that should be done before initializing the browser controls.
* @param[in] HWIN hMainWin Handle to the main browser app window
* @return ESTATUS : ESUCCESS if the WebKit has been initialized successfully,
* else EFAIL. Exact reason of the failure can be queried using 
* BrowserControl_GetLastError().
* @ingroup Init
*/
ESTATUS BrowserControl_WebKit_Initialize(HWIN hMainWin);

/**
* BrowserControl_WebKit_Finalize
* @brief Purpose : Finalizes the WebKit.
* @return ESTATUS : ESUCCESS if the WebKit has been successfully
* finalized, else EFAIL. Exact reason of the failure can be queried using
* BrowserControl_GetLastError().
* @ingroup Init
*/
ESTATUS BrowserControl_WebKit_Finalize();

//------------------------------------------------------------------------
// Creation and Destruction API
//------------------------------------------------------------------------

/**
* BrowserControl_Init
* @brief Purpose : Creates an instance of the BrowserControl
* and returns appropriate handle to it. BrowserControl_Init is the first
* phase of the Browser Control creation. Here it creates instances
* of all the supporting classes.
* @param[in] HANDLE* aBrowserHandle Handle to the BrowserControl
* @return ESTATUS : ESUCCESS if the BrowserControl instance could be created,
* else EFAIL.Exact reason of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Init
*/
ESTATUS BrowserControl_Init(HANDLE* aBrowserHandle);

/**
* BrowserControl_Finalize
* @brief Purpose : Deletes an instance of the BrowserControl
* @param[in] HANDLE aBrowserHandle Handle to the BrowserControl
* @return ESTATUS : ESUCCESS if the BrowserControl instance could be deleted,
* else EFAIL.Exact reason of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Init
*/
ESTATUS BrowserControl_Finalize(HANDLE aBrowserHandle);

/**
* BrowserControl_Create
* @brief Purpose : Second phase of the BrowserControl creation.Creates the
* widget instance which uses services of the supporting classes created in
* the first phase i.e. in BrowserControl_Init.
* @param[in] HANDLE aBrowserHandle Handle to the BrowserControl
* @param[out] GtkWidget* aWidget Pointer to the widget that is created.
* @return ESTATUS : ESUCCESS if the BrowserControl widget was successfully
* created, else EFAIL. Exact reason of the failure can be queried using
* BrowserControl_GetLastError().
* @ingroup Init
*/
ESTATUS BrowserControl_Create(HANDLE aBrowserHandle, BrowserWidget** aWidget);

/**
* BrowserControl_Delete
* @brief Purpose : Deletes the instance of the BrowserControl.
* @param[in] HANDLE aBrowserHandle Handle to the BrowserControl
* @return ESTATUS : ESUCCESS if the BrowserControl was successfully
* deleted, else EFAIL. Exact reason of the failure can be queried using
* BrowserControl_GetLastError().
* @ingroup Init
*/
ESTATUS BrowserControl_Delete(HANDLE aBrowserHandle);

//------------------------------------------------------------------------
// Settings API
//------------------------------------------------------------------------

/**
*  BrowserControl_SetHwnd
* @brief Purpose : Sets the graphics context.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] HWIN hWin : hWin, a handle window object.
* @return ESTATUS : ESUCCESS if the gc was set,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
ESTATUS BrowserControl_SetHwnd(HANDLE aBrowserHandle,const HWIN hWin);

/**
*  BrowserControl_SetHttpProxy
* @brief Purpose : Sets the server and port to be used as a proxy.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] SProxyInfo* aProxyInfo : Pointer to structure containing proxy info.
* @return ESTATUS : ESUCCESS if the proxy was set,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
ESTATUS BrowserControl_SetHttpProxy(HANDLE aBrowserHandle,const SProxyInfo* aProxyInfo);

/**
*  BrowserControl_SetIAP
* @brief Purpose : Sets the IAP(Internet Access Point) to be used for
* connection purpose.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] IAPHANDLE aSelectedIapHandle : Handle to the IAP to be used by
* this application for network connection.
* @return ESTATUS : ESUCCESS if the IAP was set,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
ESTATUS BrowserControl_SetIAP(HANDLE aBrowserHandle,const IAPHANDLE aSelectedIapHandle);

/**
*  BrowserControl_SetCacheSize
* @brief Purpose : Sets the size of the persistent store to be used
* by the cache.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] guint aCacheSize : Cache size to be set.The value specified
* is measured in bytes.
* @return ESTATUS : ESUCCESS if the property was set,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
ESTATUS BrowserControl_SetCacheSize(HANDLE aBrowserHandle
								,const guint aCacheSize);

/**
*  BrowserControl_SetProperty
* @brief Purpose : Enables/Disables a partcular browser property.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] EBrowserProperty aBrowserProperty : Property to be set.
* @param[in] gboolean aSettingValue : TRUE to enable the setting else FALSE.
* @return ESTATUS : ESUCCESS if the property was set,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
ESTATUS BrowserControl_SetProperty(HANDLE aBrowserHandle
		,const EBrowserProperty aBrowserProperty,const gboolean aSettingValue);

/**
*  BrowserControl_GetHttpProxy
* @brief Purpose : Gets the server and port to be used as a proxy.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[out] SProxyInfo* aProxyInfo : Pointer to structure to be filled with proxy info.
* @return ESTATUS : ESUCCESS if the query operation was successful, else EFAIL.
* Exact reason of the failure can be queried using 
* BrowserControl_GetLastError().
* @ingroup Settings
*/
#ifdef __SHP__
ESTATUS BrowserControl_GetHttpProxy(HANDLE aBrowserHandle,SProxyInfo* aProxyInfo);

/**
*  BrowserControl_GetIAP
* @brief Purpose : Gets the IAP(Internet Access Point) to be used for
* connection purpose.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] IAPHANDLE* aSelectedIapHandle : Handle to the IAP to be used by
* this application for network connection.
* @return ESTATUS : ESUCCESS if the query operation was successful, else EFAIL.
* Exact reason of the failure can be queried using 
* BrowserControl_GetLastError().
* @ingroup Settings
*/
ESTATUS BrowserControl_GetIAP(HANDLE aBrowserHandle,IAPHANDLE* aSelectedIapHandle);

/**
*  BrowserControl_GetCacheSize
* @brief Purpose : Gets the size of the persistent store to be used
* by the cache.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[out] guint* aCacheSize : Cache size .The value specified
* is measured in bytes.
* @return ESTATUS : ESUCCESS if the query operation was successful, else EFAIL.
* Exact reason of the failure can be queried using 
* BrowserControl_GetLastError().
* @ingroup Settings
*/
ESTATUS BrowserControl_GetCacheSize(HANDLE aBrowserHandle
								,guint* aCacheSize);

/**
*  BrowserControl_GetProperty
* @brief Purpose : Queries the value of a particular browser property.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] EBrowserProperty aBrowserProperty : Property to be queried.
* @param[out] gboolean* aSettingValue : TRUE if the setting is enabled else FALSE.
* @return ESTATUS : ESUCCESS if the query operation was successful, else EFAIL.
* Exact reason of the failure can be queried using 
* BrowserControl_GetLastError().
* @ingroup Settings
*/
ESTATUS BrowserControl_GetProperty(HANDLE aBrowserHandle
		,const EBrowserProperty aBrowserProperty,gboolean* aSettingValue);
/**
*  BrowserControl_SetDefaultFontSize
* @brief Purpose : Sets the Default Font Size.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] int aSettingsValue : Default font size.
* @return ESTATUS : ESUCCESS if the property was set,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
WEBKIT_SHP_API ESTATUS BrowserControl_SetDefaultFontSize(HANDLE aBrowserHandle
								,const int aSettingsValue);
/**
*  BrowserControl_SetDefaultFixedFontSize
* @brief Purpose : Sets the Default Font Size.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] int aSettingsValue : Default fixed font size.
* @return ESTATUS : ESUCCESS if the property was set,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
WEBKIT_SHP_API ESTATUS BrowserControl_SetDefaultFixedFontSize(HANDLE aBrowserHandle
								,const int aSettingsValue);
/**
*  BrowserControl_SetMinimumLogicalFontSize
* @brief Purpose : Sets Minimum Logical Font Size .
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] int aSettingsValue : Minimum Logical Font Size.
* @return ESTATUS : ESUCCESS if the property was set,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
WEBKIT_SHP_API ESTATUS BrowserControl_SetMinimumLogicalFontSize(HANDLE aBrowserHandle
								,const int aSettingsValue);
/**
*  BrowserControl_SetMinimumFontSize
* @brief Purpose : Sets Minimum Font Size.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] int aSettingsValue : Minimum Font Size.
* @return ESTATUS : ESUCCESS if the property was set,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/

WEBKIT_SHP_API ESTATUS BrowserControl_SetMinimumFontSize(HANDLE aBrowserHandle
								,const int aSettingsValue);
/**
*  BrowserControl_SetDefaultTextEncodingName
* @brief Purpose : Sets Default Text Encoding Name .
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] const char * aSettingsValue: Default Text Encoding Name.
* @return ESTATUS : ESUCCESS if the property was set,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
WEBKIT_SHP_API ESTATUS BrowserControl_SetDefaultTextEncodingName(HANDLE aBrowserHandle
								,const char * aSettingsValue);
/**
*  BrowserControl_SetFtpDirectoryTemplatePath
* @brief Purpose : Sets Ftp Directory TemplatePath .
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] char * aSettingsValue :Ftp DirectoryT emplate Path.
* @return ESTATUS : ESUCCESS if the path was set,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/

WEBKIT_SHP_API ESTATUS BrowserControl_SetFtpDirectoryTemplatePath(HANDLE aBrowserHandle,
																  char *aSettingsValue);
/**
*  BrowserControl_SetUserStyleSheetLocation
* @brief Purpose : Sets User Style Sheet Location .
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] const char * aSettingsValue :User Style Sheet Location .
* @return ESTATUS : ESUCCESS if the property was set,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
WEBKIT_SHP_API ESTATUS BrowserControl_SetUserStyleSheetLocation(HANDLE aBrowserHandle
								,const char * aSettingsValue);
/**
*  BrowserControl_SetStandardFontFamily
* @brief Purpose : Sets Standard Font Family .
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] const char * aSettingsValue : Standard Font Family.
* @return ESTATUS : ESUCCESS if the property was set,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
WEBKIT_SHP_API ESTATUS BrowserControl_SetStandardFontFamily(HANDLE aBrowserHandle
								,const char * aSettingsValue);
/**
*  BrowserControl_SetFixedFontFamily
* @brief Purpose : Sets Fixed Font Family .
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] const char * aSettingsValue : Fixed Font Family.
* @return ESTATUS : ESUCCESS if the property was set,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
WEBKIT_SHP_API ESTATUS BrowserControl_SetFixedFontFamily(HANDLE aBrowserHandle
								,const char * aSettingsValue);
/**
*  BrowserControl_SetSerifFontFamily
* @brief Purpose : Sets Serif Font Family.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] const char * aSettingsValue : Serif Font Family .
* @return ESTATUS : ESUCCESS if the property was set,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
WEBKIT_SHP_API ESTATUS BrowserControl_SetSerifFontFamily(HANDLE aBrowserHandle
								,const char * aSettingsValue);
/**
*  BrowserControl_SetSansSerifFontFamily
* @brief Purpose : Sets Sans Serif Font Family .
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] const char * aSettingsValue : Sans Serif Font Family.
* @return ESTATUS : ESUCCESS if the property was set,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
WEBKIT_SHP_API ESTATUS BrowserControl_SetSansSerifFontFamily(HANDLE aBrowserHandle
								,const char * aSettingsValue);
/**
*  BrowserControl_SetCursiveFontFamily
* @brief Purpose : Sets Cursive Font Family .
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] const char * aSettingsValue : Cursive Font Family.
* @return ESTATUS : ESUCCESS if the property was set,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
WEBKIT_SHP_API ESTATUS BrowserControl_SetCursiveFontFamily(HANDLE aBrowserHandle
								,const char * aSettingsValue);
/**
*  BrowserControl_SetFantasyFontFamily
* @brief Purpose : Sets Fantasy Font Family .
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] const char * aSettingsValue : Fantasy Font Family.
* @return ESTATUS : ESUCCESS if the property was set,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
WEBKIT_SHP_API ESTATUS BrowserControl_SetFantasyFontFamily(HANDLE aBrowserHandle
								,const char * aSettingsValue);
/**
*  BrowserControl_SetEditableLinkBehavior
* @brief Purpose : Sets Editable Link Behavior .
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] const EEditableLinkBehavior aSettingsValue : Editable Link Behavior.
* @return ESTATUS : ESUCCESS if the property was set,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
WEBKIT_SHP_API ESTATUS BrowserControl_SetEditableLinkBehavior(HANDLE aBrowserHandle
							,const EEditableLinkBehavior aSettingsValue);
/**
*  BrowserControl_GetDefaultFontSize
* @brief Purpose : Gets Default Font Size .
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[out] guint aSettingsValue : Default Font Size.
* @return ESTATUS : ESUCCESS if the query operation was successful,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
WEBKIT_SHP_API ESTATUS BrowserControl_GetDefaultFontSize(HANDLE aBrowserHandle
								,int *aSettingsValue);
/**
*  BrowserControl_GetDefaultFixedFontSize
* @brief Purpose : Gets Fixed Default Font Size .
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[out] guint aSettingsValue : Default Font Size.
* @return ESTATUS : ESUCCESS if the query operation was successful,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
WEBKIT_SHP_API ESTATUS BrowserControl_GetDefaultFixedFontSize(HANDLE aBrowserHandle
								,int *aSettingsValue);
/**
*  BrowserControl_GetMinimumLogicalFontSize
* @brief Purpose : Gets Minimum Logical Font Size .
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[out] guint aSettingsValue : Minimum Logical Font Size.
* @return ESTATUS : ESUCCESS if the query operation was successful,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
WEBKIT_SHP_API ESTATUS BrowserControl_GetMinimumLogicalFontSize(HANDLE aBrowserHandle
								,int *aSettingsValue);
/**
*  BrowserControl_GetMinimumFontSize
* @brief Purpose : Gets Minimum Font Size.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[out] guint aSettingsValue : Minimum Font Size.
* @return ESTATUS : ESUCCESS if the query operation was successful,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/

WEBKIT_SHP_API ESTATUS BrowserControl_GetMinimumFontSize(HANDLE aBrowserHandle
								,int *aSettingsValue);
/**
*  BrowserControl_GetDefaultTextEncodingName
* @brief Purpose : Gets Default Text Encoding Name.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[out] gchar *aSettingsValue : Default Text Encoding Name.
* @param[in, out] gulong *aSize : Size of the buffer allocated to aSettingsValue. 
* If the buffer is not enough, the required size is sent back in aSize. 
* If aSettingsValue is NULL, then the required size is sent back in aSize and EFAIL is returned.
* @return ESTATUS : ESUCCESS if the query operation was successful,else EFail.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
ESTATUS BrowserControl_GetDefaultTextEncodingName(HANDLE aBrowserHandle,gchar *aSettingsValue,gulong *aSize);
/**
*  BrowserControl_GetFtpDirectoryTemplatePath
* @brief Purpose : Gets Ftp Directory Template Path.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[out] gchar *aSettingsValue : Ftp Directory Template Path .
* @param[in, out] gulong *aSize : Size of the buffer allocated to aSettingsValue. 
* If the buffer is not enough, the required size is sent back in aSize. 
* If aSettingsValue is NULL, then the required size is sent back in aSize and EFAIL is returned.
* @return ESTATUS : ESUCCESS if the query operation was successful,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
ESTATUS BrowserControl_GetFtpDirectoryTemplatePath(HANDLE aBrowserHandle,gchar *aSettingsValue,gulong *aSize);
/**
*  BrowserControl_GetUserStyleSheetLocation
* @brief Purpose : Gets User Style Sheet Location.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[out] gchar *aSettingsValue : Serif Font Family .
* @param[in, out] gulong *aSize : Size of the buffer allocated to aSettingsValue. 
* If the buffer is not enough, the required size is sent back in aSize. 
* If aSettingsValue is NULL, then the required size is sent back in aSize and EFAIL is returned.
* @return ESTATUS : ESUCCESS if the query operation was successful,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
ESTATUS BrowserControl_GetUserStyleSheetLocation(HANDLE aBrowserHandle,gchar *aSettingsValue,gulong *aSize);
/**
*  BrowserControl_GetStandardFontFamily
* @brief Purpose : Gets Standard Font Family.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[out] gchar *aSettingsValue : Standard Font Family .
* @param[in, out] gulong *aSize : Size of the buffer allocated to aSettingsValue. 
* If the buffer is not enough, the required size is sent back in aSize. 
* If aSettingsValue is NULL, then the required size is sent back in aSize and EFAIL is returned.
* @return ESTATUS : ESUCCESS if the query operation was successful,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
ESTATUS BrowserControl_GetStandardFontFamily(HANDLE aBrowserHandle,gchar *aSettingsValue,gulong *aSize);
/**
*  BrowserControl_GetFixedFontFamily
* @brief Purpose : Gets Fixed Font Family.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[out] gchar *aSettingsValue : Fixed Font Family .
* @param[in, out] gulong *aSize : Size of the buffer allocated to aSettingsValue. 
* If the buffer is not enough, the required size is sent back in aSize. 
* If aSettingsValue is NULL, then the required size is sent back in aSize and EFAIL is returned.
* @return ESTATUS : ESUCCESS if the query operation was successful,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
ESTATUS BrowserControl_GetFixedFontFamily(HANDLE aBrowserHandle,gchar *aSettingsValue,gulong *aSize);
/**
*  BrowserControl_GetSerifFontFamily
* @brief Purpose : Gets Serif Font Family.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[out] gchar *aSettingsValue : Serif Font Family .
* @param[in, out] gulong *aSize : Size of the buffer allocated to aSettingsValue. 
* If the buffer is not enough, the required size is sent back in aSize. 
* If aSettingsValue is NULL, then the required size is sent back in aSize and EFAIL is returned.
* @return ESTATUS : ESUCCESS if the query operation was successful,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
ESTATUS BrowserControl_GetSerifFontFamily(HANDLE aBrowserHandle,gchar *aSettingsValue,gulong *aSize);
/**
*  BrowserControl_GetSansSerifFontFamily
* @brief Purpose : Gets SansSerif Font Family.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[out] gchar *aSettingsValue : SansSerif Font Family .
* @param[in, out] gulong *aSize : Size of the buffer allocated to aSettingsValue. 
* If the buffer is not enough, the required size is sent back in aSize. 
* If aSettingsValue is NULL, then the required size is sent back in aSize and EFAIL is returned.
* @return ESTATUS : ESUCCESS if the query operation was successful,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
ESTATUS BrowserControl_GetSansSerifFontFamily(HANDLE aBrowserHandle,gchar *aSettingsValue,gulong *aSize);
/**
*  BrowserControl_GetCursiveFontFamily
* @brief Purpose : Gets Cursive Font Family.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[out] gchar *aSettingsValue : Cursive Font Family .
* @param[in, out] gulong *aSize : Size of the buffer allocated to aSettingsValue. 
* If the buffer is not enough, the required size is sent back in aSize. 
* If aSettingsValue is NULL, then the required size is sent back in aSize and EFAIL is returned.
* @return ESTATUS : ESUCCESS if the query operation was successful,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
ESTATUS BrowserControl_GetCursiveFontFamily(HANDLE aBrowserHandle,gchar *aSettingsValue,gulong *aSize);
/**
*  BrowserControl_GetFantasyFontFamily
* @brief Purpose : Gets Fantasy Font Family.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[out] gchar *aSettingsValue : Fantasy Font Family .
* @param[in, out] gulong *aSize : Size of the buffer allocated to aSettingsValue. 
* If the buffer is not enough, the required size is sent back in aSize. 
* If aSettingsValue is NULL, then the required size is sent back in aSize and EFAIL is returned.
* @return ESTATUS : ESUCCESS if the query operation was successful,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
ESTATUS BrowserControl_GetFantasyFontFamily(HANDLE aBrowserHandle,gchar *aSettingsValue,gulong *aSize);

/**
*  BrowserControl_GetEditableLinkBehavior
* @brief Purpose : Gets Editable Link Behavior .
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[out] EEditableLinkBehavior aSettingsValue : Editable Link Behavior.
* @return ESTATUS : ESUCCESS if the query operation was successful,else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Settings
*/
WEBKIT_SHP_API ESTATUS BrowserControl_GetEditableLinkBehavior(HANDLE aBrowserHandle
								,EEditableLinkBehavior *aSettingsValue);
#endif //#ifdef __SHP__
//------------------------------------------------------------------------
// Page API
//------------------------------------------------------------------------

/**
*  BrowserControl_SwitchTab
* @brief Purpose : Switches the tab to the browser handle
* given.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @return ESTATUS : ESUCCESS if tab has been switched, else
* EFAIL.Exact reason of the failure can be queried using
* BrowserControl_GetLastError().
* @ingroup Page
*/
ESTATUS BrowserControl_SwitchTab(HANDLE aBrowserHandle);

/**
*  BrowserControl_LoadUrl
* @brief Purpose : Loads the resource pointed by aUrl and displays it
* in the browser control.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] gchar* aUrl : Url of the resource to be loaded.
* @return ESTATUS : ESUCCESS if resource was successfully loaded, else
* EFAIL.Exact reason of the failure can be queried using
* BrowserControl_GetLastError().
* @ingroup Page
*/
ESTATUS BrowserControl_LoadUrl(HANDLE aBrowserHandle, const gchar* aUrl);
/**
*  BrowserControl_LoadFile
* @brief Purpose : Loads the file pointed by aPath and displays it
* in the browser control.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] gchar* aPath : Path of the file to be loaded.
* @return ESTATUS : ESUCCESS if file was successfully loaded, else
* EFAIL.Exact reason of the failure can be queried using
* BrowserControl_GetLastError().
* @ingroup Page
*/
ESTATUS BrowserControl_LoadFile(HANDLE aBrowserHandle, const gchar* aPath);
/**
*  BrowserControl_LoadData
* @brief Purpose : Displays the passed data in the browser control.
* This API is meant to load a script data for scripts supported by the browser engine.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] gchar* aData : Data to be loaded.
* @return ESTATUS : ESUCCESS if data was successfully loaded, else
* EFAIL.Exact reason of the failure can be queried using
* BrowserControl_GetLastError().
* @ingroup Page
*/
ESTATUS BrowserControl_LoadData(HANDLE aBrowserHandle, const gchar* aData);
/**
*  BrowserControl_PostUrl
* @brief Purpose : Posts the url.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] gchar* aUrl : Url to be posted.
* @param[in] gchar* aContentType : Content type of the data to be posted
* @param[in] gchar* aPostData : Data to be posted
* @param[in] gchar* aBoundary : Boundary for form-data or multi-part content.
* @return ESTATUS : ESUCCESS if url was successfully posted, else
* EFAIL.Exact reason of the failure can be queried using
* BrowserControl_GetLastError().
* @ingroup Page
*/
ESTATUS BrowserControl_PostUrl(HANDLE aBrowserHandle,
								const gchar* aUrl,
								const gchar* aContentType,
								const gchar* aPostData,
								const gchar* aBoundary);

/**
*  BrowserControl_GetPageUrl
* @brief Purpose : Retrieves the url of the currently loaded page.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[out] gchar* aUrl : Url of the currently loaded page.
* @param[in,out] gulong* aSize : Size of the buffer. If the buffer is not enough,
* the required size is sent back in aSize.
* @return ESTATUS : ESUCCESS if the Url for the current page could
* be retrieved, else EFAIL.Exact reason of the failure can be queried
* using BrowserControl_GetLastError().
* @ingroup Page
*/
ESTATUS BrowserControl_GetPageUrl(HANDLE aBrowserHandle, gchar* aUrl, gulong* aSize);
/**
*  BrowserControl_GetPageTitle
* @brief Purpose : Retrieves the title of the currently loaded page.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[out] gchar* aTitle : Title of the currently loaded page.
* @param[in,out] gulong* aSize : Size of the buffer. If the buffer is not enough,
* the required size is sent back in aSize.
* @return ESTATUS : ESUCCESS if the Title for the current page could
* be retrieved, else EFAIL.Exact reason of the failure can be queried
* using BrowserControl_GetLastError().
* @ingroup Page
*/
ESTATUS BrowserControl_GetPageTitle(HANDLE aBrowserHandle, gchar* aTitle, gulong* aSize);
/**
*  BrowserControl_GetPageMimeType
* @brief Purpose : Retrieves the mime type of the currently loaded page.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[out] gchar* aType : Mime Type of the currently loaded page.
* @param[in,out] gulong* aSize : Size of the buffer. If the buffer is not enough,
* the required size is sent back in aSize.
* @return ESTATUS : ESUCCESS if the Mime Type for the current page could
* be retrieved, else EFAIL.Exact reason of the failure can be queried
* using BrowserControl_GetLastError().
* @ingroup Page
*/
ESTATUS BrowserControl_GetPageMimeType(HANDLE aBrowserHandle, gchar* aType, gulong* aSize);
/**
*  BrowserControl_GetPageSize
* @brief Purpose : Retrieves the size in bytes of the currently loaded page.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[out] gulong* aSize : Size in bytes of the currently loaded page. 
* @return ESTATUS : ESUCCESS if the Size of the current page could
* be retrieved, else EFAIL.Exact reason of the failure can be queried
* using BrowserControl_GetLastError().
* @ingroup Page
*/
ESTATUS BrowserControl_GetPageSize(HANDLE aBrowserHandle, gulong* aSize);
/**
*  BrowserControl_GetPageContent
* @brief Purpose : Retrieves the contents of the currently loaded page.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[out] gchar* aContent : Contents of the currently loaded page.
* @param[in,out] gulong* aSize Size of the buffer. If the buffer is not enough,
* the required size is sent back in aSize.
* @return ESTATUS : ESUCCESS if the Content for the current page could
* be retrieved, else EFAIL.Exact reason of the failure can be queried
* using BrowserControl_GetLastError().
* @ingroup Page
*/
ESTATUS BrowserControl_GetPageContent(HANDLE aBrowserHandle, gchar* aContent, gulong* aSize);
/**
*  BrowserControl_GetPageLastModifiedTime
* @brief Purpose : Retrieves the last modified time of the currently loaded page.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[out] gchar* aTime : Last modified time of the currently loaded page.
* @param[in,out] gulong* aSize Size of the buffer. If the buffer is not enough,
* the required size is sent back in aSize.
* @return ESTATUS : ESUCCESS if the Content for the current page could
* be retrieved, else EFAIL.Exact reason of the failure can be queried
* using BrowserControl_GetLastError().
* @ingroup Page
*/
ESTATUS BrowserControl_GetPageLastModifiedTime(HANDLE aBrowserHandle, gchar* aTime, gulong* aSize);
/**
*  BrowserControl_GetSelectedElementType
* @brief Purpose : Gets the type of the currently selected item.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[out] EPageItemType aType : Type of the selected element is returned in this.
* @return ESTATUS : ESUCCESS if the count of items could be retrieved,
* else EFAIL.Exact reason of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Page
*/
ESTATUS BrowserControl_GetSelectedElementType(HANDLE aBrowserHandle, EPageItemType* aType);
/**
*  BrowserControl_GetSelectedElementInfo
* @brief Purpose : Gets the information of the currently selected item.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] EPageItemType aType : Type of the element. Info is fetched only if the 
* currently selected element is of type aTtype.
* @param[in,out]  ElementInfoBase* aInfo : Info of the selected element is returned in this.
* aInfo will contain length parameter that is Size of the buffer. 
* If the buffer is not enough,
* the required length is sent back in aType->length.
* @return ESTATUS : ESUCCESS if the information of the selected item could be retrieved,
* else EFAIL.Exact reason of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Page
*/

/**
 *   bs0425.lee  modified for touch events 
 */

ESTATUS BrowserControl_GetSelectedElementTypeByPoint(HANDLE aBrowserHandle, EPageItemType* aType, int x, int y);
/**
 *   bs0425.lee  modified for touch events 
 */


ESTATUS BrowserControl_GetSelectedElementInfo(HANDLE aBrowserHandle, EPageItemType aType, ElementInfoBase* aInfo);
/**
*  BrowserControl_GetElementTypeUnderPoint
* @brief Purpose : Gets the type of the item under a point. This can be used by the application
* to show a menu or handle mouse/key events over the browser control.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] guint aX : X coordinate.
* @param[in] guint aY : Y coordinate.
* @param[out] EPageItemType aType : Type of the selected element is returned in this.
* @return ESTATUS : ESUCCESS if the count of items could be retrieved,
* else EFAIL.Exact reason of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Page
*/
ESTATUS BrowserControl_GetElementTypeUnderPoint(HANDLE aBrowserHandle,
												const guint aX,
												const guint aY,
												EPageItemType* aType);
/**
*  BrowserControl_GetItemCount
* @brief Purpose : retrieves the Count of the items in the page.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] EPageItemType aType : Type of the items to be counted.
* @param[out] guint* aCount : Count of the items in the page.
* @return ESTATUS : ESUCCESS if the count of items could be retrieved,
* else EFAIL.Exact reason of the failure can be queried using
* BrowserControl_GetLastError().
* @ingroup Page
*/
ESTATUS BrowserControl_GetItemCount(HANDLE aBrowserHandle, const EPageItemType aType, guint* aCount);
/**
*  BrowserControl_SavePage
* @brief Purpose : Saves the currently loaded page in a local file.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] gchar* aPath : Path of the file in which the page must be saved.
* @return ESTATUS : ESUCCESS if the current page could be saved, else
* EFAIL.Exact reason of the failure can be queried using
* BrowserControl_GetLastError().
* @ingroup Page
*/
ESTATUS BrowserControl_SavePage(HANDLE aBrowserHandle, const gchar* aPath);

/**
* BrowserControl_GetImages
* @brief Purpose : Gets list of images in the current page.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] bool bVisibleOnly : Return only visible images.
* @param[out] ImageSaveInfo** imageList : List to hold images.
* @return ESTATUS : ESUCCESS if images found, else EFAIL.
* Exact reason of the failure can be queried using
* BrowserControl_GetLastError().
* Please note user of this API need to free all the memory allocated to imageList.
* @ingroup Page
*/
ESTATUS BrowserControl_GetImages(HANDLE aBrowserHandle, BOOL bVisibleOnly, ImageSaveInfo*** imageList);

/**
* BrowserControl_GetImageCount
* @brief Purpose : Gets number of images in the current page.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] bool bVisibleOnly : Return only visible images.
* @param[out] guint* imageCount : Number of images.
* @return ESTATUS : ESUCCESS if images found, else EFAIL.
* Exact reason of the failure can be queried using
* BrowserControl_GetLastError().
* @ingroup Page
*/
ESTATUS BrowserControl_GetImageCount(HANDLE aBrowserHandle, BOOL bVisibleOnly, guint* imageCount);

/**
*  BrowserControl_SaveElement
* @brief Purpose : Saves the currently selected element in a local file.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] gchar* aPath : Path of the file in which the element must be saved.
* @return ESTATUS : ESUCCESS if the selected element could be sved, else
* EFAIL.Exact reason of the failure can be queried using
* BrowserControl_GetLastError().
* @ingroup Page
*/
ESTATUS BrowserControl_SaveElement(HANDLE aBrowserHandle, const gchar* aPath);
/**
*  BrowserControl_Reload
* @brief Purpose : Reload the current page from its url.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @return ESTATUS : ESUCCESS if the current resource could be reloaded,
* else EFAIL.Exact reason of the failure can be queried using
* BrowserControl_GetLastError().
* @ingroup Page
*/
/**
 *   bs0425.lee  modified for touch events 
 */

ESTATUS BrowserControl_SaveElementByPoint(HANDLE aBrowserHandle, const gchar* aPath, int x, int y);
/**
 *   bs0425.lee  modified for touch events 
 */

ESTATUS BrowserControl_Reload(HANDLE aBrowserHandle);
/**
*  BrowserControl_StopLoading
* @brief Purpose : Stop the loading the current page if the load is in progress.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @return ESTATUS : ESUCCESS if the loading could be stopped, else EFAIL.
* Exact reason of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Page
*/
ESTATUS BrowserControl_StopLoading(HANDLE aBrowserHandle);
/**
*  BrowserControl_Find
* @brief Purpose : Hightlights all the instances of the passed text in the
* current page and scrolls to the first instance.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] SFindInfo* aFindInfo : Text to be searched and other search parameters.
* @return ESTATUS : ESUCCESS if the passed text could be found in the current page,
* else EFAIL.Exact reason of the failure can be queried using
* BrowserControl_GetLastError().
* @ingroup Page
*/
ESTATUS BrowserControl_Find(HANDLE aBrowserHandle, const SFindInfo* aFindInfo);
/**
*  BrowserControl_FindNext
* @brief Purpose : Scrolls to the next instance of the given text in the current page.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] SFindInfo* aFindInfo : Text to be searched and other search parameters.
* @return ESTATUS : ESUCCESS if the next instance of the passed text could be found
* in the current page, else EFAIL.Exact reason of the failure can be queried using
* BrowserControl_GetLastError().
* @ingroup Page
*/
ESTATUS BrowserControl_FindNext(HANDLE aBrowserHandle, const SFindInfo* aFindInfo);
/**
*  BrowserControl_CanGoBack
* @brief Purpose : Check if the browser control can go to a previous page in history.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[out] bool* aRet : True if browser can go to a previous page. False otherwise.
* @return ESTATUS : ESUCCESS if the check could be done, else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Page
*/
ESTATUS BrowserControl_CanGoBack(HANDLE aBrowserHandle, gboolean* aRet);
/**
*  BrowserControl_CanGoForward
* @brief Purpose : Check if the browser control can go to a next page in history.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[out] bool* aRet : True if browser can go to a next page. False otherwise.
* @return ESTATUS : ESUCCESS if the check could be done, else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Page
*/
ESTATUS BrowserControl_CanGoForward(HANDLE aBrowserHandle, gboolean* aRet);
/**
*  BrowserControl_GoBack
* @brief Purpose : Load the last page in history before the current page.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @return ESTATUS : ESUCCESS if the browser control could navigate to the previous
* page in history, else EFAIL. Exact reason of the failure can be queried using
* BrowserControl_GetLastError().
* @ingroup Page
*/
ESTATUS BrowserControl_GoBack(HANDLE aBrowserHandle);
/**
*  BrowserControl_GoForward
* @brief Purpose : Load the next page in history after the current page.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @return ESTATUS : ESUCCESS if the browser control could navigate to the next page
* in history page, else EFAIL. Exact reason of the failure can be queried using
* BrowserControl_GetLastError().
* @ingroup Page
*/
ESTATUS BrowserControl_GoForward(HANDLE aBrowserHandle);
/**
*  BrowserControl_ZoomIn
* @brief Purpose : Zooms into the current visible area of the page.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] HANDLE guint aScalePercent : Amount (in percent) to be zoomed in.
* @return ESTATUS : ESUCCESS if current page view could be zoomed in, else EFAIL.
* Exact reason of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Page
*/
ESTATUS BrowserControl_ZoomIn(HANDLE aBrowserHandle, const guint aScalePercent);
/**
*  BrowserControl_ZoomOut
* @brief Purpose : Zooms out of the current visible area of the page.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] HANDLE guint aScalePercent : Amount (in percent) to be zoomed out.
* @return ESTATUS : ESUCCESS if current page view could be zoomed out, else EFAIL.
* Exact reason of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Page
*/
ESTATUS BrowserControl_ZoomOut(HANDLE aBrowserHandle, const guint aScalePercent);
/**
*  BrowserControl_GetZoom
* @brief Purpose : Retrieves the current Scaling percent.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] HANDLE guint aScalePercent : The current Scaling percent is retrurned in this.
* @return ESTATUS : ESUCCESS if current page view could be zoomed out, else EFAIL.
* Exact reason of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Page
*/
ESTATUS BrowserControl_GetZoom(HANDLE aBrowserHandle, guint* aScalePercent);

/**
*  BrowserControl_CreatePageThumbnail
* @brief Purpose : Creates the thumbnail of the current page.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] guint width : Width of thumbnail. 
* @param[in] guint height : Height of thumbnail.
* @param[in] const gchar *thumbPath : Path to store the thumbnail.
* @return ESTATUS : ESUCCESS if thumbnail is created and saved, else EFAIL.
* Exact reason of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Page
*/
ESTATUS BrowserControl_CreatePageThumbnail(HANDLE aBrowserHandle, guint width, guint height, const gchar *thumbPath);

/**
*  BrowserControl_SetTextSize
* @brief Purpose : Sets a new text size.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] guint size : New text size. 
* @return ESTATUS : ESUCCESS if text size could be set to new value, else EFAIL.
* Exact reason of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Page
*/
ESTATUS BrowserControl_SetTextSize(HANDLE aBrowserHandle, guint size);

#ifdef SFB_RSS_LINK //vamsi Jan 28 Rss Link Tag Support for SFB.
/**
*  BrowserControl_GetRssLinks
* @brief Purpose : Extracts the Rss Links in page
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] SRssLink** pRssLink : Rss Attribute Info is populated here.
* @param[in] int* rssCount : Number of Rss Links in Page is populated here.
* @param[in] gchar** url : Page URL is Populated here.
* @return ESTATUS : ESUCCESS
* @ingroup Page
*/
ESTATUS BrowserControl_GetRssLinks(HANDLE aBrowserHandle, SRssLink** pRssLink, int* rssCount, gchar** url);
#endif

//------------------------------------------------------------------------
// Error Handling Interface
//------------------------------------------------------------------------

/**
*  BrowserControl_SetErrorHandler
* @brief Purpose : Sets the error handler for various types of errors defined in EErrorType
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] EErrorType aType : Type of the error to be handled.
* @param[in] ErrorHandler aHandler : Pointer to the handler function.
* @return ESTATUS : ESUCCESS if the handler could be set, else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Error
*/
ESTATUS BrowserControl_SetErrorHandler(HANDLE aBrowserHandle, const EErrorType aType, const ErrorHandler aHandler);

/**
*  BrowserControl_GetLastError
* @brief Purpose : Gets the last error that happened in the browser control.
* @param[out] guint *aErrorCode : The error code is returned in this.
* @return void
* @ingroup Error
*/
void BrowserControl_GetLastError(ERRORCODE *aErrorCode);

//------------------------------------------------------------------------
// Observers API
//------------------------------------------------------------------------
/**
*  BrowserControl_SetLoadHandler
* @brief Purpose : Sets the handler function to be called to notify that an item is loaded.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] LoadHandler aHandler : Pointer to the load handler function.
* @return ESTATUS : ESUCCESS if the handler could be set, else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup Observer
*/
ESTATUS BrowserControl_SetLoadHandler(HANDLE aBrowserHandle, const LoadHandler aHandler);

//------------------------------------------------------------------------
// General API
//------------------------------------------------------------------------
/**
*  BrowserControl_GetVersion
* @brief Purpose : Gets the browser control version.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[out] gchar* aVersionInfo : The version string is returned in this.
* @param[in,out] gulong* aBufferSize : Size of the buffer. If the buffer is not enough,
* the required size is sent back in aSize.
* @return ESTATUS : ESUCCESS if the version info could be retrieved successfully, else EFAIL.
* Exact reason of the failure can be queried using BrowserControl_GetLastError().
* @ingroup General
*/
ESTATUS BrowserControl_GetVersion(HANDLE aBrowserHandle, gchar* aVersionInfo, gulong* aBufferSize);

/**
*  BrowserControl_GetCertificateInfo
* @brief Purpose : Gets the certificate info for the server from where page is being loaded.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[out] SCertInfo* aCertInfo : Pointer to the certificate info structure.
* @return ESTATUS : ESUCCESS if the certificate info could be retrieved, else EFAIL.
* Exact reason of the failure can be queried using BrowserControl_GetLastError().
* @ingroup General
*/
ESTATUS BrowserControl_GetCertificateInfo(HANDLE aBrowserHandle, SCertInfo* aCertInfo);

/**
*  BrowserControl_GetBrowserParameter
* @brief Purpose : Gets the browser parameters.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] EBrowserParameters aBrowserParameter : Parameter enum whose value is to be retrieved.
* @param[out] gchar* aParameterValue : Parameter value is returned in this buffer.
* @param[in,out] gulong* aBufferSize : Size of the buffer. If the buffer is not enough,
* the required size is sent back in aSize.
* @return ESTATUS : ESUCCESS if the parameter value could be retrieved successfully, else EFAIL.
* Exact reason of the failure can be queried using BrowserControl_GetLastError().
* @ingroup General
*/
ESTATUS BrowserControl_GetBrowserParameter(HANDLE aBrowserHandle, const EBrowserParameters 
aBrowserParameter,gchar* aParameterValue, gulong* aBufferSize);

/**
*  BrowserControl_SetBrowserParameter
* @brief Purpose : Sets the browser parameters.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] EBrowserParameters aBrowserParameter : Parameter enum whose value is to be set.
* @param[in] gchar* aParameterValue : Parameter value to be set.
* @return ESTATUS : ESUCCESS if the parameter could be set successfully, else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup General
*/
ESTATUS BrowserControl_SetBrowserParameter(HANDLE aBrowserHandle, const EBrowserParameters 
aBrowserParameter,const gchar* aParameterValue);


/**
*  BrowserControl_Paint
* @brief Purpose : Send a paint event to the webcore engine.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] void*  pUserData: user data with the message.
* @return ESTATUS : ESUCCESS if the parameter could be set successfully, else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup General
*/
ESTATUS BrowserControl_Paint(HANDLE aBrowserHandle, void* pUserData);

#ifdef _SFB_PAGE_PREVIEW	//vamsi Mar 03 - Page Overview
ESTATUS BrowserControl_SetPreview(HANDLE aBrowserHandle);

ESTATUS BrowserControl_IsPreview(HANDLE aBrowserHandle, BOOL* bPreview);
#endif

#ifdef SFB_VISUAL_SUPPORT
//tahseen
ESTATUS BrowserControl_VisualHistory(HANDLE aBrowserHandle,WmBitmap** pWmBitmap,void* pUserData);
#endif
/**
*  BrowserControl_HandleKeyEvent
* @brief Purpose : Handle the key pressed event.
* @param[in] HANDLE aBrowserHandle : Handle to the BrowserControl.
* @param[in] void*  pUserData: user data with the message.
* @return ESTATUS : ESUCCESS if the parameter could be set successfully, else EFAIL.Exact reason
* of the failure can be queried using BrowserControl_GetLastError().
* @ingroup General
*/

ESTATUS BrowserControl_HandleKeyEvent(HANDLE aBrowserHandle, int keyCode, BOOL isKeyUp);

/**
 *   bs0425.lee  modified for touch events 
 */
ESTATUS BrowserControl_HandleMouseEvent(HANDLE aBrowserHandle, int x, int y, int MouseEventType);
ESTATUS BrowserControl_PaneScrollBy(HANDLE aBrowserHandle, int offsetX, int offsetY);
gboolean BrowserControl_IsFocusedByPoint( HANDLE aBrowserHandle,int x, int y);
gboolean BrowserControl_IsLink( HANDLE aBrowserHandle,int x, int y);
gboolean BrowserControl_IsImageElement( HANDLE aBrowserHandle,int x, int y);
ESTATUS BrowserControl_GetSelectedElementInfoByPoint(HANDLE aBrowserHandle, EPageItemType aType, ElementInfoBase* aInfo, int x, int y);


/**
 *   bs0425.lee  modified for touch events 
 */


#if defined SHP_SUPPORT_WML
void BrowserControl_FetchRequest (HANDLE aBrowserHandle) ;
int BrowserControl_WmlItemCount(HANDLE aBrowserHandle) ;
int BrowserControl_WmlDoElement( HANDLE aBrowserHandle, int ) ;
#endif //SHP_SUPPORT_WML

#ifdef __SHP__
void BrowserControl_SetProgressHandler(HANDLE aBrowserHandle, ProgressHandler aProgressHandler);
void BrowserControl_SetTitleChangedHandler(HANDLE aBrowserHandle, TitleChangedHandler aTitleChangedHandler);
void BrowserControl_SetLinkHoveringHandler(HANDLE aBrowserHandle, LinkHoveringHandler aLinkHoveringHandler);
void BrowserControl_SetStatusbarTextChangedHandler(HANDLE aBrowserHandle, StatusbarTextChangedHandler aStatusbarTextChangedHandler);
void BrowserControl_SetIconLoadedHandler(HANDLE aBrowserHandle,IconLoadedHandler aIconLoadedHandler);
void BrowserControl_SetSelectionChangedHandler(HANDLE aBrowserHandle, SelectionChangedHandler aSelectionChangedHandler);
void BrowserControl_SetUrlSchemeHandler(HANDLE aBrowserHandle, UrlSchemeHandler aUrlSchemeHandler);
void BrowserControl_SetUIStateHandler(HANDLE aBrowserHandle, UIStateHandler aUIStateHandler);
#endif

//vamsi relayout
void BrowserControl_ReLayout(HANDLE aBrowserHandle);

#ifdef __cplusplus
} //extern "C"
#endif

#endif //__BROWSERCONTROL_INTERFACE_H__


