/*
* ==============================================================================
*  Name        : Allocator.cpp
*  Part of     : WEBKIT / MemMan
*  Interface   :
*  Description :
*  Version     : %Version%
*
* ==============================================================================
*/

// INCLUDES

#include "Allocator.h"
#include "MemCore.h"

#ifdef USE_FAST_MALLOC
#include "fast_malloc.h"
#endif

#include "SysAllocator.h"

#define MAX_UINT           (~(unsigned)0)

// CONSTANTS

// CLASS DECLARATIONS
extern size_t dlIsMemAvailable(guint32 iReqSize, guint32 *outMaxChunk = NULL);

// Global data
static SysAllocator     gSysAllocator;

//-----------------------------------------------------------------------------
// CAllocator::AddCollector
//-----------------------------------------------------------------------------
void CAllocator::AddCollector(IMemoryCollector* aCollector)
{
	gSysAllocator.addCollector(aCollector);
}

//-----------------------------------------------------------------------------
// CAllocator::SetNotifier
//-----------------------------------------------------------------------------
void CAllocator::SetNotifier(ICMMNotifier* aNotifier)
{
	gSysAllocator.setNotifier(aNotifier);
}

//-----------------------------------------------------------------------------
// CAllocator::Create()
//-----------------------------------------------------------------------------
gbool CAllocator::Create()
{
	return gtrue;
}

//-----------------------------------------------------------------------------
// CAllocator::IsOOMSent()
//-----------------------------------------------------------------------------
gbool CAllocator::IsOOMSent()
{
    return gSysAllocator.isOOMSent();
}

//-----------------------------------------------------------------------------
// CAllocator::SetInOOMState()
//-----------------------------------------------------------------------------
void CAllocator::SetInOOMState()
{
    gSysAllocator.setAllocatorInOOMState();
}

//-----------------------------------------------------------------------------
// CAllocator::ResetOOMState
//-----------------------------------------------------------------------------
void CAllocator::ResetOOMState()
{ 
    SetStatus(CMM_CHECKRESULT_NO_CMM);
    gSysAllocator.resetOOMState();
}

//-----------------------------------------------------------------------------
// CAllocator::PreCheck
//-----------------------------------------------------------------------------
gbool CAllocator::PreCheck(guint32 iRequiredSize, gbool bContiguous)
{
    unsigned availMem = gSysAllocator.availableMemory( iRequiredSize, bContiguous );

    if( availMem < iRequiredSize )
    {
        if( gtrue == bContiguous )
        {
            void* memChunk = dlmalloc( iRequiredSize, bContiguous );
            if( NULL != memChunk )
            {
                dlfree( memChunk );
                availMem = iRequiredSize;
            }
        }
        else
        {
            // Try to collect memory from dead cached resources
            availMem += gSysAllocator.collectDeadResources();

            if( availMem < iRequiredSize )
            {
                // Check if the memory chunk is available with dlmalloc
                availMem += dlIsMemAvailable(iRequiredSize - availMem);

                if( availMem < iRequiredSize )
                {
                    // Finally collect the images from current page
                    availMem += gSysAllocator.collectLiveImageResources(iRequiredSize - availMem);
                    if( availMem < iRequiredSize )
                    {
                        // Send an OOM event if the memory is not available
                        gSysAllocator.setAllocatorInOOMState();
                    }
                    else
                    {
                        gSysAllocator.setStatus(CMM_CHECKRESULT_COLLECTING);
                    }
                }
            }
        }
    }

    return (availMem >= iRequiredSize);
}

//-----------------------------------------------------------------------------
// CAllocator::PreCheck
//-----------------------------------------------------------------------------
gbool CAllocator::PreCheckWithThreshold(guint32 iRequiredSize)
{
    const unsigned reqMem = iRequiredSize + MM_OOM_THRESHOLD;

    // Check if reqMem size can be allocated without going OOM, the memory 
    // segments need not be contiguous
    const unsigned availMem = gSysAllocator.availableMemory( reqMem, false );

    return (availMem >= reqMem);
}

//-----------------------------------------------------------------------------
// CAllocator::SetStatus
//-----------------------------------------------------------------------------
void CAllocator::SetStatus(ECMMCheckResultType aType)
{
    gSysAllocator.setStatus(aType);
}

//-----------------------------------------------------------------------------
// CAllocator::Status
//-----------------------------------------------------------------------------
guint32 CAllocator::Status()
{
	return gSysAllocator.status();
}

//-----------------------------------------------------------------------------
// CAllocator::DoAlloc
//-----------------------------------------------------------------------------
void* CAllocator::DoAlloc(guint aSize)
{
#ifndef USE_FAST_MALLOC
	return CMM_MALLOC(aSize);
#else
	return dlmalloc(aSize);
#endif
}

//-----------------------------------------------------------------------------
// CAllocator::AllocFromHeap
//-----------------------------------------------------------------------------

void* CAllocator::AllocFromHeap(guint aSize)
{
	if(aSize <= 0) aSize = 1;
	void *p = DoAlloc(aSize);
	return p;
}

#ifdef SFB_GLOBAL_STATIC_HANDLING
//-----------------------------------------------------------------------------
// CAllocator::InitState
//-----------------------------------------------------------------------------
void CAllocator::InitState()
{
    gSysAllocator.init();
}

//-----------------------------------------------------------------------------
// CAllocator::DeInitState
//-----------------------------------------------------------------------------
void CAllocator::DeInitState()
{
    gSysAllocator.deinit();
}

#endif //#ifdef SFB_GLOBAL_STATIC_HANDLING

//-----------------------------------------------------------------------------
// CPlainAllocator::Calloc
//-----------------------------------------------------------------------------
void* CPlainAllocator::Calloc(guint32 aElements, guint32 aSize)
{
#ifdef USE_FAST_MALLOC
	return dlcalloc(aElements, aSize);
#else
	if(aSize <= 0) aSize = 1;
	return CMM_CALLOC(aElements, aSize);
#endif
}

//-----------------------------------------------------------------------------
// CPlainAllocator::ReAllocate
//-----------------------------------------------------------------------------
void* CPlainAllocator::ReAllocate(void* aPtr, guint aSize)
{
#ifdef USE_FAST_MALLOC
	return dlrealloc(aPtr, aSize);
#else
	if(aSize <= 0) aSize = 1;
	void* p = CMM_REALLOC(aPtr, aSize);
	return p;
#endif
}

//-----------------------------------------------------------------------------
// CPlainAllocator::Free
//-----------------------------------------------------------------------------
void CPlainAllocator::Free(void* aPtr)
{
#ifdef USE_FAST_MALLOC
	dlfree(aPtr);
#else
	CMM_FREE(aPtr);
#endif
}

//-----------------------------------------------------------------------------
// CPlainAllocator::AvailableMemory
//-----------------------------------------------------------------------------
#ifndef NDEBUG
unsigned CPlainAllocator::AvailableMemory()
{
    return gSysAllocator.availableMemory() + dlIsMemAvailable(MAX_UINT);
}
#endif // NDEBUG

void* PoolAllocate(unsigned size)
{
    return gSysAllocator.allocate(size);
}

void PoolDeallocate(void* aPtr, unsigned aSize)
{
    gSysAllocator.deallocate(aPtr, aSize);
}

//EOF
