/**
 * This file is part of the DOM implementation for KDE.
 *
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 *           (C) 2000 Simon Hausmann (hausmann@kde.org)
 *           (C) 2001 Dirk Mueller (mueller@kde.org)
 * Copyright (C) 2004, 2006 Apple Computer, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */
#include "config.h"
#include "HTMLIFrameElement.h"

#include "CSSPropertyNames.h"
#include "Frame.h"
#include "HTMLDocument.h"
#include "HTMLNames.h"
#include "RenderPartObject.h"

#if PLATFORM(SHP) && defined(SFB_SSR_RENDERING)
// CELUNITE_SSR_Change_Begin
#include "FrameView.h"
// CELUNITE_SSR_Change_End
#endif
namespace WebCore {

using namespace HTMLNames;

HTMLIFrameElement::HTMLIFrameElement(Document* doc)
    : HTMLFrameElementBase(iframeTag, doc)
#if PLATFORM(SHP) && defined(SFB_SSR_RENDERING)
// CELUNITE_SSR_Change_Begin
    , m_iOrgFrameWidth(-1)
    , m_iOrgFrameHeight(-1)
// CELUNITE_SSR_Change_End
#endif
{
}

bool HTMLIFrameElement::mapToEntry(const QualifiedName& attrName, MappedAttributeEntry& result) const
{
    if (attrName == widthAttr || attrName == heightAttr) {
        result = eUniversal;
        return false;
    }
    
    if (attrName == alignAttr) {
        result = eReplaced; // Share with <img> since the alignment behavior is the same.
        return false;
    }
    
    if (attrName == frameborderAttr) {
        result = eReplaced;
        return false;
    }

    return HTMLFrameElementBase::mapToEntry(attrName, result);
}

void HTMLIFrameElement::parseMappedAttribute(MappedAttribute *attr)
{
    if (attr->name() == widthAttr)
        addCSSLength(attr, CSS_PROP_WIDTH, attr->value());
    else if (attr->name() == heightAttr)
        addCSSLength(attr, CSS_PROP_HEIGHT, attr->value());
    else if (attr->name() == alignAttr)
        addHTMLAlignment(attr);
    else if (attr->name() == nameAttr) {
        String newNameAttr = attr->value();
        if (inDocument() && document()->isHTMLDocument()) {
            HTMLDocument* doc = static_cast<HTMLDocument* >(document());
            doc->removeDocExtraNamedItem(oldNameAttr);
            doc->addDocExtraNamedItem(newNameAttr);
        }
        oldNameAttr = newNameAttr;
    } else if (attr->name() == frameborderAttr) {
        // Frame border doesn't really match the HTML4 spec definition for iframes.  It simply adds
        // a presentational hint that the border should be off if set to zero.
        if (!attr->isNull() && !attr->value().toInt())
            // Add a rule that nulls out our border width.
            addCSSLength(attr, CSS_PROP_BORDER_WIDTH, "0");
    } 
#if PLATFORM(SHP)
	else if (attr->name() == scrollingAttr) 
	{
            m_scrolling = ScrollbarAuto;
	}
#endif //PLATFORM(SHP)
	else
        HTMLFrameElementBase::parseMappedAttribute(attr);
}

bool HTMLIFrameElement::rendererIsNeeded(RenderStyle* style)
{
    return isURLAllowed(m_URL) && style->display() != NONE;
}

RenderObject* HTMLIFrameElement::createRenderer(RenderArena* arena, RenderStyle*)
{
    return new (arena) RenderPartObject(this);
}

void HTMLIFrameElement::insertedIntoDocument()
{
    if (document()->isHTMLDocument()) {
        HTMLDocument* doc = static_cast<HTMLDocument*>(document());
        doc->addDocExtraNamedItem(oldNameAttr);
    }

    HTMLFrameElementBase::insertedIntoDocument();
}

void HTMLIFrameElement::removedFromDocument()
{
    if (document()->isHTMLDocument()) {
        HTMLDocument* doc = static_cast<HTMLDocument*>(document());
        doc->removeDocExtraNamedItem(oldNameAttr);
    }

    HTMLFrameElementBase::removedFromDocument();
}

void HTMLIFrameElement::attach()
{
    HTMLFrameElementBase::attach();

    if (RenderPartObject* renderPartObject = static_cast<RenderPartObject*>(renderer()))
        renderPartObject->updateWidget(false);
}

bool HTMLIFrameElement::isURLAttribute(Attribute* attr) const
{
    return attr->name() == srcAttr;
}

String HTMLIFrameElement::align() const
{
    return getAttribute(alignAttr);
}

void HTMLIFrameElement::setAlign(const String &value)
{
    setAttribute(alignAttr, value);
}

String HTMLIFrameElement::height() const
{
    return getAttribute(heightAttr);
}

void HTMLIFrameElement::setHeight(const String &value)
{
    setAttribute(heightAttr, value);
}

String HTMLIFrameElement::width() const
{
    return getAttribute(widthAttr);
}

void HTMLIFrameElement::setWidth(const String &value)
{
    setAttribute(widthAttr, value);
}
#if PLATFORM(SHP) && defined(SFB_SSR_RENDERING)
// CELUNITE_SSR_Change_Begin
void HTMLIFrameElement::handleContentChange()
{
    if( 0 >= contentFrame()->view()->width() || 0 >= contentFrame()->view()->height() )
        return;

    if( SSR_MODE_OFF == document()->topDocument()->getSSRMode())
    {

        if ( -1 == m_iOrgFrameWidth && -1 == m_iOrgFrameHeight)
        {

            m_iOrgFrameWidth = contentFrame()->view()->width();
#ifdef __USE_DEAULT_FRAME_HEIGHT__
            m_iOrgFrameHeight = contentFrame()->view()->height();
#else
			// For SFB-Bug#110 - The fix is to set the frame height to be minimum one line spacing tall.
			// The basic idea is to set frame height to minimum one line spacing height 
			// for the default font being used irrespective of the content inside.
			// NOTE: This fix may use more vertical spacing for the IFRAME then set by the 
			// content developer.
			int viewHeight = contentFrame()->view()->height();
			int lineSpacing = document()->renderer()->style()->font().lineSpacing();
			m_iOrgFrameHeight = (viewHeight < lineSpacing)? lineSpacing : viewHeight;
#endif
            return;
        }

        if(m_iOrgFrameWidth != contentFrame()->view()->width() ||
            m_iOrgFrameHeight != contentFrame()->view()->height() )
        {
            enforceLay0utChange(m_iOrgFrameWidth, m_iOrgFrameHeight);
        }

        return;
    }

    int contentHeight = contentFrame()->renderer()->contentHeight();
    int contentWidth =  contentFrame()->renderer()->contentWidth();
    IntRect viewGeometry = document()->topDocument()->view()->frameGeometry();
    int frameWidth = viewGeometry.width();

    int iHeight = contentFrame()->renderer()->lineHeight(false) * (  ( (int) contentWidth / frameWidth) + 2 );

    if(frameWidth >= contentWidth && contentFrame()->view()->height() == iHeight)
        return;

    enforceLay0utChange(frameWidth, iHeight);
}

void HTMLIFrameElement::enforceLay0utChange(int width, int height)
{
    Length currWidth = renderer()->style()->width();
    Length currHeight = renderer()->style()->height();

    renderer()->style()->setWidth( Length(width, currWidth.type()) );
    renderer()->style()->setHeight( Length(height, currHeight.type()) );
    renderer()->setWidth(width);
    renderer()->setHeight(height);
    renderer()->setNeedsLayout(true);
    document()->topDocument()->frame()->view()->layout();
}
// CELUNITE_SSR_Change_End
#endif

}
