/**
 * This file is part of the DOM implementation for KDE.
 *
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 *           (C) 2001 Dirk Mueller (mueller@kde.org)
 * Copyright (C) 2003 Apple Computer, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */
#include "config.h"
#include "HTMLMetaElement.h"

#include "Document.h"
#include "HTMLNames.h"
#if PLATFORM(SHP)
	#include "Frame.h"
	#include "FrameLoader.h"
	#include "DocLoader.h"
	#include "CachedCSSStyleSheet.h"
#endif
namespace WebCore {

using namespace HTMLNames;

HTMLMetaElement::HTMLMetaElement(Document* doc)
    : HTMLElement(metaTag, doc)
#if PLATFORM(SHP)
	, m_loading(false)
	, m_cachedSheet(0)
#endif
{
}

HTMLMetaElement::~HTMLMetaElement()
{
#if PLATFORM(SHP)
	if (m_cachedSheet) {
        m_cachedSheet->deref(this);
        if (m_loading)
            document()->removePendingSheet();
    }
#endif
}

void HTMLMetaElement::parseMappedAttribute(MappedAttribute* attr)
{
    if (attr->name() == http_equivAttr) {
        m_equiv = attr->value();
        process();
    } else if (attr->name() == contentAttr) {
        m_content = attr->value();
        process();
    } else if (attr->name() == nameAttr) {
        String name = attr->value();
		if( equalIgnoringCase(name, "Cache-Control") || equalIgnoringCase(name, "Expires") || equalIgnoringCase(name, "Pragma") )
		{
			m_equiv = name;
			process();
		}
    } else
        HTMLElement::parseMappedAttribute(attr);
}

void HTMLMetaElement::insertedIntoDocument()
{
    HTMLElement::insertedIntoDocument();
    process();
}

void HTMLMetaElement::process()
{
    // Get the document to process the tag, but only if we're actually part of DOM tree (changing a meta tag while
    // it's not in the tree shouldn't have any effect on the document)
    if (inDocument() && !m_equiv.isNull() && !m_content.isNull())
	{
#if PLATFORM(SHP)
		if (equalIgnoringCase(m_equiv, "Link"))
		{
			String url;
			parseContentAttributeValue(m_content, url);
			String chset = getAttribute(charsetAttr);
            if (chset.isEmpty() && document()->frame())
                chset = document()->frame()->loader()->encoding();

            m_cachedSheet = document()->docLoader()->requestCSSStyleSheet(url, chset, false, this);

			 if (m_cachedSheet)
                m_cachedSheet->ref(this);
			 document()->addPendingSheet();
		}
		else
#endif
			document()->processHttpEquiv(m_equiv, m_content);
	}
}

#if PLATFORM(SHP)
void HTMLMetaElement::parseContentAttributeValue(const String &content, String &url)
{
	int i = 0, urlIndex = 0;
	int length = content.length();
	url ="";
	while(i < length)
	{
		if((content[i] == '<') || (content[i] == '>'))
		{
			i++;
			continue;
		}
		else if(content[i] == ';')
		{
			break;
		}
		url.append(content[i]);
		urlIndex++;
		i++;
	}
	int pos = content.find("title",0,false);
	pos = pos+5;
	while(content[pos] != '=' && pos < length)
		pos++;
	if(pos < length)
		pos++;
	while(content[pos] == ' ' && pos < length)
		pos++;
	while(content[pos] != '"' && pos < length)
		pos++;
	pos++;
	String title="";
	while(content[pos] != '"' && pos < length)
	{
		title.append(content[pos]);
		pos++;
	}
	setTitle(title);

	//url = tempUrl;
	//url = "http://wap2.samsungmobile.com/test/tdev/wap/kishore/color.css";
}

void HTMLMetaElement::setCSSStyleSheet(const String& url, const String& charset, const CachedCSSStyleSheet* sheet)
{
    bool strict = !document()->inCompatMode();
    m_sheet = new CSSStyleSheet(this, url, charset);
    m_sheet->parseString(sheet->sheetText(strict), strict);
    m_sheet->setTitle(title());

    m_loading = false;
    m_sheet->checkLoaded();
}

bool HTMLMetaElement::sheetLoaded()
{
    if (!isLoading()) {
        document()->removePendingSheet();
        return true;
    }
    return false;
}

bool HTMLMetaElement::isLoading() const
{
    if (m_loading)
        return true;
    if (!m_sheet)
        return false;
    return static_cast<CSSStyleSheet *>(m_sheet.get())->isLoading();
}

StyleSheet* HTMLMetaElement::sheet() const
{
    return m_sheet.get();
}
#endif

String HTMLMetaElement::content() const
{
    return getAttribute(contentAttr);
}

void HTMLMetaElement::setContent(const String& value)
{
    setAttribute(contentAttr, value);
}

String HTMLMetaElement::httpEquiv() const
{
    return getAttribute(http_equivAttr);
}

void HTMLMetaElement::setHttpEquiv(const String& value)
{
    setAttribute(http_equivAttr, value);
}

String HTMLMetaElement::name() const
{
    return getAttribute(nameAttr);
}

void HTMLMetaElement::setName(const String& value)
{
    setAttribute(nameAttr, value);
}

String HTMLMetaElement::scheme() const
{
    return getAttribute(schemeAttr);
}

void HTMLMetaElement::setScheme(const String &value)
{
    setAttribute(schemeAttr, value);
}

}
