/**
 * This file is part of the DOM implementation for KDE.
 *
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 *           (C) 2001 Dirk Mueller (mueller@kde.org)
 * Copyright (C) 2003, 2004, 2005, 2006, 2007 Apple Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */
#include "config.h"
#include "HTMLScriptElement.h"

#include "CachedScript.h"
#include "DocLoader.h"
#include "Document.h"
#include "EventNames.h"
#include "Frame.h"
#include "FrameLoader.h"
#include "HTMLNames.h"
#include "kjs_proxy.h"
#include "MIMETypeRegistry.h"
#include "Text.h"

#include "chrome.h"
#include "page.h"
#include "cstring.h"

namespace WebCore {

using namespace HTMLNames;
using namespace EventNames;

HTMLScriptElement::HTMLScriptElement(Document *doc)
    : HTMLElement(scriptTag, doc)
    , m_cachedScript(0)
    , m_createdByParser(false)
    , m_evaluated(false)
#if PLATFORM(SHP)
    , m_loadComplete(false)
#endif
{
}

HTMLScriptElement::~HTMLScriptElement()
{
    if (m_cachedScript)
        m_cachedScript->deref(this);
}

bool HTMLScriptElement::isURLAttribute(Attribute *attr) const
{
    return attr->name() == srcAttr;
}

void HTMLScriptElement::childrenChanged(bool changedByParser)
{
    // If a node is inserted as a child of the script element
    // and the script element has been inserted in the document
    // we evaluate the script.
    if (!m_createdByParser && inDocument() && firstChild())
        evaluateScript(document()->url(), text());
    HTMLElement::childrenChanged(changedByParser);
}

void HTMLScriptElement::parseMappedAttribute(MappedAttribute *attr)
{
    const QualifiedName& attrName = attr->name();
    if (attrName == srcAttr) {

#if USE(LOW_BANDWIDTH_DISPLAY)
#if PLATFORM(SHP)
        if ((m_evaluated || m_cachedScript || m_createdByParser || !inDocument()) && false == document()->inLowBandwidthDisplay())
#else
        if (m_evaluated || m_cachedScript || m_createdByParser || !inDocument())
#endif
#else
        if (m_evaluated || m_cachedScript || m_createdByParser || !inDocument())
#endif

            return;

        // FIXME: Evaluate scripts in viewless documents.
        // See http://bugs.webkit.org/show_bug.cgi?id=5727
        if (!document()->frame())
            return;
    
        const AtomicString& url = attr->value();
        if (!url.isEmpty()) {
#if PLATFORM(SHP)
			AtomicString charset = getAttribute(charsetAttr) ;
			if(charset.isNull() || charset.isEmpty())
			{
				charset = document()->characterSet() ;
			}
            m_cachedScript = document()->docLoader()->requestScript(url, charset, this);
			//m_cachedScript = document()->docLoader()->requestScript(url, getAttribute(charsetAttr), this);
#else
            m_cachedScript = document()->docLoader()->requestScript(url, getAttribute(charsetAttr));
#endif
            if (m_cachedScript)
                m_cachedScript->ref(this);
            else
                dispatchHTMLEvent(errorEvent, true, false);
        }
    } else if (attrName == onloadAttr)
        setHTMLEventListener(loadEvent, attr);
    else
        HTMLElement::parseMappedAttribute(attr);
}

void HTMLScriptElement::finishParsingChildren()
{
    // The parser just reached </script>. If we have no src and no text,
    // allow dynamic loading later.
    if (getAttribute(srcAttr).isEmpty() && text().isEmpty())
        setCreatedByParser(false);
    HTMLElement::finishParsingChildren();
}

void HTMLScriptElement::insertedIntoDocument()
{
    HTMLElement::insertedIntoDocument();

#if !PLATFORM(SHP)
    // This assert is not valid for SHP mode since we might have 
    // already created the cached script under low bandwidth mode
    ASSERT(!m_cachedScript);
#endif // !PLATFORM(SHP)

#if PLATFORM(SHP)
    document()->setLastScriptTypeSupported(shouldExecuteAsJavaScript());

#if USE(LOW_BANDWIDTH_DISPLAY)
    if(true == document()->inLowBandwidthDisplay() && m_cachedScript)
        return;
#endif // USE(LOW_BANDWIDTH_DISPLAY)

#endif // PLATFORM(SHP)

    if (m_createdByParser)
        return;
    
    // FIXME: Eventually we'd like to evaluate scripts which are inserted into a 
    // viewless document but this'll do for now.
    // See http://bugs.webkit.org/show_bug.cgi?id=5727
    if (!document()->frame())
        return;
    
    const AtomicString& url = getAttribute(srcAttr);
    if (!url.isEmpty()) {
        String scriptSrcCharset = getAttribute(charsetAttr).domString().stripWhiteSpace();
        if (scriptSrcCharset.isEmpty()) {
            if (Frame* frame = document()->frame())
                scriptSrcCharset = frame->loader()->encoding();
        }
#if PLATFORM(SHP)
        m_cachedScript = document()->docLoader()->requestScript(url, scriptSrcCharset, this);
#else
        m_cachedScript = document()->docLoader()->requestScript(url, scriptSrcCharset);
#endif
        if (m_cachedScript)
            m_cachedScript->ref(this);
        else
            dispatchHTMLEvent(errorEvent, true, false);
        return;
    }

    // If there's an empty script node, we shouldn't evaluate the script
    // because if a script is inserted afterwards (by setting text or innerText)
    // it should be evaluated, and evaluateScript only evaluates a script once.
    String scriptString = text();    
    if (!scriptString.isEmpty())
        evaluateScript(document()->url(), scriptString);
}

void HTMLScriptElement::removedFromDocument()
{
    HTMLElement::removedFromDocument();

    if (m_cachedScript) {
        m_cachedScript->deref(this);
        m_cachedScript = 0;
    }
}

#if PLATFORM(SHP)
bool HTMLScriptElement::evaluateCachedScript()
{
    if (m_cachedScript) 
    {
        RefPtr<HTMLScriptElement> protect(this);
        
        if (!m_cachedScript->errorOccurred())
        {
            evaluateScript(m_cachedScript->url(), m_cachedScript->script());
        }


        if(m_cachedScript)
        {
            m_cachedScript->deref(this);
            m_cachedScript = 0;
        }
    }

    return m_loadComplete;
}
#endif


void HTMLScriptElement::notifyFinished(CachedResource* o)
{
    CachedScript *cs = static_cast<CachedScript *>(o);

    ASSERT(cs == m_cachedScript);

#if PLATFORM(SHP)
    m_loadComplete = true;
#endif

#if PLATFORM(SHP)
#if USE(LOW_BANDWIDTH_DISPLAY)
    if(document() && document()->inLowBandwidthDisplay())
    {
        if (cs->errorOccurred())
            dispatchHTMLEvent(errorEvent, true, false);
        else
            dispatchHTMLEvent(loadEvent, false, false);

        return;
    }
#endif
#endif


    // Evaluating the script could lead to a garbage collection which
    // can delete the script element so we need to protect it.
    RefPtr<HTMLScriptElement> protect(this);
    
    if (cs->errorOccurred())
        dispatchHTMLEvent(errorEvent, true, false);
    else {
        evaluateScript(cs->url(), cs->script());
        dispatchHTMLEvent(loadEvent, false, false);
    }

    // script evaluation may have dereffed it already
    if (m_cachedScript) {
        m_cachedScript->deref(this);
        m_cachedScript = 0;
    }
}

#ifdef SFB_GLOBAL_STATIC_HANDLING
    static AtomicString* validLanguages;
#define VALID_LANG_STR (12)

void initHTMLScriptElement()
{
    validLanguages = new AtomicString[VALID_LANG_STR];

    validLanguages[0] = "javascript";
    validLanguages[1] = "javascript1.0";
    validLanguages[2] = "javascript1.1";
    validLanguages[3] = "javascript1.2";
    validLanguages[4] = "javascript1.3";
    validLanguages[5] = "javascript1.4";
    validLanguages[6] = "javascript1.5";
    validLanguages[7] = "javascript1.6";
    validLanguages[8] = "javascript1.7";
    validLanguages[9] = "livescript";
    validLanguages[10] = "ecmascript";
    validLanguages[11] = "jscript";
}

#endif


bool HTMLScriptElement::shouldExecuteAsJavaScript()
{
    /*
         Mozilla 1.8 accepts javascript1.0 - javascript1.7, but WinIE 7 accepts only javascript1.1 - javascript1.3.
         Mozilla 1.8 and WinIE 7 both accept javascript and livescript.
         WinIE 7 accepts ecmascript and jscript, but Mozilla 1.8 doesn't.
         Neither Mozilla 1.8 nor WinIE 7 accept leading or trailing whitespace.
         We want to accept all the values that either of these browsers accept, but not other values.
     */
#ifndef SFB_GLOBAL_STATIC_HANDLING
    static const AtomicString validLanguages[] = {
        "javascript",
        "javascript1.0",
        "javascript1.1",
        "javascript1.2",
        "javascript1.3",
        "javascript1.4",
        "javascript1.5",
        "javascript1.6",
        "javascript1.7",
        "livescript",
        "ecmascript",
        "jscript"
    };
#endif


#ifndef SFB_GLOBAL_STATIC_HANDLING
    static const unsigned validLanguagesCount = sizeof(validLanguages) / sizeof(validLanguages[0]); 
#else
    static const unsigned validLanguagesCount = VALID_LANG_STR;     
#endif

    const AtomicString& type = getAttribute(typeAttr);
    if (!type.isEmpty()) {
        String lowerType = type.domString().stripWhiteSpace().lower();
        if (MIMETypeRegistry::isSupportedJavaScriptMIMEType(lowerType))
            return true;

        return false;
    }

    const AtomicString& language = getAttribute(languageAttr);
    if (!language.isEmpty()) {
        String lowerLanguage = language.domString().lower();
        for (unsigned i = 0; i < validLanguagesCount; ++i)
            if (lowerLanguage == validLanguages[i])
                return true;

        return false;
    }

    // No type or language is specified, so we assume the script to be JavaScript
    return true;
}

void HTMLScriptElement::evaluateScript(const String& url, const String& script)
{
    if (m_evaluated)
        return;
    
    if (!shouldExecuteAsJavaScript())
        return;
    
    Frame* frame = document()->frame();
    if (frame) {
        KJSProxy* proxy = frame->scriptProxy();
        if (proxy) {
            m_evaluated = true;
            proxy->evaluate(url, 0, script);
            Document::updateDocumentsRendering();
        }
    }
}

String HTMLScriptElement::text() const
{
    String val = "";
    
    for (Node *n = firstChild(); n; n = n->nextSibling()) {
        if (n->isTextNode())
            val += static_cast<Text *>(n)->data();
    }
    
    return val;
}

void HTMLScriptElement::setText(const String &value)
{
    ExceptionCode ec = 0;
    int numChildren = childNodeCount();
    
    if (numChildren == 1 && firstChild()->isTextNode()) {
        static_cast<Text *>(firstChild())->setData(value, ec);
        return;
    }
    
    if (numChildren > 0) {
        removeChildren();
    }
    
    appendChild(document()->createTextNode(value.impl()), ec);
}

String HTMLScriptElement::htmlFor() const
{
    // DOM Level 1 says: reserved for future use.
    return String();
}

void HTMLScriptElement::setHtmlFor(const String &/*value*/)
{
    // DOM Level 1 says: reserved for future use.
}

String HTMLScriptElement::event() const
{
    // DOM Level 1 says: reserved for future use.
    return String();
}

void HTMLScriptElement::setEvent(const String &/*value*/)
{
    // DOM Level 1 says: reserved for future use.
}

String HTMLScriptElement::charset() const
{
    return getAttribute(charsetAttr);
}

void HTMLScriptElement::setCharset(const String &value)
{
    setAttribute(charsetAttr, value);
}

bool HTMLScriptElement::defer() const
{
    return !getAttribute(deferAttr).isNull();
}

void HTMLScriptElement::setDefer(bool defer)
{
    setAttribute(deferAttr, defer ? "" : 0);
}

String HTMLScriptElement::src() const
{
    return document()->completeURL(getAttribute(srcAttr));
}

void HTMLScriptElement::setSrc(const String &value)
{
    setAttribute(srcAttr, value);
}

String HTMLScriptElement::type() const
{
    return getAttribute(typeAttr);
}

void HTMLScriptElement::setType(const String &value)
{
    setAttribute(typeAttr, value);
}

bool HTMLScriptElement::toString (HTMLSrcString &htmlSrc) const
{
	String tempString ;
	htmlSrc.append ("<") ;
	htmlSrc.append (Element::nodeName()) ;

	NamedAttrMap *attrMap = Element::attributes(true);
	if (attrMap) {
		unsigned numAttrs = attrMap->length();
		for (unsigned i = 0; i < numAttrs; i++) {
			htmlSrc.append (" ") ;

			Attribute *attribute = attrMap->attributeItem(i);
			//result += ;
			htmlSrc.append (attribute->name().toString()) ;
			if (!attribute->value().isNull()) {
				htmlSrc.append ("=\"") ;
				String hrefSrc = attribute->name().toString();
				if(hrefSrc == "src")	{
					String charset("");
					DocLoader* loader = document()->docLoader();
					CachedScript* kScript = 0;
					String url = src() ;
					if (loader->checkCacheResource (url))
						kScript = loader->requestScript(url, charset);

					if (kScript)	{
						String scriptValue = kScript->script() ;
						PlatformFileHandle filehandle = 0 ;
						String path = document()->getFilePath() ;
						String fileName = document()->validateFilename(url,path);
						bool success = true ;
						success = document()->page()->chrome()->savePageFileContext (filehandle, fileName,
							path, scriptValue.utf8().data(), scriptValue.utf8().length(), true) ;
						if (!success)	{
							path.append (fileName) ;
							deleteFile (path) ;
							fileName = "" ;
							path = "" ;
							}
						//path.append (fileName) ;
						htmlSrc.append (fileName) ;
					}
				}
				else	{
					tempString = attribute->value() ;
					htmlSrc.append (tempString) ;
				}
				htmlSrc.append ("\"") ;
			}
		}
		htmlSrc.append (">") ;
	}
	for (Node *child = firstChild(); child != NULL; child = child->nextSibling()) 
	{
		child->toString(htmlSrc);			
	}
	htmlSrc.append ("</") ;
	htmlSrc.append (Element::nodeName()) ;
	htmlSrc.append (">") ;
	return true ;
}

}
