/*
    This file is part of the KDE libraries

    Copyright (C) 1998 Lars Knoll (knoll@mpi-hd.mpg.de)
    Copyright (C) 2001 Dirk Mueller (mueller@kde.org)
    Copyright (C) 2002 Waldo Bastian (bastian@kde.org)
    Copyright (C) 2006 Samuel Weinig (sam.weinig@gmail.com)
    Copyright (C) 2004, 2005, 2006 Apple Computer, Inc.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.

    This class provides all functionality needed for loading images, style sheets and html
    pages from the web. It has a memory cache for these objects.
*/

#include "config.h"
#include "CachedScript.h"

#include "Cache.h"
#include "CachedResourceClient.h"
#include "CachedResourceClientWalker.h"
#include "loader.h"
#include <wtf/Vector.h>
#if defined(SFB_SUPPORT_WMLSCRIPT)
#include "docloader.h"
#include "CString.h"
#include "WMLDocument.h"
#include "WMLScriptInterface.h"
#endif//SFB_SUPPORT_WMLSCRIPT

namespace WebCore {

CachedScript::CachedScript(DocLoader* dl, const String& url, const String& charset)
    : CachedResource(url, Script)
    , m_encoding(charset)
{
    // It's javascript we want.
    // But some websites think their scripts are <some wrong mimetype here>
    // and refuse to serve them if we only accept application/x-javascript.
    setAccept("*/*");
    // load the file
    cache()->loader()->load(dl, this, false);
    m_loading = true;
    if (!m_encoding.isValid())
        m_encoding = Latin1Encoding();
}
#if defined ( SFB_SUPPORT_WMLSCRIPT )
CachedScript::CachedScript(DocLoader* dl, const String& url, const String& charset, Type type)
    : CachedResource(url, type)
    , m_encoding(charset)
{
    // It's javascript we want.
    // But some websites think their scripts are <some wrong mimetype here>
    // and refuse to serve them if we only accept application/x-javascript.
    setAccept("*/*");
    // load the file
    cache()->loader()->load(dl, this, false);
    m_loading = true;
    if (!m_encoding.isValid())
        m_encoding = Latin1Encoding();
}
#endif
CachedScript::~CachedScript()
{
}

#if defined(SFB_SUPPORT_WMLSCRIPT)
WMLCachedScript::WMLCachedScript( DocLoader* dl, const String& url, const String& charset)
:CachedScript(dl,url,charset, WmlScript)
,m_buffer(0) 
,m_bufferLen(0)
{	
	m_doc = static_cast <WMLDocument*>(dl->doc()) ;
	ASSERT (m_doc) ;
    //cache()->loader()->load(dl, this, false);
}

WMLCachedScript::~WMLCachedScript()
{
}

void WMLCachedScript::data(PassRefPtr<SharedBuffer> data, bool allDataReceived)
{
	if(!allDataReceived )
		return ; 
	checkNotify() ;
	String ScriptUrl = url() ;
	String scriptMimetype = m_response.mimeType() ;
	char* bufferval = ( char *) data->data() ; 
	unsigned int bufferLength = data->size() ; 
	
	//if ( scriptMimetype == "text/vnd.wap.wmlscript" )
	if ( scriptMimetype == "text/vnd.wap.wmlscript" )
	{
		bufferContent(ScriptUrl, bufferval, bufferLength ) ;	
	}
	else if( scriptMimetype == "text/vnd.wap.wmlsc" )
	{
		setcompiledBuffer( ScriptUrl, bufferval, bufferLength ) ;
	}

	if (true == m_doc->scriptInterface()->isWmlScriptSuspended())
	{
		if (m_doc->scriptInterface()->downloadUrl == ScriptUrl)
		{
			/* the content type of the load string should start with "text/" */
			if (scriptMimetype.startsWith(String("text/")))
			{
				/* The content can be displayed to the user eg. in case of load string */
				setInterfaceBuffer(bufferval, bufferLength) ;
			}
			else
			{
				/* The content is not a valid content as the content types do not match */
				setInterfaceBuffer(NULL, 0) ;
			}
			CString ctype = scriptMimetype.utf8() ;
			m_doc->scriptInterface()->contentType = (char *)ctype.data() ;
			m_doc->scriptInterface()->resumeScriptExec(NULL) ;
			// Reset the m_doc->scriptInterface()->downloadUrl here
		}
		else if (ScriptUrl.contains(".wmls"))
		{
			m_doc->scriptInterface()->resumeScriptExec(NULL) ;
		}

		
	}
	m_doc->scriptInterface()->setWmlsecExecOnEventScript(ScriptUrl) ;
}

void WMLCachedScript::setInterfaceBuffer (char * buffer, int bufferLen )
{
	m_doc->scriptInterface()->setBuffer (buffer, bufferLen ) ;
}

void WMLCachedScript::bufferContent (  const String& url,  char * buffer, int bufferLen )
{
	m_doc->scriptInterface()->bufferContent ( url, buffer, bufferLen ) ;
}

void WMLCachedScript::setcompiledBuffer( const String& url,  char * buffer, int bufferLen )
{
	m_doc->scriptInterface()->setcompiledBuffer(url, buffer, bufferLen ) ;
}
void WMLCachedScript::setBuffer ( char* buffer, int buflen )
{
	m_buffer = buffer ; 
	m_bufferLen = buflen ;
}
void WMLCachedScript::getBuffer (char **app_buffer, int *ap_bufferLen)
{
	int iIndex ;
	*app_buffer = new char[m_bufferLen];
	for (iIndex=0; iIndex<m_bufferLen; iIndex++)
	{
		(*app_buffer)[iIndex] = m_buffer[iIndex] ;
	}
	*ap_bufferLen = m_bufferLen ;
}
void WMLCachedScript::checkNotify()
{
	if (m_loading)
        return;
}
#endif//SFB_SUPPORT_WMLSCRIPT
void CachedScript::ref(CachedResourceClient* c)
{
    CachedResource::ref(c);
    if (!m_loading)
        c->notifyFinished(this);
}

void CachedScript::setEncoding(const String& chs)
{
    TextEncoding encoding(chs);
    if (encoding.isValid())
        m_encoding = encoding;
}

String CachedScript::encoding() const
{
    return m_encoding.name();
}

void CachedScript::data(PassRefPtr<SharedBuffer> data, bool allDataReceived)
{
    if (!allDataReceived)
        return;

    m_data = data;
    setEncodedSize(m_data.get() ? m_data->size() : 0);
    if (m_data.get())
        m_script = m_encoding.decode(m_data->data(), encodedSize());
    m_loading = false;
    checkNotify();
}

void CachedScript::checkNotify()
{
    if (m_loading)
        return;

    CachedResourceClientWalker w(m_clients);
    while (CachedResourceClient* c = w.next())
        c->notifyFinished(this);
}

void CachedScript::error()
{
    m_loading = false;
    m_errorOccurred = true;
    checkNotify();
}

}
