/*
* ==============================================================================
*  Name         : Navigation.h
*  Part of      : WEBCORE / Page
*  Description  : Interface of Navigation
*  Date         : April 17, 2008
*  Authors      : Vineet Choudhary (vineet.c@siso.com),
*                 Prashant Nevase (n.prashant@siso.com)
* ==============================================================================
*/


#ifndef __NAVIGATION_H__
#define __NAVIGATION_H__

#if PLATFORM(SHP)
#include "CMMDefs.h"
#endif // PLATFORM(SHP)

namespace WebCore {

class Page;
struct NavigationCoreType;
class IntPoint;
class FloatPoint;
class FrameView;
class Frame;
class Node;
enum ScrollDirection;
class IntRect;
enum MouseEventType;

enum ENavigationModeType
{
	NAVIGATION_MODE_VIRTUAL_CURSOR,
	NAVIGATION_MODE_4_WAY_NAVIGATION,
	NAVIGATION_MODE_SCROLL_ONLY
};

struct FastScrollingType
{
	int scrollOffsetMaxValueThreshold;
	int scrollOffsetMultiplier;
	int scrollOffsetLastMultiplier;
	bool isFastScrolling;
};

//the constants in this enum MUST be mutually exclusive
enum ENavigationType
{
	NAVIGATION_NONE = 0x00,
	NAVIGATION_UP = 0x01,
	NAVIGATION_DOWN = 0x02,
	NAVIGATION_LEFT = 0x04,
	NAVIGATION_RIGHT = 0x08
};

//freeze constants
#define NAVIGATION_FREEZED_NONE		(NAVIGATION_NONE)
#define NAVIGATION_FREEZED_UP		(NAVIGATION_UP)
#define NAVIGATION_FREEZED_DOWN		(NAVIGATION_DOWN)
#define NAVIGATION_FREEZED_LEFT		(NAVIGATION_LEFT)
#define NAVIGATION_FREEZED_RIGHT	(NAVIGATION_RIGHT)

class CNavigation
	CMM_MODIFIED
{
public:
	//STATIC METHODS
	//for static global handling
	static void initNavigationStatics();

	static void init();
	static void finalize();

	static CNavigation* createInstance();
	static void destroyInstance(CNavigation* instance);

	//this function should be called only from browser settings
	static void setNavigationMode(ENavigationModeType navigationMode);
	static ENavigationModeType getNavigationMode();

	//for multiple window support
	static void switchPage(CNavigation* navigation);
	
	//NON-OVERRIDABLE METHODS (these are common to all derived classes)
	void setMainFrameView(FrameView* mainFrameView);
	void setGC(int hGC);
	void setPage(Page* page);
	FrameView* getCurrentFrameView() { return m_frameView; }

#ifdef SUPPORT_SMART_TOUCH_FOCUSING
	IntPoint getSmartTouchLocation(IntPoint wndPoint);
	void setSmartTouchEnabled(bool enabled) { m_smartTouchEnabled = enabled; }
	bool isSmartTouchEnabled() { return m_smartTouchEnabled; }
	void setSmartTouchFocusRange(int range); //valid values from 0 to default range
#endif
	

	//OVERRIDABLE METHODS
	virtual void setFrameView(FrameView* frameView) = 0;

	//for new page load in the same window
	virtual void resetPage() = 0;
	virtual void pageLoadingFinished() = 0;

	virtual void updateVerticalStats() = 0;
	virtual void updateHorizontalStats() = 0;

	virtual void handleNavigationEvent(int keyCode, bool isKeyUp) = 0;

	virtual void getPointerLocation(FrameView** frameView, int* x, int* y) = 0;
	virtual void setPointerLocation(int x, int y) = 0;

	virtual void scrollBy(int xOffset, int yOffset) = 0;

#ifdef SUPPORT_TOUCH_SCREEN
	virtual void touchPressed(IntPoint wndPoint) = 0;
	virtual void touchMoved(IntPoint wndPoint) = 0;
	virtual void touchReleased(IntPoint wndPoint) = 0;
#endif

	virtual void doPostPaintAction() = 0;
	virtual void doDefaultAction() = 0;

	virtual Node* getHoveredNode() = 0;


protected:
	CNavigation();
	~CNavigation();

	void startFastScrolling();
	void stopFastScrolling();
	void sendVirtualMouseEvent(MouseEventType type, IntPoint wndPoint, IntPoint contentsPoint);
	Frame* getSubframeForTargetNode(Node* node);
	FrameView* frameViewAtWindowPoint(IntPoint wndPoint);
	float getDistnace(FloatPoint p1, FloatPoint p2);
	bool canNodeAtPointHaveFocus(IntPoint documentPoint, Node*& node);
	bool canNodeHaveFocus(Node* node);

	int m_hGC;
	Page* m_page;
	FrameView* m_mainFrameView;
	FrameView* m_frameView;

	Node* m_hoveredNode;

	int m_scrollOffset;

	//virtual pointer location
	int m_pointerX;
	int m_pointerY;

	///for handling the navigation freezes
	int m_navigationFreezed;


	FastScrollingType m_fastScrolling;
	static int m_defaultScrollStep;
	static int m_defaultAccelerationStep;

	//the one and only instance for sharing common functionality
	static CNavigation* m_navigation;

private:

#ifdef SUPPORT_SMART_TOUCH_FOCUSING
    IntPoint ClosestPointOfRectFromPoint(IntRect rect, IntPoint fromPoint);
	IntPoint ClosestPointOfNodeFromPoint(RenderObject* renderer, IntPoint fromPoint);
	void getRenderersAtSmartTouchLocation(Vector<RenderObject*>& renderersList, IntPoint docPoint);

	bool m_smartTouchEnabled;
	int m_smartTouchFocusRange;
#endif

	static void doSetNavigationMode();
	static void copyToNewInstance(CNavigation* toInstance, CNavigation* fromInstance);


	static NavigationCoreType* m_navigationCore;
};

} //namespace WebCore

#endif