/*
 * Copyright (C) 2006, 2007 Apple Inc. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "config.h"
#include "Page.h"

#include "CSSStyleSelector.h"
#include "Chrome.h"
#include "ChromeClient.h"
#include "ContextMenuClient.h"
#include "ContextMenuController.h"
#include "EditorClient.h"
#include "DragController.h"
#include "FileSystem.h"
#include "FocusController.h"
#include "Frame.h"
#include "FrameLoader.h"
#include "FrameTree.h"
#include "FrameView.h"
#include "HistoryItem.h"
#if (PLATFORM(SHP)) && (!defined(JASMINE_PERFORMANCE_TWEAKS))
#include "InspectorController.h"
#endif
#include "Logging.h"
#include "ProgressTracker.h"
#include "RenderWidget.h"
#include "SelectionController.h"
#include "Settings.h"
#include "StringHash.h"
#include "TextResourceDecoder.h"
#include "Widget.h"
#include <kjs/collector.h>
#include <kjs/JSLock.h>
#include <wtf/HashMap.h>

#if defined(SHP_SUPPORT_WML)
#include "Document.h"
#include "WMLCardElement.h"
#include "HTMLNames.h"
#endif

#if PLATFORM(SHP)
extern bool g_inFinalize;
#endif
using namespace KJS;

namespace WebCore {

#if defined(SHP_SUPPORT_WML)
using namespace HTMLNames;
#endif


static HashSet<Page*>* allPages;
static HashMap<String, HashSet<Page*>*>* frameNamespaces;



#ifndef NDEBUG
WTFLogChannel LogWebCorePageLeaks =  { 0x00000000, "", WTFLogChannelOn };

struct PageCounter { 
    static int count; 
    ~PageCounter() 
    { 
        if (count)
            LOG(WebCorePageLeaks, "LEAK: %d Page\n", count);
    }
};
int PageCounter::count = 0;
static PageCounter pageCounter;
#endif

#ifdef SFB_GLOBAL_STATIC_HANDLING

void deInitPage()
{
	if(allPages)
        allPages->clear();

	if(frameNamespaces)
	    frameNamespaces->clear();
}

void initPage()
{
	allPages = NULL;
	frameNamespaces = NULL;
#ifndef NDEBUG
		PageCounter::count = 0;
#endif

}
#endif//SFB_GLOBAL_STATIC_HANDLING
Page::Page(ChromeClient* chromeClient, ContextMenuClient* contextMenuClient, EditorClient* editorClient, DragClient* dragClient, InspectorClient* inspectorClient)
    : m_chrome(new Chrome(this, chromeClient))
    , m_dragCaretController(new SelectionController(0, true))
    , m_dragController(new DragController(this, dragClient))
    , m_focusController(new FocusController(this))
    , m_contextMenuController(new ContextMenuController(this, contextMenuClient))
#if (PLATFORM(SHP)) && (!defined(JASMINE_PERFORMANCE_TWEAKS))
    , m_inspectorController(new InspectorController(this, inspectorClient))
#endif
    , m_settings(new Settings(this))
    , m_progress(new ProgressTracker)
    , m_backForwardList(new BackForwardList(this))
    , m_editorClient(editorClient)
    , m_frameCount(0)
    , m_tabKeyCyclesThroughElements(true)
    , m_defersLoading(false)
    , m_inLowQualityInterpolationMode(false)
#if (PLATFORM(SHP)) && (!defined(JASMINE_PERFORMANCE_TWEAKS))
    , m_parentInspectorController(0)
#endif
    , m_didLoadUserStyleSheet(false)
    , m_userStyleSheetModificationTime(0)
	, m_group(0)
#if PLATFORM(SHP)
	, m_navigation(0)
	, m_pageLoadingComplete(false)
    , m_isXTHMLFail(false)
#ifdef HTTP_POST_UPLOAD_SUPPORT
	, m_uploadControlNode(0)
#endif // HTTP_POST_UPLOAD_SUPPORT
#endif
{
    if (!allPages) {
        allPages = new HashSet<Page*>;
        setFocusRingColorChangeFunction(setNeedsReapplyStyles);
    }

    ASSERT(!allPages->contains(this));
    allPages->add(this);

#ifndef NDEBUG
    ++PageCounter::count;
#endif
#if PLATFORM(SHP)
	g_inFinalize = false;
#endif
}

Page::~Page()
{
#if PLATFORM(SHP)
	m_navigation = 0;

#ifdef HTTP_POST_UPLOAD_SUPPORT
	m_uploadControlNode = 0;
#endif // HTTP_POST_UPLOAD_SUPPORT

#endif

    m_mainFrame->setView(0);
    setGroupName(String());
    allPages->remove(this);
    
    for (Frame* frame = mainFrame(); frame; frame = frame->tree()->traverseNext())
        frame->pageDestroyed();
    m_editorClient->pageDestroyed();
#if (PLATFORM(SHP)) && (!defined(JASMINE_PERFORMANCE_TWEAKS))
    m_inspectorController->pageDestroyed();
#endif

    m_backForwardList->close();

#ifndef NDEBUG
    --PageCounter::count;

    // Cancel keepAlive timers, to ensure we release all Frames before exiting.
    // It's safe to do this because we prohibit closing a Page while JavaScript
    // is executing.
    Frame::cancelAllKeepAlive();
#endif

    m_chrome.clear();
    m_dragCaretController.clear();
    m_dragController.clear();
    m_focusController.clear();
    m_contextMenuController.clear();
#if (PLATFORM(SHP)) && (!defined(JASMINE_PERFORMANCE_TWEAKS))
    m_inspectorController.clear();
#endif
    m_settings.clear();
    m_progress.clear();
    m_backForwardList.clear();
}

void Page::setMainFrame(PassRefPtr<Frame> mainFrame)
{
    ASSERT(!m_mainFrame); // Should only be called during initialization
    m_mainFrame = mainFrame;
}

BackForwardList* Page::backForwardList()
{
    return m_backForwardList.get();
}

bool Page::goBack()
{
    HistoryItem* item = m_backForwardList->backItem();

#if PLATFORM(SHP)
	if (item)
		m_backForwardList->goBack() ;
//SFB00001637 
	else
		m_mainFrame->loader()->sendAlertMsgforBackwordPage();

#endif 
#if defined(SHP_SUPPORT_WML)
	if(m_mainFrame->document()->htmlMode() == Document::Wml)
	{
		//return checkForLocalCardAndSetActiveId(item,FrameLoadTypeBack);
		ResourceRequest request ;
		m_mainFrame->loader()->setAsyncRequest(request, item, FrameLoadTypeBack) ;
		return true ;
	}
	else
	{
#endif
    
    if (item) {
        goToItem(item, FrameLoadTypeBack);
        return true;
    }
    return false;
#if defined(SHP_SUPPORT_WML)
	}
#endif
}

bool Page::goForward()
{
    HistoryItem* item = m_backForwardList->forwardItem();
    
#ifdef __SHP__
	if (item)
		m_backForwardList->goForward() ;
#endif //__SHP__

#if defined(SHP_SUPPORT_WML)
	if(m_mainFrame->document()->htmlMode() == Document::Wml)
	{
		return checkForLocalCardAndSetActiveId(item,FrameLoadTypeForward);
	}
	else
	{
#endif

    if (item) {
        goToItem(item, FrameLoadTypeForward);
        return true;
    }
    return false;
#if defined(SHP_SUPPORT_WML)
	}
#endif
}

void Page::goToItem(HistoryItem* item, FrameLoadType type)
{
    // Abort any current load if we're going to a history item
    m_mainFrame->loader()->stopAllLoaders();
    m_mainFrame->loader()->goToItem(item, type);
}

void Page::setGroupName(const String& name)
{
    if (m_group && !m_group->name().isEmpty()) {
        ASSERT(m_group != m_singlePageGroup.get());
        ASSERT(!m_singlePageGroup);
        m_group->removePage(this);
    }

    if (name.isEmpty())
        m_group = 0;
    else {
        m_singlePageGroup.clear();
        m_group = PageGroup::pageGroup(name);
        m_group->addPage(this);
    }
}

String Page::groupName() const 
{     
    static String nullString;
    return m_group ? m_group->name() : nullString;
}

void Page::initGroup()
{
    ASSERT(!m_singlePageGroup);
    ASSERT(!m_group);
    m_singlePageGroup.set(new PageGroup(this));
    m_group = m_singlePageGroup.get();
}


const HashSet<Page*>* Page::frameNamespace() const
{
    return (frameNamespaces && !m_groupName.isEmpty()) ? frameNamespaces->get(m_groupName) : 0;
}

const HashSet<Page*>* Page::frameNamespace(const String& groupName)
{
    return (frameNamespaces && !groupName.isEmpty()) ? frameNamespaces->get(groupName) : 0;
}

const HashSet<Page *>* Page::getAllPages()
{
	return allPages;
}

void Page::setNeedsReapplyStyles()
{
    if (!allPages)
        return;
    HashSet<Page*>::iterator end = allPages->end();
    for (HashSet<Page*>::iterator it = allPages->begin(); it != end; ++it)
        for (Frame* frame = (*it)->mainFrame(); frame; frame = frame->tree()->traverseNext())
            frame->setNeedsReapplyStyles();
}

static Frame* incrementFrame(Frame* curr, bool forward, bool wrapFlag)
{
    return forward
        ? curr->tree()->traverseNextWithWrap(wrapFlag)
        : curr->tree()->traversePreviousWithWrap(wrapFlag);
}

bool Page::findString(const String& target, TextCaseSensitivity caseSensitivity, FindDirection direction, bool shouldWrap)
{
    if (target.isEmpty() || !mainFrame())
        return false;

    Frame* frame = focusController()->focusedOrMainFrame();
    Frame* startFrame = frame;
    do {
        if (frame->findString(target, direction == FindDirectionForward, caseSensitivity == TextCaseSensitive, false, true)) {
            if (frame != startFrame)
                startFrame->selectionController()->clear();
            focusController()->setFocusedFrame(frame);
            return true;
        }
        frame = incrementFrame(frame, direction == FindDirectionForward, shouldWrap);
    } while (frame && frame != startFrame);

    // Search contents of startFrame, on the other side of the selection that we did earlier.
    // We cheat a bit and just research with wrap on
    if (shouldWrap && !startFrame->selectionController()->isNone()) {
        bool found = startFrame->findString(target, direction == FindDirectionForward, caseSensitivity == TextCaseSensitive, true, true);
        focusController()->setFocusedFrame(frame);
        return found;
    }

    return false;
}

unsigned int Page::markAllMatchesForText(const String& target, TextCaseSensitivity caseSensitivity, bool shouldHighlight, unsigned limit)
{
    if (target.isEmpty() || !mainFrame())
        return 0;

    unsigned matches = 0;

    Frame* frame = mainFrame();
    do {
        frame->setMarkedTextMatchesAreHighlighted(shouldHighlight);
        matches += frame->markAllMatchesForText(target, caseSensitivity == TextCaseSensitive, (limit == 0) ? 0 : (limit - matches));
        frame = incrementFrame(frame, true, false);
    } while (frame);

    return matches;
}

void Page::unmarkAllTextMatches()
{
    if (!mainFrame())
        return;

    Frame* frame = mainFrame();
    do {
        if (Document* document = frame->document())
            document->removeMarkers(DocumentMarker::TextMatch);
        frame = incrementFrame(frame, true, false);
    } while (frame);
}

const Selection& Page::selection() const
{
    return focusController()->focusedOrMainFrame()->selectionController()->selection();
}

void Page::setDefersLoading(bool defers)
{
    if (defers == m_defersLoading)
        return;

    m_defersLoading = defers;
    for (Frame* frame = mainFrame(); frame; frame = frame->tree()->traverseNext())
        frame->loader()->setDefersLoading(defers);
}

void Page::clearUndoRedoOperations()
{
    m_editorClient->clearUndoRedoOperations();
}

bool Page::inLowQualityImageInterpolationMode() const
{
    return m_inLowQualityInterpolationMode;
}

void Page::setInLowQualityImageInterpolationMode(bool mode)
{
    m_inLowQualityInterpolationMode = mode;
}

void Page::userStyleSheetLocationChanged()
{
#if !FRAME_LOADS_USER_STYLESHEET
    // FIXME: We should provide a way to load other types of URLs than just
    // file: (e.g., http:, data:).
    if (m_settings->userStyleSheetLocation().isLocalFile())
        m_userStyleSheetPath = m_settings->userStyleSheetLocation().fileSystemPath();
    else
        m_userStyleSheetPath = String();

    m_didLoadUserStyleSheet = false;
    m_userStyleSheet = String();
    m_userStyleSheetModificationTime = 0;
#endif
}

const String& Page::userStyleSheet() const
{
    if (m_userStyleSheetPath.isEmpty()) {
        ASSERT(m_userStyleSheet.isEmpty());
        return m_userStyleSheet;
    }

    time_t modTime;
    if (!getFileModificationTime(m_userStyleSheetPath, modTime)) {
        // The stylesheet either doesn't exist, was just deleted, or is
        // otherwise unreadable. If we've read the stylesheet before, we should
        // throw away that data now as it no longer represents what's on disk.
        m_userStyleSheet = String();
        return m_userStyleSheet;
    }

    // If the stylesheet hasn't changed since the last time we read it, we can
    // just return the old data.
    if (m_didLoadUserStyleSheet && modTime <= m_userStyleSheetModificationTime)
        return m_userStyleSheet;

    m_didLoadUserStyleSheet = true;
    m_userStyleSheet = String();
    m_userStyleSheetModificationTime = modTime;

    // FIXME: It would be better to load this asynchronously to avoid blocking
    // the process, but we will first need to create an asynchronous loading
    // mechanism that is not tied to a particular Frame. We will also have to
    // determine what our behavior should be before the stylesheet is loaded
    // and what should happen when it finishes loading, especially with respect
    // to when the load event fires, when Document::close is called, and when
    // layout/paint are allowed to happen.
    RefPtr<SharedBuffer> data = SharedBuffer::createWithContentsOfFile(m_userStyleSheetPath);
    if (!data)
        return m_userStyleSheet;

    m_userStyleSheet = TextResourceDecoder("text/css").decode(data->data(), data->size());

    return m_userStyleSheet;
}

#if defined(SHP_SUPPORT_WML)
bool Page::checkForLocalCardAndSetActiveId(HistoryItem *item, FrameLoadType loadType)
{
	if(item)
	{
		String url = item->urlString();
		String targetUrl = m_mainFrame->document()->completeURL(url);
		//It may be link like #card3, remove first character, get card id, card3 in this case
	
		int pos = url.find('#');
		if(pos == -1) // # not found in url, means it can be normal url like google.com
		{
			goToItem(item, loadType);
			return true;
		}
		else if(pos == 0) // first char is '#', so it may be local card
		{
			url = url.substring(1);
		}
		else   //For case like samepage.wml#card2, we should not call base class, should just set active id.
		{
			//Get current url from document
			String docUrl = m_mainFrame->document()->url();
			String docUrlPrefix = "";  //holds document url excluding card id
			int revpos;
			if((revpos = docUrl.reverseFind('#')) == -1)
			{
				docUrlPrefix = docUrl;
			}
			else
			{
				docUrlPrefix = docUrl.substring(0,revpos);
			}

			String targetUrlPrefix = "";  //holds target url excluding card id
			if((revpos = targetUrl.reverseFind('#')) == -1)
			{
				targetUrlPrefix = targetUrl;
			}
			else
			{
				targetUrlPrefix = targetUrl.substring(0,revpos);
			}

			if(docUrlPrefix != targetUrlPrefix) //Check whether request is with in the page or not
			{
				goToItem(item, loadType);
				if(loadType == FrameLoadTypeBack)
				{
					m_mainFrame->loader()->setForwardNavigation(false);
				}
				else if(loadType == FrameLoadTypeForward)
				{
					m_mainFrame->loader()->setForwardNavigation(true);
				}
				return true;
			}
			else //card in same page, set active id
			{
				url = url.substring(pos+1);
			}
		}

		//set active id first and start from event handling from that card
		m_mainFrame->document()->resetTemplateEventFlags();
		m_mainFrame->document()->setActiveCardId(url);
		HTMLCollection *pCardCollection = new HTMLCollection (m_mainFrame->document(), HTMLCollection::CardTag) ;
		bool bIsEventHandled = false;
		do
		{
			m_mainFrame->document()->setIsCardChanged(false);
			String activeId = m_mainFrame->document()->getActiveCardId();
			WmlCardElement *pCard = NULL;
			for(Node *pNode = pCardCollection->firstItem(); pNode; pNode = pCardCollection->nextItem())
			{
				pCard = static_cast<WmlCardElement *>(pNode);
				if(pCard->id() == activeId)
				{
					break;
				}
			}
			bIsEventHandled = false;
			if(pCard) //This is card node, call handleEvent and check whether card id changed or not
			{
				if(loadType == FrameLoadTypeBack)
				{
					pCard->handleEvent(NavigationEvent,false, bIsEventHandled);
				}
				else if(loadType == FrameLoadTypeForward)
				{
					pCard->handleEvent(NavigationEvent,true, bIsEventHandled);
				}
			}
		}while(m_mainFrame->document()->isCardChanged());

		if(false == bIsEventHandled)
		{
			//Event flags will be set in the above loop.
			//Event flags decide whether to do tasks in template event elements or not.
			HTMLCollection *pTemplateCollection = new HTMLCollection (m_mainFrame->document(), HTMLCollection::TemplateTag) ;
			Node *pNode = pTemplateCollection->firstItem() ;
			
			if(pNode) //process template node first
			{
				if(loadType == FrameLoadTypeBack)
				{
					pNode->handleEvent(NavigationEvent,false, bIsEventHandled);
				}
				else if(loadType == FrameLoadTypeForward)
				{
					pNode->handleEvent(NavigationEvent,true, bIsEventHandled);
				}
			}
			delete pTemplateCollection;
		}

		//Reset bIsEventAlreadyHandled flag of all cards
		m_mainFrame->document()->resetCardEventFlags(pCardCollection);
		m_mainFrame->document()->resetTemplateEventFlags();
		m_mainFrame->document()->reRender();
		delete pCardCollection;
		return true;
	}
	return false;
}
#endif



void Page::removeAllVisitedLinks()
{
    if (!allPages)
        return;
    HashSet<PageGroup*> groups;
    HashSet<Page*>::iterator pagesEnd = allPages->end();
    for (HashSet<Page*>::iterator it = allPages->begin(); it != pagesEnd; ++it) {
        if (PageGroup* group = (*it)->groupPtr())
            groups.add(group);
    }
    HashSet<PageGroup*>::iterator groupsEnd = groups.end();
    for (HashSet<PageGroup*>::iterator it = groups.begin(); it != groupsEnd; ++it)
        (*it)->removeVisitedLinks();
}

void Page::allVisitedStateChanged(PageGroup* group)
{
    ASSERT(group);
    ASSERT(allPages);
    HashSet<Page*>::iterator pagesEnd = allPages->end();
    for (HashSet<Page*>::iterator it = allPages->begin(); it != pagesEnd; ++it) {
        Page* page = *it;
        if (page->m_group != group)
            continue;
        for (Frame* frame = page->m_mainFrame.get(); frame; frame = frame->tree()->traverseNext()) {
            if (CSSStyleSelector* styleSelector = frame->document()->styleSelector())
                styleSelector->allVisitedStateChanged();
        }
    }
}

void Page::visitedStateChanged(PageGroup* group, unsigned visitedLinkHash)
{
    ASSERT(group);
    ASSERT(allPages);
    HashSet<Page*>::iterator pagesEnd = allPages->end();
    for (HashSet<Page*>::iterator it = allPages->begin(); it != pagesEnd; ++it) {
        Page* page = *it;
        if (page->m_group != group)
            continue;
        for (Frame* frame = page->m_mainFrame.get(); frame; frame = frame->tree()->traverseNext()) {
            if (CSSStyleSelector* styleSelector = frame->document()->styleSelector())
                styleSelector->visitedStateChanged(visitedLinkHash);
        }
    }
}

#if PLATFORM(SHP)
bool Page::isAddedToFileContext(const String& absUrl) const
{
    return m_savePageMap.contains(absUrl);

}

String Page::getFileName (const String& absUrl)
{
    String filename = m_savePageMap.get(absUrl) ;
    return filename ;
}

void Page::addSavePageList (const String& absUrl, const String& fileName)
{
    if (!m_savePageMap.contains(absUrl))
        m_savePageMap.add(absUrl, fileName) ;
}

bool Page::getPageOverViewSettings()
{
	return settings()->arePageOverViewEnabled();
}
#endif //PLATFORM(SHP

} // namespace WebCore
