/*
 * Copyright (C) 2006, 2007 Apple Inc.  All rights reserved.
 * Copyright (C) 2006 Justin Haygood <jhaygood@spsu.edu>.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "ScrollView.h"

#include "Chrome.h"
#include "ChromeClient.h"
#include "FloatRect.h"

#include "Frame.h"
#include "FrameView.h"
#include "RenderTheme.h"
#include "RenderFrame.h"

#include "GraphicsContext.h"
#include "IntRect.h"
#include "Page.h"
#include "PlatformScrollBar.h"
#include "PlatformMouseEvent.h"
#include "PlatformWheelEvent.h"
#include "ScrollBar.h"

#include "NotImplemented.h"

#include <algorithm>
#include <wtf/Assertions.h>
#include <wtf/HashSet.h>

#include "EventMgr.h"
#include "gwes.h"
#include "ShpJasmine.h"


#include "Navigation.h"

#ifdef USE_NATIVE_WIDGET_SUPPORT
#include "NativeEditField.h"
#include "FocusController.h"
#endif

using namespace std;

static const int MagicShpScrollConstant = 13;
// bs0425.lee modified for touch scrollbar. 
static WebCore::IntRect m_trackRect;
#define extendRange		40

#define _SFB_BACKINGSTORE_IMPROVE_LOADPERF

namespace WebCore {

class ScrollViewScrollbar : public PlatformScrollbar {
public:
    ScrollViewScrollbar(ScrollbarClient*, ScrollbarOrientation, ScrollbarControlSize);
	ScrollViewScrollbar(ScrollbarClient*, ScrollbarOrientation, IntRect);
};

class ScrollView::ScrollViewPrivate : public ScrollbarClient
CMM_MODIFIED_ADD
{
public:
    ScrollViewPrivate(ScrollView* _view)
        : view(_view)
        , hasStaticBackground(false)
        , scrollbarsSuppressed(false)
        , vScrollbarMode(ScrollbarAuto)
        , hScrollbarMode(ScrollbarAuto)
        , inUpdateScrollbars(false)
    {}

    ~ScrollViewPrivate()
    {
        setHasHorizontalScrollbar(false);
        setHasVerticalScrollbar(false);
    }

    void scrollBackingStore(const IntSize& scrollDelta);

    void setHasHorizontalScrollbar(bool hasBar);
    void setHasVerticalScrollbar(bool hasBar);

    virtual void valueChanged(Scrollbar*);
    virtual IntRect windowClipRect() const;
	virtual bool isActive() const;

    ScrollView* view;
    bool hasStaticBackground;
    bool scrollbarsSuppressed;
    ScrollbarMode vScrollbarMode;
    ScrollbarMode hScrollbarMode;
    RefPtr<ScrollViewScrollbar> vBar;
    RefPtr<ScrollViewScrollbar> hBar;
    IntSize scrollOffset;
    IntSize contentsSize;
    IntSize viewPortSize;
    bool inUpdateScrollbars;
    HashSet<Widget*> children;
};

ScrollViewScrollbar::ScrollViewScrollbar(ScrollbarClient* client, ScrollbarOrientation orientation, ScrollbarControlSize size)
    : PlatformScrollbar(client, orientation, size)
{
}

#ifdef USE_PLATFORM_SCROLLBARS
//this constructor is added for iframeviews
ScrollViewScrollbar::ScrollViewScrollbar(ScrollbarClient* client, ScrollbarOrientation orientation, IntRect rect)
	: PlatformScrollbar(client, orientation, rect)
{
}
#endif

void ScrollView::ScrollViewPrivate::setHasHorizontalScrollbar(bool hasBar)
{
    if (Scrollbar::hasPlatformScrollbars()) 
	{
        if (hasBar && !hBar) 
		{
#ifdef USE_PLATFORM_SCROLLBARS
			//for iframeview, there is a clipping issue of native scrollbars,
			//so we need to handle the scrollbar painting separately.
			if (static_cast<FrameView *>(view)->isIFrameView())
			{
				// the IFrameView::paint takes care of the scrollbar position
				// hence we hide the scrollbar by setting its x position to 10000
				hBar = new ScrollViewScrollbar(this, HorizontalScrollbar, 
												IntRect(10000, 0, 
														view->width(), 
														view->height()) );	
			}
			else
#endif //USE_PLATFORM_SCROLLBARS
			{
				hBar = new ScrollViewScrollbar(this, HorizontalScrollbar, RegularScrollbar);
			}

			view->addChild(hBar.get());
        } 
		else if (!hasBar && hBar) 
		{
            view->removeChild(hBar.get());
            hBar = 0;
        }
    }
}

void ScrollView::ScrollViewPrivate::setHasVerticalScrollbar(bool hasBar)
{
    if (Scrollbar::hasPlatformScrollbars()) 
	{
        if (hasBar && !vBar) 
		{
#ifdef USE_PLATFORM_SCROLLBARS
			//for iframeview, there is a clipping issue of native scrollbars,
			//so we need to handle the scrollbar painting separately.
			if (static_cast<FrameView *>(view)->isIFrameView())
			{
				// the IFrameView::paint takes care of the scrollbar position
				// hence we hide the scrollbar by setting its x position to 10000
				vBar = new ScrollViewScrollbar(this, VerticalScrollbar, 
												IntRect(10000, 0, 
														view->width(), 
														view->height()) );	
			}
			else
#endif //USE_PLATFORM_SCROLLBARS
			{
				vBar = new ScrollViewScrollbar(this, VerticalScrollbar, RegularScrollbar);
			}

			view->addChild(vBar.get());
        } 
		else if (!hasBar && vBar) 
		{
            view->removeChild(vBar.get());
            vBar = 0;
        }
    }
}


void ScrollView::ScrollViewPrivate::scrollBackingStore(const IntSize& scrollDelta)
{
    // Since scrolling is double buffered, we will be blitting the scroll view's intersection
    // with the clip rect every time to keep it smooth.
    IntRect clipRect = view->windowClipRect();
    IntRect scrollViewRect = view->convertToContainingWindow(IntRect(0, 0, view->visibleWidth(), view->visibleHeight()));

    IntRect updateRect = clipRect;
    updateRect.intersect(scrollViewRect);

    //FIXME update here?

    if (!hasStaticBackground) // The main frame can just blit the WebView window
       // FIXME: Find a way to blit subframes without blitting overlapping content
       view->scrollBackingStore(-scrollDelta.width(), -scrollDelta.height(), scrollViewRect, clipRect);
    else  {
       // We need to go ahead and repaint the entire backing store.  Do it now before moving the
       // plugins.
       view->addToDirtyRegion(updateRect);
       view->updateBackingStore();
    }

    view->geometryChanged();

    // Now update the window (which should do nothing but a blit of the backing store's updateRect and so should
    // be very fast).

	ScrollView* parentView = view;
	while(parentView->parent())
		parentView = parentView->parent();

	parentView->updateOnScroll();
}

void ScrollView::ScrollViewPrivate::valueChanged(Scrollbar* bar)
{
    // Figure out if we really moved.
    IntSize newOffset = scrollOffset;
    if (bar) {
        if (bar == hBar)
            newOffset.setWidth(bar->value());
        else if (bar == vBar)
            newOffset.setHeight(bar->value());
    }
    IntSize scrollDelta = newOffset - scrollOffset;
    if (scrollDelta == IntSize())
        return;
    scrollOffset = newOffset;

    if (scrollbarsSuppressed)
        return;

// shkim
// To improve scroll speed
#if !defined(_SFB_BACKINGSTORE)
    scrollBackingStore(scrollDelta);
    static_cast<FrameView*>(view)->frame()->sendScrollEvent();
#endif

}

IntRect ScrollView::ScrollViewPrivate::windowClipRect() const
{
    return static_cast<const FrameView*>(view)->windowClipRect(false);
}

bool ScrollView::ScrollViewPrivate::isActive() const
{
    Page* page = static_cast<const FrameView*>(view)->frame()->page();
    return page && page->focusController()->isActive();
}

ScrollView::ScrollView()
    : m_data(new ScrollViewPrivate(this)),
	m_isPageLoadComplete(false)
{}

ScrollView::~ScrollView()
{
	delete m_data;
}

PlatformScrollbar* ScrollView::horizontalScrollBar() const
{
    return m_data->hBar.get();
}

PlatformScrollbar* ScrollView::verticalScrollBar() const
{
    return m_data->vBar.get();
}

IntPoint ScrollView::convertChildToSelf(const Widget* child, const IntPoint& point) const
{
    IntPoint newPoint = point;
    if (child != m_data->hBar && child != m_data->vBar)
        newPoint = point - scrollOffset();
    return Widget::convertChildToSelf(child, newPoint);
}

IntPoint ScrollView::convertSelfToChild(const Widget* child, const IntPoint& point) const
{
    IntPoint newPoint = point;
    if (child != m_data->hBar && child != m_data->vBar)
        newPoint = point + scrollOffset();
    return Widget::convertSelfToChild(child, newPoint);
}

void ScrollView::geometryChanged() const
{
    HashSet<Widget*>::const_iterator end = m_data->children.end();
    for (HashSet<Widget*>::const_iterator current = m_data->children.begin(); current != end; ++current)
        (*current)->geometryChanged();
}

void ScrollView::paint(GraphicsContext* context, const IntRect& rect)
{
    // FIXME: This code is here so we don't have to fork FrameView.h/.cpp.
    // In the end, FrameView should just merge with ScrollView.
    ASSERT(isFrameView());

    if (context->paintingDisabled())
        return;

    IntRect documentDirtyRect = rect;
#if !defined(_SFB_BACKINGSTORE)
    documentDirtyRect.intersect(frameGeometry());
#endif

    context->save();

    context->translate(x(), y());
    documentDirtyRect.move(-x(), -y());

    context->translate(-contentsX(), -contentsY());
	documentDirtyRect.move(contentsX(), contentsY());
//iframe change
//#if !defined(_SFB_BACKINGSTORE) 
    context->clip(enclosingIntRect(visibleContentRect()));
//#endif

	if (!documentDirtyRect.isEmpty())
	{
	    static_cast<const FrameView*>(this)->frame()->paint(context, documentDirtyRect);
	}
    context->restore();

#if !defined(_SFB_BACKINGSTORE)
    // Now paint the scrollbars.
    if (!m_data->scrollbarsSuppressed && (m_data->hBar || m_data->vBar)) {
        context->save();
        IntRect scrollViewDirtyRect = rect;
        scrollViewDirtyRect.intersect(frameGeometry());
        context->translate(x(), y());
        scrollViewDirtyRect.move(-x(), -y());
        if (m_data->hBar)
            m_data->hBar->paint(context, scrollViewDirtyRect);
        if (m_data->vBar)
            m_data->vBar->paint(context, scrollViewDirtyRect);

        /*
         * FIXME: TODO: Check if that works with RTL
         */
        // Fill the scroll corner with white.
        IntRect hCorner;
        if (m_data->hBar && width() - m_data->hBar->width() > 0) {
            hCorner = IntRect(m_data->hBar->width(),
                              height() - m_data->hBar->height(),
                              width() - m_data->hBar->width(),
                              m_data->hBar->height());
            if (hCorner.intersects(scrollViewDirtyRect))
                context->fillRect(hCorner, Color::white);
        }

        if (m_data->vBar && height() - m_data->vBar->height() > 0) {
            IntRect vCorner(width() - m_data->vBar->width(),
                            m_data->vBar->height(),
                            m_data->vBar->width(),
                            height() - m_data->vBar->height());
            if (vCorner != hCorner && vCorner.intersects(scrollViewDirtyRect))
                context->fillRect(vCorner, Color::white);
        }

        context->restore();
    }
#endif
}

void ScrollView::setFrameGeometry(const IntRect& newGeometry)
{
    ASSERT(isFrameView());
    IntRect oldGeometry = frameGeometry();
    Widget::setFrameGeometry(newGeometry);

    if (newGeometry == oldGeometry)
        return;
    if (newGeometry.width() != oldGeometry.width() || newGeometry.height() != oldGeometry.height()) {
        updateScrollbars(m_data->scrollOffset);
        static_cast<FrameView*>(this)->setNeedsLayout();
    }

    geometryChanged();
}

void ScrollView::addToDirtyRegion(const IntRect& containingWindowRect)
{
    ASSERT(isFrameView());
    const FrameView* frameView = static_cast<const FrameView*>(this);
    Page* page = frameView->frame() ? frameView->frame()->page() : 0;
    if (!page)
        return;
    page->chrome()->addToDirtyRegion(containingWindowRect);
}

void ScrollView::scrollBackingStore(int dx, int dy, const IntRect& scrollViewRect, const IntRect& clipRect)
{
    ASSERT(isFrameView());
    const FrameView* frameView = static_cast<const FrameView*>(this);
    Page* page = frameView->frame() ? frameView->frame()->page() : 0;
    if (!page)
        return;
    page->chrome()->scrollBackingStore(dx, dy, scrollViewRect, clipRect);
}

void ScrollView::updateBackingStore()
{
    ASSERT(isFrameView());
    const FrameView* frameView = static_cast<const FrameView*>(this);
    Page* page = frameView->frame() ? frameView->frame()->page() : 0;
    if (!page)
        return;
    page->chrome()->updateBackingStore();
}

IntSize ScrollView::maximumScroll() const
{
    IntSize delta = (m_data->contentsSize - IntSize(visibleWidth(), visibleHeight())) - scrollOffset();
    delta.clampNegativeToZero();
    return delta;
}

HashSet<Widget*>* ScrollView::children() const
{
    return &m_data->children;
}

void ScrollView::updateScrollbars(const IntSize& desiredOffset)
{
    // Don't allow re-entrancy into this function.
    if (m_data->inUpdateScrollbars)
        return;

    // FIXME: This code is here so we don't have to fork FrameView.h/.cpp.
    // In the end, FrameView should just merge with ScrollView.
    if (static_cast<const FrameView*>(this)->frame()->prohibitsScrolling())
        return;

    m_data->inUpdateScrollbars = true;

    bool hasVerticalScrollbar = m_data->vBar;
    bool hasHorizontalScrollbar = m_data->hBar;
    bool oldHasVertical = hasVerticalScrollbar;
    bool oldHasHorizontal = hasHorizontalScrollbar;
    ScrollbarMode hScroll = m_data->hScrollbarMode;
    ScrollbarMode vScroll = m_data->vScrollbarMode;

    //Celunite - SDG - Not considering the width and height of the scroll bars
    //while deciding wether we want a particular scroll bar or not. 
    //if we consider the width and height we end up getting a horizontal 
    //scroll bar in single coln redering and wml pages
    const int cVerticalWidth = PlatformScrollbar::verticalScrollbarWidth();
    const int cHorizontalHeight = PlatformScrollbar::horizontalScrollbarHeight();
	const int cMinThumbSize = PlatformScrollbar::minThumbSize();

    for (int pass = 0; pass < 2; pass++) {
        bool scrollsVertically;
        bool scrollsHorizontally;

        if (!m_data->scrollbarsSuppressed && (hScroll == ScrollbarAuto || vScroll == ScrollbarAuto)) {
            // Do a layout if pending before checking if scrollbars are needed.
#if (PLATFORM(SHP)) && (!defined(JASMINE_PERFORMANCE_TWEAKS))
            if (hasVerticalScrollbar != oldHasVertical || hasHorizontalScrollbar != oldHasHorizontal)
                static_cast<FrameView*>(this)->layout();
#endif
            scrollsVertically = (vScroll == ScrollbarAlwaysOn) || (vScroll == ScrollbarAuto && contentsHeight() > height());
            if (scrollsVertically)
                scrollsHorizontally = (hScroll == ScrollbarAlwaysOn) || (hScroll == ScrollbarAuto && contentsWidth() + cVerticalWidth > width());
            else {
                scrollsHorizontally = (hScroll == ScrollbarAlwaysOn) || (hScroll == ScrollbarAuto && contentsWidth() > width());
                if (scrollsHorizontally)
                    scrollsVertically = (vScroll == ScrollbarAlwaysOn) || (vScroll == ScrollbarAuto && contentsHeight() + cHorizontalHeight > height());
            }
        }
        else {
            scrollsHorizontally = (hScroll == ScrollbarAuto) ? hasHorizontalScrollbar : (hScroll == ScrollbarAlwaysOn);
            scrollsVertically = (vScroll == ScrollbarAuto) ? hasVerticalScrollbar : (vScroll == ScrollbarAlwaysOn);
        }

		//for width less than vertical scrollbar width, vertical scrollbar shouldn't be shown
		if(width() < cVerticalWidth || height() < cMinThumbSize)
			scrollsVertically = false;

		//for height less than horizontal scrollbar height, horizontal scrollbar shouldn't be shown
		if(height() < cHorizontalHeight || width() < cMinThumbSize)
			scrollsHorizontally = false;


        if (hasVerticalScrollbar != scrollsVertically) {
            m_data->setHasVerticalScrollbar(scrollsVertically);
            hasVerticalScrollbar = scrollsVertically;

			if(hasVerticalScrollbar)
			{
				static_cast<FrameView *>(this)->frame()->page()->navigationInstance()->updateVerticalStats();
			}
        }

        if (hasHorizontalScrollbar != scrollsHorizontally) {
            m_data->setHasHorizontalScrollbar(scrollsHorizontally);
            hasHorizontalScrollbar = scrollsHorizontally;

			if(hasHorizontalScrollbar)
			{
				static_cast<FrameView *>(this)->frame()->page()->navigationInstance()->updateHorizontalStats();
			}
        }
    }

    // Set up the range (and page step/line step).
    IntSize maxScrollPosition(contentsWidth() - visibleWidth(), contentsHeight() - visibleHeight());
    IntSize scroll = desiredOffset.shrunkTo(maxScrollPosition);
    scroll.clampNegativeToZero();
#if defined(_SFB_BACKINGSTORE)
	IntSize oldScrollOffset = m_data->scrollOffset;
#endif
	if (m_data->hBar) {
        int clientWidth = visibleWidth();
        m_data->hBar->setEnabled(contentsWidth() > clientWidth);
        int pageStep = (clientWidth - PAGE_KEEP);
        if (pageStep < 0) pageStep = clientWidth;
        IntRect oldRect(m_data->hBar->frameGeometry());
        IntRect hBarRect = IntRect(0,
                                   height() - m_data->hBar->height(),
                                   width() - (m_data->vBar ? m_data->vBar->width() : 0),
                                   m_data->hBar->height());
        m_data->hBar->setRect(hBarRect);
        if (!m_data->scrollbarsSuppressed && oldRect != m_data->hBar->frameGeometry())
            m_data->hBar->invalidate();

        if (m_data->scrollbarsSuppressed)
            m_data->hBar->setSuppressInvalidation(true);
        m_data->hBar->setSteps(LINE_STEP, pageStep);
        m_data->hBar->setProportion(clientWidth, contentsWidth());
        m_data->hBar->setValue(scroll.width());
        if (m_data->scrollbarsSuppressed)
            m_data->hBar->setSuppressInvalidation(false);
    }

	if (m_data->vBar) {
        int clientHeight = visibleHeight();
        m_data->vBar->setEnabled(contentsHeight() > clientHeight);
        int pageStep = (clientHeight - PAGE_KEEP);
        if (pageStep < 0) pageStep = clientHeight;
        IntRect oldRect(m_data->vBar->frameGeometry());
        IntRect vBarRect = IntRect(width() - m_data->vBar->width(),
                                   0,
                                   m_data->vBar->width(),
                                   height() - (m_data->hBar ? m_data->hBar->height() : 0));
        m_data->vBar->setRect(vBarRect);
        if (!m_data->scrollbarsSuppressed && oldRect != m_data->vBar->frameGeometry())
            m_data->vBar->invalidate();

        if (m_data->scrollbarsSuppressed)
            m_data->vBar->setSuppressInvalidation(true);
        m_data->vBar->setSteps(LINE_STEP, pageStep);
        m_data->vBar->setProportion(clientHeight, contentsHeight());
        m_data->vBar->setValue(scroll.height());
        if (m_data->scrollbarsSuppressed)
            m_data->vBar->setSuppressInvalidation(false);
    }

#if defined(_SFB_BACKINGSTORE)
	//shkim
	// To improve scroll speed, scroll horizontal and vertical at the same time.
	if (m_data->vBar || m_data->hBar )
	{
		IntSize newOffset = oldScrollOffset;
		if (m_data->hBar)
			newOffset.setWidth(m_data->hBar->value());
		if (m_data->vBar)
			newOffset.setHeight(m_data->vBar->value());
	
		IntSize scrollDelta = newOffset - oldScrollOffset;
		if (scrollDelta != IntSize())
		{
			m_data->scrollBackingStore(scrollDelta);
			static_cast<FrameView*>(m_data->view)->frame()->sendScrollEvent();
		}
	}
#endif

    if (oldHasVertical != (m_data->vBar != 0) || oldHasHorizontal != (m_data->hBar != 0))
        geometryChanged();

    // See if our offset has changed in a situation where we might not have scrollbars.
    // This can happen when editing a body with overflow:hidden and scrolling to reveal selection.
    // It can also happen when maximizing a window that has scrollbars (but the new maximized result
    // does not).
    IntSize scrollDelta = scroll - m_data->scrollOffset;
    if (scrollDelta != IntSize()) {
       m_data->scrollOffset = scroll;
       m_data->scrollBackingStore(scrollDelta);
    }

    m_data->inUpdateScrollbars = false;
}

int ScrollView::visibleWidth() const
{
#ifdef _SFB_BACKINGSTORE
	return  width();
#else
     return width() - (m_data->vBar ? m_data->vBar->width() : 0);
#endif
}

int ScrollView::visibleHeight() const
{
#ifdef _SFB_BACKINGSTORE
	return height() ;
#else
    return height() - (m_data->hBar ? m_data->hBar->height() : 0);
#endif
}

FloatRect ScrollView::visibleContentRect() const
{
    return FloatRect(contentsX(), contentsY(), visibleWidth(), visibleHeight());
}

FloatRect ScrollView::visibleContentRectConsideringExternalScrollers() const
{
    return visibleContentRect();
}

void ScrollView::setContentsPos(int newX, int newY)
{
    int dx = newX - contentsX();
    int dy = newY - contentsY();

#if defined(_SFB_BACKINGSTORE)
		if( (dx == 0) && (dy == 0) )
		return;
	const FrameView* frameView = static_cast<const FrameView*>(this);
    Page* page = frameView->frame() ? frameView->frame()->page() : 0;
    if (!page)
        return;
	page->chrome()->client()->scrollTo( IntPoint(dx, dy) );
#else
	
    scrollBy(dx, dy);
#endif

}

void ScrollView::resizeContents(int w, int h)
{
    IntSize newSize(w, h);
    if (m_data->contentsSize == newSize)
        return;

    m_data->contentsSize = newSize;

	// In case of framesets to avoid each frame to have its own scroll bar we set
	// the size of the frameset to its contents size (refer RenderFrameSet::layout). 
	// Hence we need to relayout everytime size of any frame changes.

	// Figure out if the page has a frame set if yes get the top level FrameView and FrameSet.
	// also we find out  the height of the frame with max height.
	FrameView *currentView = static_cast<FrameView *>(this);
	FrameView *parentView = static_cast<FrameView *>(currentView ->parent());

	if ((parentView && parentView->frame()->isFrameSet()))
	{
		while (currentView->parent())
			currentView = static_cast<FrameView *>(currentView->parent());

		currentView->setNeedsLayout();
	}

	if(IntSize() != newSize)
	{
		//update stats as page load is in progress
		//update horizontal stats
		static_cast<FrameView *>(this)->frame()->page()->navigationInstance()->updateHorizontalStats();
		//update vertical stats
		static_cast<FrameView *>(this)->frame()->page()->navigationInstance()->updateVerticalStats();
	}

    updateScrollbars(m_data->scrollOffset);


//#ifdef USE_NATIVE_WIDGET_SUPPORT
//
//	FrameView* view = static_cast<FrameView *>(this);
//	Document* document = view->frame()->document();
//	Element* focusElement = static_cast<Element *> (document->focusedNode());
//
//   	NativeEditField* pNativeEdit = NativeEditField::getNativeEditFieldInstance();
//	ASSERT(pNativeEdit);
//    if(pNativeEdit->isActivated())
//    {
//		pNativeEdit->deactivate();
//    }
//
//    if(focusElement)
//  {
//       forbidEventDispatch();
//		document->page()->focusController()->setFocusedNode(0, document->frame());
//		focusElement->focus(true);
//    allowEventDispatch();
//}
//
//#endif //#ifdef USE_NATIVE_WIDGET_SUPPORT
}

// As we drop our decision to extend the height of frames of a frameset
// to maximum height the code below is commented out
#if 0
void ScrollView::setFrameSetHeight(const int& height, RenderObject *frameSet)
{
	frameSet->setHeight(height);

	RenderObject* child = frameSet->firstChild();
	while (child)
	{
		if (child->isFrameSet())
			setFrameSetHeight(height, child);
		//else
		child = child->nextSibling();
	}
}
#endif

int ScrollView::contentsX() const
{
    return scrollOffset().width();
}

int ScrollView::contentsY() const
{
    return scrollOffset().height();
}

int ScrollView::contentsWidth() const
{
    return m_data->contentsSize.width();
}

int ScrollView::contentsHeight() const
{
    return m_data->contentsSize.height();
}

IntPoint ScrollView::windowToContents(const IntPoint& windowPoint) const
{
    IntPoint viewPoint = convertFromContainingWindow(windowPoint);
    return viewPoint + scrollOffset();
}

IntPoint ScrollView::contentsToWindow(const IntPoint& contentsPoint) const
{
    IntPoint viewPoint = contentsPoint - scrollOffset();
    return convertToContainingWindow(viewPoint);
}


IntSize ScrollView::scrollOffset() const
{
    return m_data->scrollOffset;
}

void ScrollView::scrollOffset(IntSize newOffset)
{
	m_data->scrollOffset = newOffset;
}

bool ScrollView::areScrollbarsSuppressed()
{
	return m_data->scrollbarsSuppressed;
}

void ScrollView::scrollBy(int dx, int dy)
{
    IntSize scrollOffset = m_data->scrollOffset;
    IntSize newScrollOffset = scrollOffset + IntSize(dx, dy).shrunkTo(maximumScroll());
    newScrollOffset.clampNegativeToZero();

    if (newScrollOffset == scrollOffset)
        return;

	IntSize delta = newScrollOffset - scrollOffset;
	static_cast<FrameView *>(this)->frame()->page()->navigationInstance()->scrollBy(delta.width(), delta.height());
	
	updateScrollbars(newScrollOffset);
}

void ScrollView::scrollRectIntoViewRecursively(const IntRect& r)
{
    IntPoint p(max(0, r.x()), max(0, r.y()));
    ScrollView* view = this;
    while (view) {
        view->setContentsPos(p.x(), p.y());
        p.move(view->x() - view->scrollOffset().width(), view->y() - view->scrollOffset().height());
        view = static_cast<ScrollView*>(view->parent());
    }
}

WebCore::ScrollbarMode ScrollView::hScrollbarMode() const
{
    return m_data->hScrollbarMode;
}

WebCore::ScrollbarMode ScrollView::vScrollbarMode() const
{
    return m_data->vScrollbarMode;
}

void ScrollView::suppressScrollbars(bool suppressed, bool repaintOnSuppress)
{
    m_data->scrollbarsSuppressed = suppressed;
    if (repaintOnSuppress)
        updateScrollbars(m_data->scrollOffset);
}

void ScrollView::setHScrollbarMode(ScrollbarMode newMode)
{
    if (m_data->hScrollbarMode != newMode) {
        m_data->hScrollbarMode = newMode;
        updateScrollbars(m_data->scrollOffset);
    }
}

void ScrollView::setVScrollbarMode(ScrollbarMode newMode)
{
    if (m_data->vScrollbarMode != newMode) {
        m_data->vScrollbarMode = newMode;
        updateScrollbars(m_data->scrollOffset);
    }
}

void ScrollView::setScrollbarsMode(ScrollbarMode newMode)
{
    m_data->hScrollbarMode = m_data->vScrollbarMode = newMode;
    updateScrollbars(m_data->scrollOffset);
}

void ScrollView::setStaticBackground(bool flag)
{
    m_data->hasStaticBackground = flag;
}
#if PLATFORM (SHP) && defined(_SFB_BACKINGSTORE)
bool ScrollView::inWindow(IntRect rect) const 
{ 
	const FrameView* frameView = static_cast<const FrameView*>(this);
    Page* page = frameView->frame() ? frameView->frame()->page() : 0;
    if (!page)
        return false;
	IntRect viewportRect = page->chrome()->client()->viewportRect();
	if(viewportRect.intersects(rect))
		return true;
	else
		return false; 
} 
#else
bool ScrollView::inWindow() const 
{ 
    notImplemented();
    return true; 
}
#endif
void ScrollView::wheelEvent(PlatformWheelEvent& e)
{
    // Determine how much we want to scroll.  If we can move at all, we will accept the event.
    IntSize maxScrollDelta = maximumScroll();
    if ((e.deltaX() < 0 && maxScrollDelta.width() > 0) ||
        (e.deltaX() > 0 && scrollOffset().width() > 0) ||
        (e.deltaY() < 0 && maxScrollDelta.height() > 0) ||
        (e.deltaY() > 0 && scrollOffset().height() > 0))
        e.accept();

    scrollBy(e.deltaX() * LINE_STEP, e.deltaY() * LINE_STEP);
}


PlatformScrollbar* ScrollView::scrollbarUnderMouse(const PlatformMouseEvent& mouseEvent)
{
    IntPoint viewPoint = convertFromContainingWindow(mouseEvent.pos());
    if (m_data->hBar && m_data->hBar->frameGeometry().contains(viewPoint))
        return m_data->hBar.get();
    if (m_data->vBar && m_data->vBar->frameGeometry().contains(viewPoint))
        return m_data->vBar.get();
    return 0;
}

// 2008.07.29. bs0425.lee modified for touch scrollbar.

IntRect ScrollView::getScrollbarData(const IntPoint& mousePosition ) const
{
	PlatformScrollbar* c_Bar;
	IntRect c_thumbRect;

	if (m_data->hBar)
	{
		c_Bar=m_data->hBar.get();
		c_thumbRect=c_Bar->thumbRect();
		m_trackRect=c_Bar->trackRect();

		// extend the range of touch scroll. 
		m_trackRect.setY(m_trackRect.y()-extendRange);
		c_thumbRect.setY(c_thumbRect.y()-extendRange);

		m_trackRect.setHeight(m_trackRect.height()+extendRange);
		c_thumbRect.setHeight(c_thumbRect.height()+extendRange);

		if(c_thumbRect.contains(mousePosition))
			return c_thumbRect;
	}

    if (m_data->vBar)
	{
		c_Bar=m_data->vBar.get();
		c_thumbRect=c_Bar->thumbRect();
		m_trackRect=c_Bar->trackRect();

		m_trackRect.setX(m_trackRect.x()-extendRange);
		c_thumbRect.setX(c_thumbRect.x()-extendRange);
	
		m_trackRect.setWidth(m_trackRect.width()+extendRange);
		c_thumbRect.setWidth(c_thumbRect.width()+extendRange);

		if(c_thumbRect.contains(mousePosition))
              return c_thumbRect;
	}
	
    return IntRect(0, 0, 0, 0);    
}


IntRect ScrollView::getTrackRect() const
  {   
	  return m_trackRect;
 }
 // for touch scrollbar.

#if PLATFORM (SHP)
bool ScrollView::scroll(ScrollDirection direction, ScrollGranularity granularity, float multiplier /* = 1.0*/)
{
    if  ((direction == ScrollUp || direction == ScrollDown) && m_data->vBar)
        return m_data->vBar->scroll(direction, granularity, multiplier);
    else if (m_data->hBar)
        return m_data->hBar->scroll(direction, granularity, multiplier);

	return false;
}
#else
bool ScrollView::scroll(ScrollDirection direction, ScrollGranularity granularity)
{
    if  ((direction == ScrollUp || direction == ScrollDown) && m_data->vBar)
        return m_data->vBar->scroll(direction, granularity,  1.0);
    else if (m_data->hBar)
        return m_data->hBar->scroll(direction, granularity,  1.0);

	return false;
}
#endif

void ScrollView::addChild(Widget* child)
{
    child->setParent(this);
    m_data->children.add(child);
}

void ScrollView::removeChild(Widget* child)
{
    child->setParent(0);
    m_data->children.remove(child);
}

void ScrollView::updateContents(const IntRect& rect, bool now)
{

#ifndef SFB_PROGRESSIVE_RENDERING
	if (!m_isPageLoadComplete || rect.isEmpty())
        return;
#endif

    IntPoint windowPoint = contentsToWindow(rect.location());
    IntRect containingWindowRect = rect;
    containingWindowRect.setLocation(windowPoint);

	// Cache the dirty spot.
    addToDirtyRegion(containingWindowRect);

#ifdef _SFB_BACKINGSTORE
	if(false == inWindow(rect))
#else
	if(false == rect.intersects(frameGeometry()))
#endif
        return;

# if SFB_WIDGET
	// Fix B0100142435 amitg@samsung.com
	FrameView* view = static_cast<FrameView *>(this);
	/* Expose Event already sent for this Webcontrol instance */
	if (view->frame()->getExposeEvent())
		return;
#else
	if(SFBalIsExposeEventPresent())
		return;
#endif

	WmRect* wmRect = (WmRect*)SFBalMemAlloc( sizeof(WmRect) );
	wmRect->x = containingWindowRect.x();
	wmRect->y = containingWindowRect.y();
	wmRect->w = containingWindowRect.width();
	wmRect->h = containingWindowRect.height();



#ifdef MULTIPLE_INSTANCE_SUPPORT	//Pramod - Multiple instance support
#ifndef SFB_WIDGET
	// Fix B0100142435 amitg@samsung.com
	FrameView* view = static_cast<FrameView *>(this);
#endif
	void* pWebControlInstance = (void*)view->frame()->getWebControlInstance();
# ifdef SFB_WIDGET
	view->frame()->setExposeEvent(true);
# endif
	SFBalSendEvent(BAL_APP_BROWSER, BAL_EVENT_BROWSER_EXPOSE, wmRect, 0, 0, pWebControlInstance);
#else
	SFBalSendEvent(BAL_APP_BROWSER, BAL_EVENT_BROWSER_EXPOSE, wmRect, 0, 0);
#endif
	
}

void ScrollView::update()
{
#ifndef SFB_PROGRESSIVE_RENDERING
	if (!m_isPageLoadComplete)
        return;
#endif


	IntRect rect = frameGeometry();

	// Cache the dirty spot.
    addToDirtyRegion(rect);
# ifdef SFB_WIDGET
	// Fix B0100142435 amitg@samsung.com
	/* Expose Event already sent for this Webcontrol instance */
	FrameView* view = static_cast<FrameView *>(this);
	if (view->frame()->getExposeEvent())
		return;
# else
	if(SFBalIsExposeEventPresent())
		return;
# endif

	WmRect* wmRect = (WmRect*)SFBalMemAlloc( sizeof(WmRect) );
	wmRect->x = rect.x();
	wmRect->y = rect.y();
	wmRect->w = rect.width();
	wmRect->h = rect.height();


#ifdef MULTIPLE_INSTANCE_SUPPORT  //Pramod - Multiple instance support
#ifndef SFB_WIDGET
	// Fix B0100142435 amitg@samsung.com
	FrameView* view = static_cast<FrameView *>(this);
#endif
	void* pWebControlInstance = (void*)view->frame()->getWebControlInstance();
# ifdef SFB_WIDGET
	view->frame()->setExposeEvent(true);
# endif
	SFBalSendEvent(BAL_APP_BROWSER, BAL_EVENT_BROWSER_EXPOSE, wmRect, 0, 0, pWebControlInstance);
#else
	SFBalSendEvent(BAL_APP_BROWSER, BAL_EVENT_BROWSER_EXPOSE, wmRect, 0, 0);
#endif

}

void ScrollView::updateOnScroll()
{
#ifndef SFB_PROGRESSIVE_RENDERING
	if (!m_isPageLoadComplete)
        return;
#endif

	IntRect rect = frameGeometry();

	WmRect* wmRect = (WmRect*)SFBalMemAlloc( sizeof(WmRect) );
	wmRect->x = rect.x();
	wmRect->y = rect.y();
	wmRect->w = rect.width();
	wmRect->h = rect.height();

#if !defined(_SFB_BACKINGSTORE)
	// Cache the dirty spot.
    addToDirtyRegion(rect);
#endif

#ifdef MULTIPLE_INSTANCE_SUPPORT	//Pramod - Multiple instance support
	FrameView* view = static_cast<FrameView *>(this);
	void* pWebControlInstance = (void*)view->frame()->getWebControlInstance();
	SFBalSendEvent(BAL_APP_BROWSER, BAL_EVENT_BROWSER_SCROLL_EXPOSE, wmRect, 0, 0, pWebControlInstance);
#else
	SFBalSendEvent(BAL_APP_BROWSER, BAL_EVENT_BROWSER_SCROLL_EXPOSE, wmRect, 0, 0);
#endif
	
}

} // namespace WebCore
