/*
 * Copyright (C) 2006, 2007 Apple Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
 
#ifdef SUPPORT_NP_PLUGIN

#include "config.h"
#include "ShpJasmine.h"
#include "PluginDatabase.h"

#include "KURL.h"
#include "Frame.h"
#include "PluginPackage.h"
#include "NotImplemented.h"


namespace WebCore {

static PluginDatabase* pluginsDB = 0;

#ifdef SFB_GLOBAL_STATIC_HANDLING
void deInitPluginDatabaseShp()
{
    // delete the database once per browser instance
    // since we might be reading the list from the registry
    delete pluginsDB;
    pluginsDB = 0;
}
#endif

PluginDatabase* PluginDatabase::installedPlugins()
{
	PRINT("PG : PluginDatabase::installedPlugins ENTERED \n");

    if (!pluginsDB) {
		PRINT("PG : create new Plugin data base \n");
		pluginsDB = new PluginDatabase;
        pluginsDB->refresh();
    }
	PRINT("PG : PluginDatabase::installedPlugins EXITS \n");
    return pluginsDB;
}

bool PluginDatabase::refresh()
{
    PluginSet newPlugins;

    bool pluginSetChanged = false;
	PRINT("PG : PluginDatabase::refresh ENTERED \n");

    if (!m_plugins.isEmpty()) {
			
		// if the Hashset is not empty, no need to flush the hashset
		// and get the plugin data again, becoz in SHP we cannot 
		// install the plugins at runtime, not removing the below code, 
		// as it will be needed in future when SHP supports runtime installation
#if 1
		pluginSetChanged = false;
#else
        m_registeredMIMETypes.clear();

        PluginSet pluginsToUnload = m_plugins;

        PluginSet::const_iterator end = newPlugins.end();
        for (PluginSet::const_iterator it = newPlugins.begin(); it != end; ++it)
            pluginsToUnload.remove(*it);

        end = m_plugins.end();
        for (PluginSet::const_iterator it = m_plugins.begin(); it != end; ++it)
            newPlugins.remove(*it);

        // Unload plugins
        end = pluginsToUnload.end();
        for (PluginSet::const_iterator it = pluginsToUnload.begin(); it != end; ++it)
            m_plugins.remove(*it);

        // Add new plugins
        end = newPlugins.end();
        for (PluginSet::const_iterator it = newPlugins.begin(); it != end; ++it)
            m_plugins.add(*it);

        pluginSetChanged = !pluginsToUnload.isEmpty() || !newPlugins.isEmpty();
#endif

    } else {

		// Create a new set of plugins
		newPlugins = getPluginsInPaths();

        m_plugins = newPlugins;
        PluginSet::const_iterator end = newPlugins.end();
        for (PluginSet::const_iterator it = newPlugins.begin(); it != end; ++it)
            m_plugins.add(*it);

        pluginSetChanged = !newPlugins.isEmpty();

		// Register plug-in MIME types
		end = m_plugins.end();
		for (PluginSet::const_iterator it = m_plugins.begin(); it != end; ++it) {
			// Get MIME types
			MIMEToDescriptionsMap::const_iterator map_end = (*it)->mimeToDescriptions().end();
			for (MIMEToDescriptionsMap::const_iterator map_it = (*it)->mimeToDescriptions().begin(); map_it != map_end; ++map_it) {
				m_registeredMIMETypes.add(map_it->first);	//MIME types which have been extracted from DLLinfo struct in PluginPackage::fetchInfo
			}
		}
    }
	PRINT("PG : PluginDatabase::refresh EXITS \n");
	return pluginSetChanged;
}

Vector<PluginPackage*> PluginDatabase::plugins() const
{
    Vector<PluginPackage*> result;

    PluginSet::const_iterator end = m_plugins.end();
    for (PluginSet::const_iterator it = m_plugins.begin(); it != end; ++it)
        result.append((*it).get());

    return result;
}

PluginSet PluginDatabase::getPluginsInPaths() const
{
    // FIXME: This should be a case insensitive set.
    PluginSet plugins;

	PRINT("PG : PluginDatabase::getPluginsInPaths ENTERED\n");
	NPGluePluginDLLDirectory* pluginDirectory = SFB_GetPluginDLLDirectory();
	if (!pluginDirectory)
	{
		PRINT("PG : PluginDatabase::getPluginsInPaths NPGLUE API returned NULL \n");
		// return ERROR
	}
	else
	{
		// PG : the number of PLUGINs info is in the pluginDirectory MACRO should not be used
		for(int i = 0; i < pluginDirectory->numOfDLLs; i++)
		{
			// PG : we should use Calloc, so that no need to initialize with zero
			
			pluginDirectory->DLLs[i]->SetPluginStringInfo();
			
			NPGluePluginDLLInfo* pluginDLLInfo = (NPGluePluginDLLInfo*)fastCalloc(1, sizeof(NPGluePluginDLLInfo));
			if(!pluginDLLInfo)
			{
				PRINT("PG : PluginDatabase::getPluginsInPaths NO MEMORY for pluginDLLInfo\n");
				// return error;
			}
			else
			{
				pluginDLLInfo->szName = (char *)malloc(pluginDirectory->DLLs[i]->iNameStringSize);
				if(pluginDLLInfo->szName == NULL)
				{
					PRINT("fastCalloc FAILED in PluginDatabase::getPluginsInPaths  pluginDLLInfo->szName\n");
					break;
				}
				pluginDLLInfo->szDescription = (char *)malloc(pluginDirectory->DLLs[i]->iDescriptionStringSize);
				if(pluginDLLInfo->szDescription == NULL)
				{
					PRINT("fastCalloc FAILED in PluginDatabase::getPluginsInPaths  pluginDLLInfo->szDescription\n");
					break;
				}
				pluginDLLInfo->szMimeTypes = (char *)malloc(pluginDirectory->DLLs[i]->iMimeTypesStringSize);
				if(pluginDLLInfo->szMimeTypes == NULL)
				{
					PRINT("fastCalloc FAILED in PluginDatabase::getPluginsInPaths  pluginDLLInfo->szMimeTypes\n");
					break;
				}
				pluginDLLInfo->szExtentions = (char *)malloc(pluginDirectory->DLLs[i]->iExtentionsStringSize);
				if(pluginDLLInfo->szExtentions == NULL)
				{
					PRINT("fastCalloc FAILED in PluginDatabase::getPluginsInPaths  pluginDLLInfo->szExtentions\n");
					break;
				}

				pluginDirectory->DLLs[i]->GetPluginDLLInfo(pluginDLLInfo);

				PluginPackage* pluginPackage = PluginPackage::createPackage(pluginDLLInfo);
				if (pluginPackage)
					plugins.add(pluginPackage);
			}
		}
	}

	PRINT("PG : PluginDatabase::getPluginsInPaths EXITS\n");
	return plugins;
}

static inline Vector<int> parseVersionString(const String& versionString)
{
   Vector<int> ret;
	notImplemented();
	return ret;
}

// This returns whether versionA is higher than versionB
static inline bool compareVersions(const Vector<int>& versionA, const Vector<int>& versionB)
{
    notImplemented();
    // If we come here, the versions are either the same or versionB has an extra component, just return false
    return false;
}

bool PluginDatabase::isMIMETypeRegistered(const String& mimeType)
{
	PRINT("PG : PluginDatabase::isMIMETypeRegistered ENTERED\n");

    if (mimeType.isNull())
        return false;
    if (m_registeredMIMETypes.contains(mimeType))
        return true;
    // No plugin was found, try refreshing the database and searching again
	PRINT("PG : PluginDatabase::isMIMETypeRegistered EXITS\n");
	return (refresh() && m_registeredMIMETypes.contains(mimeType));
}

PluginPackage* PluginDatabase::pluginForMIMEType(const String& mimeType)
{
	PRINT("PG : PluginDatabase::pluginForMIMEType ENTERED\n");

    if (mimeType.isEmpty())
        return 0;

    String key = mimeType.lower();
    PluginPackage* plugin = 0;
    PluginSet::const_iterator end = m_plugins.end();

    for (PluginSet::const_iterator it = m_plugins.begin(); it != end; ++it)
        if ((*it)->mimeToDescriptions().contains(key))
            plugin = (*it).get();
	PRINT("PG : PluginDatabase::pluginForMIMEType EXITS\n");
	return plugin;
}

String PluginDatabase::MIMETypeForExtension(const String& extension) const
{
	PRINT("PG : PluginDatabase::MIMETypeForExtension ENTERED\n");
	if (extension.isEmpty())
        return String();

    PluginSet::const_iterator end = m_plugins.end();
    String mimeType;
    PluginPackage* plugin = 0;

    for (PluginSet::const_iterator it = m_plugins.begin(); it != end; ++it) {
        MIMEToExtensionsMap::const_iterator mime_end = (*it)->mimeToExtensions().end();

        for (MIMEToExtensionsMap::const_iterator mime_it = (*it)->mimeToExtensions().begin(); mime_it != mime_end; ++mime_it) {
            const Vector<String>& extensions = mime_it->second;
            for (unsigned i = 0; i < extensions.size(); i++) {
                if (equalIgnoringCase(extensions[i], extension)) {
                    mimeType = mime_it->first;
                    plugin = (*it).get();                    
                }
            }
        }
    }

	PRINT("PG : PluginDatabase::MIMETypeForExtension EXITS\n");
    return mimeType;
	
}

PluginPackage* PluginDatabase::findPlugin(const KURL& url, String& mimeType)
{
	PRINT("PG : PluginDatabase::findPlugin ENTERED\n");
	PluginPackage* plugin = pluginForMIMEType(mimeType);

	if (!plugin) {
        String filename = url.lastPathComponent();
        if (!filename.endsWith("/")) {
            int extensionPos = filename.reverseFind('.');
            if (extensionPos != -1) {
                String extension = filename.substring(extensionPos + 1);

                mimeType = MIMETypeForExtension(extension);
                plugin = pluginForMIMEType(mimeType);
            }
        }
	}

	// FIXME: if no plugin could be found, query Windows for the mime type
    // corresponding to the extension.
	PRINT("PG : PluginDatabase::findPlugin EXITS\n");

    return plugin;
}

}
#endif //SUPPORT_NP_PLUGIN 

