/*
 * Copyright (C) 2006, 2007 Apple Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
#ifdef SUPPORT_NP_PLUGIN

#include "config.h"
#include "PluginView.h"

#include "Document.h"
#include "DocumentLoader.h"
#include "Element.h"
#include "EventNames.h"
#include "FrameLoader.h"
#include "FrameLoadRequest.h"
#include "FrameTree.h"
#include "Frame.h"
#include "FrameView.h"
#include "GraphicsContext.h"
#include "GWES.h"
#include "Image.h"
#include "HTMLNames.h"
#include "HTMLPlugInElement.h"
#include "KeyboardEvent.h"
#include "MIMETypeRegistry.h"
#include "MouseEvent.h"
#include "NotImplemented.h"
#include "Page.h"
#include "FocusController.h"
#include "PlatformMouseEvent.h"
#include "PluginPackage.h"
#include "kjs_binding.h"
#include "kjs_proxy.h"
#include "kjs_window.h"
#include "PluginDatabase.h"
#include "PluginDebug.h"
#include "PluginPackage.h"
#include "npruntime_impl.h"
#include "runtime_root.h"
#include "Settings.h"
#include <kjs/JSLock.h>
#include <kjs/value.h>
#include <wtf/ASCIICType.h>
//for Browser expose event
#include "ShpJasmine.h"
//to access isLoadingComplete
#include "FrameLoaderClient.h"
#include "MemCore.h"

#include "ChromeClient.h"
#if PLATFORM(SHP)
#include "CMMDefs.h"
#endif // PLATFORM(SHP)

using KJS::ExecState;
using KJS::JSLock;
using KJS::JSObject;
using KJS::JSValue;
using KJS::UString;
using KJS::Window;

using std::min;

using namespace WTF;
namespace WebCore {

using namespace EventNames;
using namespace HTMLNames;

class PluginRequest
CMM_MODIFIED
{
public:
    PluginRequest(const FrameLoadRequest& frameLoadRequest, bool sendNotification, void* notifyData, bool shouldAllowPopups)
        : m_frameLoadRequest(frameLoadRequest)
        , m_notifyData(notifyData)
        , m_sendNotification(sendNotification)
        , m_shouldAllowPopups(shouldAllowPopups) { }
public:
    const FrameLoadRequest& frameLoadRequest() const { return m_frameLoadRequest; }
    void* notifyData() const { return m_notifyData; }
    bool sendNotification() const { return m_sendNotification; }
    bool shouldAllowPopups() const { return m_shouldAllowPopups; }
private:
    FrameLoadRequest m_frameLoadRequest;
    void* m_notifyData;
    bool m_sendNotification;
    bool m_shouldAllowPopups;
};

static const double MessageThrottleTimeInterval = 0.001;
static int s_callingPlugin;

class PluginMessageThrottlerWin
CMM_MODIFIED
{
public:
    PluginMessageThrottlerWin(PluginView* pluginView)
        : m_back(0), m_front(0)
        , m_pluginView(pluginView)
        , m_messageThrottleTimer(this, &PluginMessageThrottlerWin::messageThrottleTimerFired)
    {
        // Initialize the free list with our inline messages
        for (unsigned i = 0; i < NumInlineMessages - 1; i++)
            m_inlineMessages[i].next = &m_inlineMessages[i + 1];
        m_inlineMessages[NumInlineMessages - 1].next = 0;
        m_freeInlineMessages = &m_inlineMessages[0];
    }

    ~PluginMessageThrottlerWin()
    {
        PluginMessage* next;

        for (PluginMessage* message = m_front; message; message = next) {
            next = message->next;
            freeMessage(message);
        }
    }

private:
    struct PluginMessage
    CMM_MODIFIED
    {

        UINT msg;

        struct PluginMessage* next;
    };

    void messageThrottleTimerFired(Timer<PluginMessageThrottlerWin>*)
    {
        PluginMessage* message = m_front;
        m_front = m_front->next;
        if (message == m_back)
            m_back = 0;

       freeMessage(message);

        if (m_front)
            m_messageThrottleTimer.startOneShot(MessageThrottleTimeInterval);


    }

    PluginMessage* allocateMessage()
    {
        PluginMessage *message;

        if (m_freeInlineMessages) {
            message = m_freeInlineMessages;
            m_freeInlineMessages = message->next;
        } else
            message = new PluginMessage;

        return message;
    }

    bool isInlineMessage(PluginMessage* message)
    {
        return message >= &m_inlineMessages[0] && message <= &m_inlineMessages[NumInlineMessages - 1];
    }

    void freeMessage(PluginMessage* message)
    {
        if (isInlineMessage(message)) {
            message->next = m_freeInlineMessages;
            m_freeInlineMessages = message;
        } else
            delete message;
    }

    PluginView* m_pluginView;
    PluginMessage* m_back;
    PluginMessage* m_front;

    static const int NumInlineMessages = 4;
    PluginMessage m_inlineMessages[NumInlineMessages];
    PluginMessage* m_freeInlineMessages;

    Timer<PluginMessageThrottlerWin> m_messageThrottleTimer;
};

static String scriptStringIfJavaScriptURL(const KURL& url)
{
    if (!url.string().startsWith("javascript:", false))
        return String();

    // This returns an unescaped string
    return KURL::decode_string(url.deprecatedString().mid(11));
}

PluginView* PluginView::s_currentPluginView = 0;

#ifdef SFB_GLOBAL_STATIC_HANDLING
void initPluginView()
{
    s_callingPlugin = 0;
    PluginView::initStatics();
}
#endif


static bool registerPluginView()
{
	notImplemented();
	return true;
}



void PluginView::popPopupsStateTimerFired(Timer<PluginView>*)
{
	PRINT("PG :  PluginView::popPopupsStateTimerFired ENTERED \n");
    popPopupsEnabledState();
	PRINT("PG :  PluginView::popPopupsStateTimerFired EXITS \n");
}


static bool isWindowsMessageUserGesture(UINT message)
{
	notImplemented();
	return false;
}


void PluginView::updateWindow() const
{
	PRINT("PG :  PluginView::updateWindow ENTERED \n");

	if(parent())
	{
#if defined(_SFB_BACKINGSTORE)
		//This check is already there in the scrollview::updatecontents
		IntRect rect = frameGeometry();
		//calclute the position recursively for inner frame (frame or iframe)
		FrameView* parentView = static_cast<FrameView*>(m_parentFrame->view());
		rect = IntRect(parentView->contentsToWindow(frameGeometry().location()), frameGeometry().size());
		int y = rect.y();
		int x = rect.x();

		while(parentView)
		{
			y += parentView->frameGeometry().y();
			x += parentView->frameGeometry().x();

			parentView = static_cast<FrameView*>(parentView->parent());
		}

		rect.setX(x);
		rect.setY(y);

		parent()->updateContents(rect, TRUE);
#else
		// TODO : now the scrollbars are drawn by the engine, 
		// check if the viewport rect takes care of that
		FloatRect	fRect		= static_cast<FrameView*>(parent())->visibleContentRectConsideringExternalScrollers();
		IntRect		viewPortRect;
	
		viewPortRect.setX(fRect.x());
		viewPortRect.setY(fRect.y());
		viewPortRect.setHeight(fRect.height());
		viewPortRect.setWidth(fRect.width());

		//check if the plugin is in the visible viewport
		if(pluginRect.intersects(viewPortRect))
		{
			pluginRect.intersect(viewPortRect);
			parent()->updateContents(pluginRect, TRUE);
		}
#endif
    }
	PRINT("PG :  PluginView::updateWindow EXITS \n");
}

IntRect PluginView::windowClipRect() const
{
	PRINT("PG :  PluginView::windowClipRect ENTERED \n");
	// TODO : use this function to pass the clipping rect to the plugin
    // Start by clipping to our bounds.
    IntRect clipRect(m_windowRect);
    
    // Take our element and get the clip rect from the enclosing layer and frame view.
    RenderLayer* layer = m_element->renderer()->enclosingLayer();
    FrameView* parentView = m_element->document()->view();
    clipRect.intersect(parentView->windowClipRectForLayer(layer, true));

    return clipRect;
}

void PluginView::setFrameGeometry(const IntRect& rect)
{
	PRINT("PG :  PluginView::setFrameGeometry ENTERED \n");

#if !PLATFORM(SHP)
    if (m_element->document()->printing())
        return;
#endif

    if (rect != frameGeometry())
        Widget::setFrameGeometry(rect);

    //geometryChanged();
	PRINT("PG :  PluginView::setFrameGeometry EXITS \n");
}

void PluginView::setPlatformGC() const
{
	if(!m_isStarted)
		return;
	if (Page* page = m_parentFrame->page())
	{
		int hWin;
		int hGC;
		page->chrome()->client()->getWindowHandles(&hWin, &hGC);
		if(m_GCHandle != hGC)
		{
			NPEvent npEvent;
			npEvent.wParam = static_cast<uint32>(hGC);
			npEvent.lParam = 0;
			npEvent.event  = PLUGIN_UI_EVENT_SET_GC;

			KJS::JSLock::DropAllLocks dropAllLocks;
			setCallingPlugin(true);
			bool result = m_plugin->pluginFuncs()->event(m_instance, &npEvent);
			setCallingPlugin(false);
			m_GCHandle = hGC;
		}
	}
}

void PluginView::geometryChanged() const
{
	PRINT("PG :  PluginView::geometryChanged ENTERED \n");
	if(!m_isStarted )
		return;
	if (!m_plugin || !m_plugin->pluginFuncs()->getvalue || !m_plugin->pluginFuncs()->event)
		return;
	setPlatformGC();
	
	if (!parent())
        return;
    ASSERT(parent()->isFrameView());
	FrameView* frameView = static_cast<FrameView*>(parent());
	BOOL isVersionSupported = FALSE;
	BOOL isCreated = FALSE; 

	setCallingPlugin(true);
	m_plugin->pluginFuncs()->getvalue(m_instance, NPPVisVersionSupported, &isVersionSupported);
	setCallingPlugin(false);

	if(isVersionSupported == FALSE)
		return;

#ifdef SFB_GWES_TRANSFORM
		IntRect rect(frameGeometry());

		rect = IntRect(frameView->contentsToWindow(frameGeometry().location()), frameGeometry().size());

		float zoomFactor = m_parentFrame->page()->chrome()->client()->getZoomFactor();
		int frameRectX = m_parentFrame->view()->contentsX();
		int frameRectY = m_parentFrame->view()->contentsY();
		IntRect backingStoreRect = m_parentFrame->page()->chrome()->client()->backingStoreRect();
		IntRect oldWindowRect = m_scalewindowRect;
	
		//calclute the position recursively for inner frame (frame or iframe)
		FrameView* parentView = static_cast<FrameView*>(m_parentFrame->view());
		int y = rect.y();
		int x = rect.x();

		while(parentView)
		{
			y += parentView->frameGeometry().y();
			x += parentView->frameGeometry().x();

			parentView = static_cast<FrameView*>(parentView->parent());
		}

		rect.setX(x);
		rect.setY(y);
		m_scalewindowRect.setX((rect.x() - (backingStoreRect.x() - frameRectX)) * zoomFactor);
		m_scalewindowRect.setY((rect.y() - (backingStoreRect.y() - frameRectY)) * zoomFactor);
		m_scalewindowRect.setWidth(rect.width() * zoomFactor);
		m_scalewindowRect.setHeight(rect.height() * zoomFactor);

		// both APIs check if the new and old values are same
		relocatePluginWindow(m_scalewindowRect.location(), oldWindowRect.location());
		if(m_scalewindowRect.size() != oldWindowRect.size())
		{
			IntSize clippedSize = m_scalewindowRect.size().shrunkTo(m_maxPluginWindowSize);
			if(m_optimumSizeSaved.isEmpty())
			{
				m_optimumSizeSaved = m_optimumSizeSaved.expandedTo(clippedSize);
			}
			else
			{
				// optimum Size will have the last working size
				m_optimumSizeSaved = m_optimumSizeSaved.expandedTo(clippedSize);
				m_optimumSizeSaved = m_optimumSizeSaved.shrunkTo(oldWindowRect.size());
			}
			resizePluginWindow( m_scalewindowRect.size(), oldWindowRect.size()); 
		}
#else
    if (!m_plugin->pluginFuncs()->event)
        return ;
    IntRect oldWindowRect = m_windowRect;

    m_windowRect = IntRect(frameView->contentsToWindow(frameGeometry().location()), frameGeometry().size());
	if(oldWindowRect != m_windowRect)
	{
		NPEvent npEvent;
		NPWindow windowpos;
		memset(&windowpos, 0, sizeof(windowpos));

		windowpos.x		 = m_windowRect.x();
		windowpos.y		 = m_windowRect.y();
		windowpos.width  = m_windowRect.width();
		windowpos.height = m_windowRect.height();

		npEvent.event	 = PLUGIN_UI_EVENT_WINPOS_CHANGED;
		npEvent.lParam   = reinterpret_cast<uint32>(&windowpos);
		KJS::JSLock::DropAllLocks dropAllLocks;
		setCallingPlugin(true);
		bool result = m_plugin->pluginFuncs()->event(m_instance, &npEvent);
		setCallingPlugin(false);
	}	
#endif


	PRINT("PG :  PluginView::geometryChanged EXITS \n");
}

void PluginView::setFocus()
{
	PRINT("PG :  PluginView::setFocus ENTERED \n");

	// <TODO> : set focus to platform window in case of windowed plugin
    Widget::setFocus();
	PRINT("PG :  PluginView::setFocus EXITS \n");

}

void PluginView::show()
{
	PRINT("PG :  PluginView::show ENTERED \n");
    m_isVisible = true;

	// <TODO> : call shp's showWindow

    Widget::show();
	PRINT("PG :  PluginView::show EXITS \n");
}
//TODO : check if this is needed for windowless plugins
void PluginView::hide()
{
	PRINT("PG :  PluginView::hide EXITS \n");

	m_isVisible = false;
    Widget::hide();
	PRINT("PG :  PluginView::hide EXITS \n");
}

void PluginView::paintMissingPluginIcon(const char* name, GraphicsContext* context, const IntRect& rect)
{
	PRINT("PG :  PluginView::paintMissingPluginIcon ENTERED \n");
	
    if(!Image::LoadAndPaintPlatformResource(name, context, rect))
	{
		PRINT("PG :  PluginView::paintMissingPluginIcon DRAW bitmap failed \n");
	}
	PRINT("PG :  PluginView::paintMissingPluginIcon EXITS \n");
}

bool PluginView::dispatchNPEvent(NPEvent& npEvent)
{
	PRINT("PG :  PluginView::dispatchNPEvent ENTERED \n");
    if (!m_plugin->pluginFuncs()->event)
        return false;

    bool shouldPop = false;

    if (m_plugin->pluginFuncs()->version < NPVERS_HAS_POPUPS_ENABLED_STATE && isWindowsMessageUserGesture(npEvent.event)) {
        pushPopupsEnabledState(true);
        shouldPop = true;
    }

    KJS::JSLock::DropAllLocks dropAllLocks;
    setCallingPlugin(true);
    bool result = m_plugin->pluginFuncs()->event(m_instance, &npEvent);
    setCallingPlugin(false);

    if (shouldPop)
        popPopupsEnabledState();

	PRINT("PG :  PluginView::dispatchNPEvent EXITS \n");
    return result;
}

void PluginView::paint(GraphicsContext* context, const IntRect& _rect)
{
	PRINT("PG :  PluginView::paint ENTERED \n");

    if (context->paintingDisabled() || !parent())
        return;

	ASSERT(parent()->isFrameView());
	
    // paint missing plugin untill all the streams have finished downloading
	if ((!m_isStarted) || (!m_streams.isEmpty()))
	{
		// do not draw the missig plugin icon.. while loading
		return;
    }
	setPlatformGC();
	IntRect oldWindowRect = m_scalewindowRect; 
	m_scalewindowRect = context->getCTM().mapRect(frameGeometry());

	if(m_scalewindowRect.size() != oldWindowRect.size())
	{
		IntSize clippedSize = m_scalewindowRect.size().shrunkTo(m_maxPluginWindowSize);
		if(m_optimumSizeSaved.isEmpty())
		{
			m_optimumSizeSaved = m_optimumSizeSaved.expandedTo(clippedSize);
		}
		else
		{
			// optimum Size will have the last working size
			m_optimumSizeSaved = m_optimumSizeSaved.expandedTo(clippedSize);
			m_optimumSizeSaved = m_optimumSizeSaved.shrunkTo(oldWindowRect.size());
		}
	}

	BOOL isVersionSupported = FALSE;
	BOOL isCreated = FALSE; 

	setCallingPlugin(true);
	if (m_plugin || m_plugin->pluginFuncs()->getvalue)
	{
		m_plugin->pluginFuncs()->getvalue(m_instance, NPPVisVersionSupported, &isVersionSupported);
		m_plugin->pluginFuncs()->getvalue(m_instance, NPPVisCreated, &isCreated);
	}
	setCallingPlugin(false);

	// create the plugin only if the version is supported
	if(isVersionSupported == TRUE)
	{
		relocatePluginWindow(m_scalewindowRect.location(), oldWindowRect.location());
		if(isCreated == FALSE )
		{
			if(TRUE == resizePluginWindow( m_scalewindowRect.size(), oldWindowRect.size()))
			{
#ifdef SFB_GWES_TRANSFORM
				NPEvent npEvent;
				npEvent.event  = PLUGIN_UI_EVENT_CREATE;
				npEvent.wParam = static_cast<uint32>(m_GCHandle);	
				npEvent.lParam = m_protoAccountHandle;
				if(FALSE == dispatchNPEvent(npEvent))
				{
					// Destroy the plugin if creation fails
					NPSavedData* savedData = 0;
					setCallingPlugin(true);
					NPError npErr = m_plugin->pluginFuncs()->destroy(m_instance, &savedData);
					setCallingPlugin(false);
					LOG_NPERROR(npErr);
					// also draw the missing icon
					IntRect dst = frameGeometry();
					paintMissingPluginIcon("missingPlugin", context, dst);
				}				
#else
				geometryChanged();
				NPEvent npEvent;
				npEvent.event  = PLUGIN_UI_EVENT_CREATE;
				PlatformGraphicsContext* hGC = context->platformContext();
				npEvent.wParam = static_cast<uint32>(*hGC);
				npEvent.lParam = m_protoAccountHandle;
				if(FALSE == dispatchNPEvent(npEvent))
				{
				    // Destroy the plugin if creation fails
					NPSavedData* savedData = 0;
					setCallingPlugin(true);
					NPError npErr = m_plugin->pluginFuncs()->destroy(m_instance, &savedData);
					setCallingPlugin(false);
					LOG_NPERROR(npErr);
				}
#endif
			}
			else // not enough memory to create plugin show missung plugin , we are no destroying coz, it can be created once zoomed out
			{
				IntRect dst = frameGeometry();
				paintMissingPluginIcon("missingPlugin", context, dst);
				return;	
			}
		}
		else
		{
#ifdef SFB_GWES_TRANSFORM
			if(TRUE == resizePluginWindow( m_scalewindowRect.size(), oldWindowRect.size()))
			{
				NPEvent npEvent;
				npEvent.event  = PLUGIN_UI_EVENT_PAINT;
				dispatchNPEvent(npEvent);
			}
#else
			geometryChanged();
			NPEvent npEvent;
			npEvent.event  = PLUGIN_UI_EVENT_PAINT;
			dispatchNPEvent(npEvent);
#endif
		}
	}
	else // flash version not supported or creation failed
	{
		// DRAW THE MISSING PLUGIN IMAGE
		IntRect dst = frameGeometry();
        paintMissingPluginIcon("missingPlugin", context, dst);
	}
	PRINT("PG :  PluginView::paint EXITS \n");
}

//Checking available memory 
BOOL PluginView::CheckAvailableMemory(int height, int width, int HGC) const
{
	int maxsizeRequired = 0, datasize = 0, availableFreechunk = 0;
	//TODO : varify GC
	int bytesperpixel = ((WmGetGCPointer(HGC))->gcAttrib.bitsPerPixel)/8;
	BOOL result = TRUE;
	availableFreechunk = SFBalMemGetStatEx();
	datasize = height * width * bytesperpixel;

	if(availableFreechunk <= datasize)
		result = FALSE;
	else
		if ((CMM_CURRENTLY_ALLOCATED_HEAP_SIZE + datasize) > (0.85 * CMM_TOTAL_HEAP_SIZE))
			result = FALSE;
	return result;
}

void PluginView::handleKeyboardEvent(KeyboardEvent* event)
{
	PRINT("PG :  PluginView::handleKeyboardEvent ENTEREED \n");
    notImplemented();
	PRINT("PG :  PluginView::handleKeyboardEvent EXITS \n");
}

void PluginView::handleMouseEvent(MouseEvent* event)
{
	PRINT("PG :  PluginView::handleMouseEvent ENTERED \n");

    // Currently, Widget::setCursor is always called after this function in EventHandler.cpp
    // and since we don't want that we set ignoreNextSetCursor to true here to prevent that.

	if(!m_isStarted || !m_isVisible || !parent())
		return;
    IntPoint p = static_cast<FrameView*>(parent())->contentsToWindow(IntPoint(event->pageX(), event->pageY()));
	NPWindow windowpos;
   	NPEvent npEvent;
    memset(&windowpos, 0, sizeof(windowpos));

#if defined(SFB_GWES_TRANSFORM)
		//TODO : we do not have the context here so can not calculate the scaling
		
		float zoomFactor = m_parentFrame->page()->chrome()->client()->getZoomFactor();

		int frameRectX = m_parentFrame->view()->contentsX();
		int frameRectY = m_parentFrame->view()->contentsY();

		IntRect backingStoreRect = m_parentFrame->page()->chrome()->client()->backingStoreRect();

		/*backingStoreRect.move(frameRectX - backingStoreRect.x(), frameRectY - backingStoreRect.y());*/

		//calclute the position recursively for inner frame (frame or iframe)
		FrameView* parentView = static_cast<FrameView*>(m_parentFrame->view());
		int y = p.y();
		int x = p.x();

		while(parentView)
		{
			y += parentView->frameGeometry().y();
			x += parentView->frameGeometry().x();

			parentView = static_cast<FrameView*>(parentView->parent());
		}

		p.setX(x);
		p.setY(y);

		windowpos.x = (p.x() - (backingStoreRect.x() - frameRectX)) * zoomFactor;
		windowpos.y = (p.y() - (backingStoreRect.y() - frameRectY)) * zoomFactor;
#else
    windowpos.x		 = p.x();
    windowpos.y		 = p.y();
#endif
	
	npEvent.lParam   = reinterpret_cast<uint32>(&windowpos);
    npEvent.wParam   = 0;
	npEvent.event	 = PLUGIN_UI_EVENT_MOUSE;

	if(mousemoveEvent == event->type())
	{
		npEvent.wParam 
		= static_cast<uint32>(PLUGIN_UI_EVENT_MOUSE_MOVE);
    }
	else 
		if(mouseoverEvent == event->type())
		{
			npEvent.wParam 
			= static_cast<uint32>(PLUGIN_UI_EVENT_MOUSE_OVER);
		}
		else 
			if(mousedownEvent == event->type())
			{
				// Focus the plugin
				if (Page* page = m_parentFrame->page())
					page->focusController()->setFocusedFrame(m_parentFrame);
				m_parentFrame->document()->setFocusedNode(m_element);
				npEvent.wParam   
				= static_cast<uint32>(PLUGIN_UI_EVENT_MOUSE_DOWN);
			}
			else 
				if(mouseupEvent == event->type())
				{
					npEvent.wParam   
					= static_cast<uint32>(PLUGIN_UI_EVENT_MOUSE_UP);
				}
				else 
					if(mouseoutEvent == event->type())
					{
						npEvent.wParam   
						= static_cast<uint32>(PLUGIN_UI_EVENT_MOUSE_OUT);		
					}
					else 
						if(clickEvent == event->type())
						{
							npEvent.wParam 
							= static_cast<uint32>(PLUGIN_UI_EVENT_MOUSE_CLICK);		
						}
						else
						{
							return;
						}

    KJS::JSLock::DropAllLocks;
    if (!dispatchNPEvent(npEvent))
        event->setDefaultHandled();
	PRINT("PG :  PluginView::handleMouseEvent EXITS \n");

}

void PluginView::handleEvent(Event* event)
{
	PRINT("PG :  PluginView::handleEvent ENTERED \n");

    if (!m_plugin)
        return;

    if (event->isMouseEvent())
        handleMouseEvent(static_cast<MouseEvent*>(event));
    else if (event->isKeyboardEvent())
        handleKeyboardEvent(static_cast<KeyboardEvent*>(event));

	PRINT("PG :  PluginView::handleEvent EXITS \n");
}

void PluginView::setParent(ScrollView* parent)
{
	PRINT("PG :  PluginView::setParent ENTERED \n");

    Widget::setParent(parent);

    if (parent)
        init();
    else {
        // If the plug-in window or one of its children have the focus, we need to
        // clear it to prevent the web view window from being focused because that can
        // trigger a layout while the plugin element is being detached.
       // <TODO> : set the focus
    }
	PRINT("PG :  PluginView::setParent EXITS \n");
}

void PluginView::attachToWindow()
{
	PRINT("PG :  PluginView::attachToWindow ENTERED \n");
    if (m_attachedToWindow)
        return;

    m_attachedToWindow = true;
    // <TODO> : call show window of SHP
	PRINT("PG :  PluginView::attachToWindow EXITS \n");
}

void PluginView::detachFromWindow()
{
	PRINT("PG :  PluginView::detachFromWindow ENTERED \n");
    if (!m_attachedToWindow)
        return;

	// <TODO> : call hide window of SHP

	m_attachedToWindow = false;
	PRINT("PG :  PluginView::detachFromWindow EXITS \n");
}

void PluginView::setNPWindowRect(const IntRect& rect)
{
	PRINT("PG :  PluginView::setNPWindowRect ENTERED \n");
    if (!m_isStarted )
        return;

	IntRect WindowRect = IntRect(static_cast<FrameView*>(parent())->contentsToWindow(rect.location()), rect.size());

	m_npWindow.type		= NPWindowTypeDrawable;
	m_npWindow.x		= WindowRect.x();
	m_npWindow.y		= WindowRect.y();
    m_npWindow.width	= WindowRect.width();
    m_npWindow.height	= WindowRect.height();
	//TODO : clip rect is to be calculated
    m_npWindow.clipRect.left	= 0;	//x
    m_npWindow.clipRect.top		= 0;	//y
    m_npWindow.clipRect.right	= WindowRect.width(); //w
    m_npWindow.clipRect.bottom	= WindowRect.height(); // h

	if (m_plugin->pluginFuncs()->setwindow) {
        KJS::JSLock::DropAllLocks dropAllLocks;
        setCallingPlugin(true);
        m_plugin->pluginFuncs()->setwindow(m_instance, &m_npWindow);
        setCallingPlugin(false);
    }
	PRINT("PG :  PluginView::setNPWindowRect EXITS \n");
}

bool PluginView::start()
{
	PRINT("PG :  PluginView::start ENTERED \n");
    if (m_isStarted)
        return false;

    ASSERT(m_plugin);
    ASSERT(m_plugin->pluginFuncs()->newp);
	ASSERT(m_plugin->pluginFuncs()->getvalue);

    NPError npErr, npErr1;
	NPWindow window;
	memset(&window, 0, sizeof(window));
	
    PluginView::setCurrentPluginView(this);
    {
        KJS::JSLock::DropAllLocks dropAllLocks;
        setCallingPlugin(true);
		//PG <TBD> : pass the TAG information through argc and argv
		PRINT("PG :  PluginView::start NPP_New called \n");

        npErr = m_plugin->pluginFuncs()->newp((NPMIMEType)m_mimeType.data(), m_instance, m_mode, m_paramCount, m_paramNames, m_paramValues, NULL);
		npErr1 = m_plugin->pluginFuncs()->getvalue(m_instance, NPPVpluginWindowSize, &window);
        setCallingPlugin(false);
        LOG_NPERROR(npErr);
    }
    PluginView::setCurrentPluginView(0);

    if ((npErr != NPERR_NO_ERROR) || (npErr1 != NPERR_NO_ERROR))
        return false;
	m_maxPluginWindowSize.setWidth(window.width);
	m_maxPluginWindowSize.setHeight(window.height);

    m_isStarted = true;

	PRINT("PG :  PluginView::start EXITS \n");
    return true;
}

void PluginView::stop()
{
	PRINT("PG :  PluginView::stop ENTERED \n");
    if (!m_isStarted)
        return;

    HashSet<RefPtr<PluginStream> > streams = m_streams;
    HashSet<RefPtr<PluginStream> >::iterator end = streams.end();
    for (HashSet<RefPtr<PluginStream> >::iterator it = streams.begin(); it != end; ++it) {
        (*it)->stop();
        disconnectStream((*it).get());
    }

    ASSERT(m_streams.isEmpty());

    m_isStarted = false;


    KJS::JSLock::DropAllLocks;

    // Clear the window
    m_npWindow.window = 0;
    if (m_plugin->pluginFuncs()->setwindow && !m_quirks.contains(PluginQuirkDontSetNullWindowHandleOnDestroy)) {
        setCallingPlugin(true);
        m_plugin->pluginFuncs()->setwindow(m_instance, &m_npWindow);
        setCallingPlugin(false);
    }

    // Destroy the plugin
    NPSavedData* savedData = 0;
    setCallingPlugin(true);
    NPError npErr = m_plugin->pluginFuncs()->destroy(m_instance, &savedData);
    setCallingPlugin(false);
    LOG_NPERROR(npErr);

    if (savedData) {
        if (savedData->buf)
            NPN_MemFree(savedData->buf);
        NPN_MemFree(savedData);
    }

    m_instance->pdata = 0;
	PRINT("PG :  PluginView::stop EXITS \n");
}

void PluginView::setCurrentPluginView(PluginView* pluginView)
{
	PRINT("PG :  PluginView::setCurrentPluginView ENTERED \n");
    s_currentPluginView = pluginView;
	PRINT("PG :  PluginView::setCurrentPluginView EXITS \n");
}

PluginView* PluginView::currentPluginView()
{
	PRINT("PG :  PluginView::currentPluginView ENTERED \n");
	PRINT("PG :  PluginView::currentPluginView EXITS \n");
    return s_currentPluginView;
}

static char* createUTF8String(const String& str)
{
    CString cstr = str.utf8();
    char* result = reinterpret_cast<char*>(fastMalloc(cstr.length() + 1));

    strncpy(result, cstr.data(), cstr.length() + 1);

    return result;
}

static void freeStringArray(char** stringArray, int length)
{
    if (!stringArray)
        return;

    for (int i = 0; i < length; i++)
        fastFree(stringArray[i]);

    fastFree(stringArray);
}

static bool getString(KJSProxy* proxy, JSValue* result, String& string)
{
    if (!proxy || !result || result->isUndefined())
        return false;
    JSLock lock;

    ExecState* exec = proxy->globalObject()->globalExec();
    UString ustring = result->toString(exec);
    exec->clearException();

    string = ustring;
    return true;
}

void PluginView::performRequest(PluginRequest* request)
{

    // don't let a plugin start any loads if it is no longer part of a document that is being
    // displayed unless the loads are in the same frame as the plugin.
    const String& targetFrameName = request->frameLoadRequest().frameName();
	PRINT("PG :  PluginView::performRequest ENTERED \n");
    if (m_parentFrame->loader()->documentLoader() != m_parentFrame->loader()->activeDocumentLoader() &&
        (targetFrameName.isNull() || m_parentFrame->tree()->find(targetFrameName) != m_parentFrame))
        return;

    KURL requestURL = request->frameLoadRequest().resourceRequest().url();
    String jsString = scriptStringIfJavaScriptURL(requestURL);

    if (jsString.isNull()) {
        // if this is not a targeted request, create a stream for it. otherwise,
        // just pass it off to the loader
        if (targetFrameName.isEmpty()) {
            PluginStream* stream = new PluginStream(this, m_parentFrame, request->frameLoadRequest().resourceRequest(), request->sendNotification(), request->notifyData(), plugin()->pluginFuncs(), instance(), m_quirks);
            m_streams.add(stream);
            stream->start();
        } else {
            // TODO: over loaded load() API not working check this
            m_parentFrame->loader()->load(request->frameLoadRequest().resourceRequest());

            // FIXME: <rdar://problem/4807469> This should be sent when the document has finished loading
            if (request->sendNotification()) {
                KJS::JSLock::DropAllLocks dropAllLocks;
                setCallingPlugin(true);
                m_plugin->pluginFuncs()->urlnotify(m_instance, requestURL.deprecatedString().utf8(), NPRES_DONE, request->notifyData());
                setCallingPlugin(false);
            }
        }
        return;
    }

    // Targeted JavaScript requests are only allowed on the frame that contains the JavaScript plugin
    // and this has been made sure in ::load.
    ASSERT(targetFrameName.isEmpty() || m_parentFrame->tree()->find(targetFrameName) == m_parentFrame);

    // Executing a script can cause the plugin view to be destroyed, so we keep a reference to the parent frame.
    RefPtr<Frame> parentFrame = m_parentFrame;
    JSValue* result = m_parentFrame->loader()->executeScript(jsString, request->shouldAllowPopups());

    if (targetFrameName.isNull()) {
        String resultString;

        CString cstr;
        if (getString(parentFrame->scriptProxy(), result, resultString))
            cstr = resultString.utf8();

        RefPtr<PluginStream> stream = new PluginStream(this, m_parentFrame, request->frameLoadRequest().resourceRequest(), request->sendNotification(), request->notifyData(), plugin()->pluginFuncs(), instance(), m_quirks);
        m_streams.add(stream);
        stream->sendJavaScriptStream(requestURL, cstr);
    }
	PRINT("PG :  PluginView::performRequest EXITS \n");

}

void PluginView::requestTimerFired(Timer<PluginView>* timer)
{
	PRINT("PG :  PluginView::requestTimerFired ENTERED \n");

	ASSERT(timer == &m_requestTimer);
    ASSERT(m_requests.size() > 0);

    PluginRequest* request = m_requests[0];
    m_requests.remove(0);

    // Schedule a new request before calling performRequest since the call to
    // performRequest can cause the plugin view to be deleted.
    if (m_requests.size() > 0)
        m_requestTimer.startOneShot(0);

    performRequest(request);
    delete request;
	PRINT("PG :  PluginView::requestTimerFired EXITS \n");

}

void PluginView::scheduleRequest(PluginRequest* request)
{
	PRINT("PG :  PluginView::scheduleRequest ENTERED \n");
    m_requests.append(request);
    m_requestTimer.startOneShot(0);
	PRINT("PG :  PluginView::scheduleRequest EXITS \n");
}

NPError PluginView::load(const FrameLoadRequest& frameLoadRequest, bool sendNotification, void* notifyData)
{
	PRINT("PG :  PluginView::load ENTERED \n");
    ASSERT(frameLoadRequest.resourceRequest().httpMethod() == "GET" || frameLoadRequest.resourceRequest().httpMethod() == "POST");

    KURL url = frameLoadRequest.resourceRequest().url();

    if (url.isEmpty())
        return NPERR_INVALID_URL;

    const String& targetFrameName = frameLoadRequest.frameName();
    String jsString = scriptStringIfJavaScriptURL(url);

    if (!jsString.isNull()) {
        Settings* settings = m_parentFrame->settings();
        if (!settings || !settings->isJavaScriptEnabled()) {
            // Return NPERR_GENERIC_ERROR if JS is disabled. This is what Mozilla does.
            return NPERR_GENERIC_ERROR;
        }

        if (!targetFrameName.isNull() && m_parentFrame->tree()->find(targetFrameName) != m_parentFrame) {
            // For security reasons, only allow JS requests to be made on the frame that contains the plug-in.
            return NPERR_INVALID_PARAM;
        }
    }

    PluginRequest* request = new PluginRequest(frameLoadRequest, sendNotification, notifyData, arePopupsAllowed());
    scheduleRequest(request);

	PRINT("PG :  PluginView::load EXITS \n");
    return NPERR_NO_ERROR;

}

static KURL makeURL(const KURL& baseURL, const char* relativeURLString)
{
    DeprecatedString urlString = DeprecatedString::fromLatin1(relativeURLString);

    // Strip return characters
    urlString.replace('\n', "");
    urlString.replace('\r', "");

    return KURL(baseURL, urlString);
}

NPError PluginView::getURLNotify(const char* url, const char* target, void* notifyData)
{
    FrameLoadRequest frameLoadRequest;

	PRINT("PG :  PluginView::getURLNotify ENTERED \n");
    frameLoadRequest.setFrameName(target);
    frameLoadRequest.resourceRequest().setHTTPMethod("GET");
    frameLoadRequest.resourceRequest().setURL(makeURL(m_baseURL, url));

	PRINT("PG :  PluginView::getURLNotify EXITS \n");
    return load(frameLoadRequest, true, notifyData);
}

NPError PluginView::getURL(const char* url, const char* target)
{
    FrameLoadRequest frameLoadRequest;
	PRINT("PG :  PluginView::getURL ENTERED \n");
    frameLoadRequest.setFrameName(target);
    frameLoadRequest.resourceRequest().setHTTPMethod("GET");
    frameLoadRequest.resourceRequest().setURL(makeURL(m_baseURL, url));

	PRINT("PG :  PluginView::getURL EXITS \n");
    return load(frameLoadRequest, false, 0);
}

static inline bool startsWithBlankLine(const Vector<char>& buffer)
{
    return buffer.size() > 0 && buffer[0] == '\n';
}

static inline int locationAfterFirstBlankLine(const Vector<char>& buffer)
{
    const char* bytes = buffer.data();
    unsigned length = buffer.size();

    for (unsigned i = 0; i < length - 4; i++) {
        // Support for Acrobat. It sends "\n\n".
        if (bytes[i] == '\n' && bytes[i + 1] == '\n')
            return i + 2;

        // Returns the position after 2 CRLF's or 1 CRLF if it is the first line.
        if (bytes[i] == '\r' && bytes[i + 1] == '\n') {
            i += 2;
            if (i == 2)
                return i;
            else if (bytes[i] == '\n')
                // Support for Director. It sends "\r\n\n" (3880387).
                return i + 1;
            else if (bytes[i] == '\r' && bytes[i + 1] == '\n')
                // Support for Flash. It sends "\r\n\r\n" (3758113).
                return i + 2;
        }
    }

    return -1;
}

static inline const char* findEOL(const char* bytes, unsigned length)
{
    // According to the HTTP specification EOL is defined as
    // a CRLF pair. Unfortunately, some servers will use LF
    // instead. Worse yet, some servers will use a combination
    // of both (e.g. <header>CRLFLF<body>), so findEOL needs
    // to be more forgiving. It will now accept CRLF, LF or
    // CR.
    //
    // It returns NULL if EOLF is not found or it will return
    // a pointer to the first terminating character.
    for (unsigned i = 0; i < length; i++) {
        if (bytes[i] == '\n')
            return bytes + i;
        if (bytes[i] == '\r') {
            // Check to see if spanning buffer bounds
            // (CRLF is across reads). If so, wait for
            // next read.
            if (i + 1 == length)
                break;

            return bytes + i;
        }
    }

    return 0;
}

static inline String capitalizeRFC822HeaderFieldName(const String& name)
{
    bool capitalizeCharacter = true;
    String result;

    for (unsigned i = 0; i < name.length(); i++) {
        UChar c;

        if (capitalizeCharacter && name[i] >= 'a' && name[i] <= 'z')
            c = toASCIIUpper(name[i]);
        else if (!capitalizeCharacter && name[i] >= 'A' && name[i] <= 'Z')
            c = toASCIILower(name[i]);
        else
            c = name[i];

        if (name[i] == '-')
            capitalizeCharacter = true;
        else
            capitalizeCharacter = false;

        result.append(c);
    }

    return result;

}

static inline HTTPHeaderMap parseRFC822HeaderFields(const Vector<char>& buffer, unsigned length)
{
    const char* bytes = buffer.data();
    const char* eol;
    String lastKey;
    HTTPHeaderMap headerFields;

    // Loop ove rlines until we're past the header, or we can't find any more end-of-lines
    while ((eol = findEOL(bytes, length))) {
        const char* line = bytes;
        int lineLength = eol - bytes;

        // Move bytes to the character after the terminator as returned by findEOL.
        bytes = eol + 1;
        if ((*eol == '\r') && (*bytes == '\n'))
            bytes++; // Safe since findEOL won't return a spanning CRLF.

        length -= (bytes - line);
        if (lineLength == 0)
            // Blank line; we're at the end of the header
            break;
        else if (*line == ' ' || *line == '\t') {
            // Continuation of the previous header
            if (lastKey.isNull()) {
                // malformed header; ignore it and continue
                continue;
            } else {
                // Merge the continuation of the previous header
                String currentValue = headerFields.get(lastKey);
                String newValue = DeprecatedString::fromLatin1(line, lineLength);

                headerFields.set(lastKey, currentValue + newValue);
            }
        } else {
            // Brand new header
            const char* colon;
            for (colon = line; *colon != ':' && colon != eol; colon++) {
                // empty loop
            }
            if (colon == eol)
                // malformed header; ignore it and continue
                continue;
            else {
                lastKey = capitalizeRFC822HeaderFieldName(DeprecatedString::fromLatin1(line, colon - line));
                String value;

                for (colon++; colon != eol; colon++) {
                    if (*colon != ' ' && *colon != '\t')
                        break;
                }
                if (colon == eol)
                    value = "";
                else
                    value = DeprecatedString::fromLatin1(colon, eol - colon);

                String oldValue = headerFields.get(lastKey);
                if (!oldValue.isNull()) {
                    String tmp = oldValue;
                    tmp += ", ";
                    tmp += value;
                    value = tmp;
                }

                headerFields.set(lastKey, value);
            }
        }
    }

    return headerFields;
}

NPError PluginView::handlePost(const char* url, const char* target, uint32 len, const char* buf, bool file, void* notifyData, bool sendNotification, bool allowHeaders)
{
	PRINT("PG :  PluginView::handlePost ENTERED \n");
    if (!url || !len || !buf)
        return NPERR_INVALID_PARAM;

    FrameLoadRequest frameLoadRequest;

    HTTPHeaderMap headerFields;
    Vector<char> buffer;

    if (file) {
        //<TODO> : Handle file
    } else {
        buffer.resize(len);
        memcpy(buffer.data(), buf, len);
    }

    const char* postData = buffer.data();
    int postDataLength = buffer.size();

    if (allowHeaders) {
        if (startsWithBlankLine(buffer)) {
            postData++;
            postDataLength--;
        } else {
            int location = locationAfterFirstBlankLine(buffer);
            if (location != -1) {
                // If the blank line is somewhere in the middle of the buffer, everything before is the header
                headerFields = parseRFC822HeaderFields(buffer, location);
                unsigned dataLength = buffer.size() - location;

                // Sometimes plugins like to set Content-Length themselves when they post,
                // but WebFoundation does not like that. So we will remove the header
                // and instead truncate the data to the requested length.
                String contentLength = headerFields.get("Content-Length");

                if (!contentLength.isNull())
                    dataLength = min(contentLength.toInt(), (int)dataLength);
                headerFields.remove("Content-Length");

                postData += location;
                postDataLength = dataLength;
            }
        }
    }

    frameLoadRequest.resourceRequest().setHTTPMethod("POST");
    frameLoadRequest.resourceRequest().setURL(makeURL(m_baseURL, url));
    frameLoadRequest.resourceRequest().addHTTPHeaderFields(headerFields);
    frameLoadRequest.resourceRequest().setHTTPBody(PassRefPtr<FormData>(new FormData(postData, postDataLength)));
    frameLoadRequest.setFrameName(target);

	PRINT("PG :  PluginView::handlePost EXITS \n");
    return load(frameLoadRequest, sendNotification, notifyData);
}

NPError PluginView::postURLNotify(const char* url, const char* target, uint32 len, const char* buf, NPBool file, void* notifyData)
{
	PRINT("PG :  PluginView::postURLNotify ENTERED \n");
	PRINT("PG :  PluginView::postURLNotify EXITS \n");
    return handlePost(url, target, len, buf, file, notifyData, true, true);
}

NPError PluginView::postURL(const char* url, const char* target, uint32 len, const char* buf, NPBool file)
{
	PRINT("PG :  PluginView::postURL ENTERED \n");
	PRINT("PG :  PluginView::postURL EXITS \n");
	// As documented, only allow headers to be specified via NPP_PostURL when using a file.
    return handlePost(url, target, len, buf, file, 0, false, file);
}

NPError PluginView::newStream(NPMIMEType type, const char* target, NPStream** stream)
{
	PRINT("PG :  PluginView::newStream ENTERED \n");

    notImplemented();
    // Unsupported
	PRINT("PG :  PluginView::newStream EXITS \n");
    return NPERR_GENERIC_ERROR;
}

int32 PluginView::write(NPStream* stream, int32 len, void* buffer)
{
	PRINT("PG :  PluginView::write ENTERED \n");

    notImplemented();
    // Unsupported
	PRINT("PG :  PluginView::write EXITS \n");
    return -1;
}

NPError PluginView::destroyStream(NPStream* stream, NPReason reason)
{
    PluginStream* browserStream = static_cast<PluginStream*>(stream->ndata);
	PRINT("PG :  PluginView::destroyStream ENTERED \n");

    if (!stream || PluginStream::ownerForStream(stream) != m_instance)
        return NPERR_INVALID_INSTANCE_ERROR;

    browserStream->cancelAndDestroyStream(reason);
	PRINT("PG :  PluginView::destroyStream EXITS \n");
    return NPERR_NO_ERROR;
}

const char* PluginView::userAgent()
{
	PRINT("PG :  PluginView::userAgent ENTERED \n");

    notImplemented();
	PRINT("PG :  PluginView::userAgent EXITS \n");
    return "";
}

void PluginView::status(const char* message)
{
   String s = DeprecatedString::fromLatin1(message);
   PRINT("PG :  PluginView::status ENTERED \n");

   if (Page* page = m_parentFrame->page())
        page->chrome()->setStatusbarText(m_parentFrame, s);
   PRINT("PG :  PluginView::status EXITS \n");
}

NPError PluginView::getValue(NPNVariable variable, void* value)
{
   PRINT("PG :  PluginView::getValue ENTERED \n");
    switch (variable) {
        case NPNVWindowNPObject: {
            NPObject* windowScriptObject = m_parentFrame->windowScriptNPObject();

            // Return value is expected to be retained, as described here: <http://www.mozilla.org/projects/plugin/npruntime.html>
            if (windowScriptObject)
                _NPN_RetainObject(windowScriptObject);

            void** v = (void**)value;
            *v = windowScriptObject;
  		    PRINT("PG :  PluginView::getValue EXITS \n");
            return NPERR_NO_ERROR;
        }

        case NPNVPluginElementNPObject: {
            NPObject* pluginScriptObject = 0;

            if (m_element->hasTagName(appletTag) || m_element->hasTagName(embedTag) || m_element->hasTagName(objectTag))
                pluginScriptObject = static_cast<HTMLPlugInElement*>(m_element)->getNPObject();

            // Return value is expected to be retained, as described here: <http://www.mozilla.org/projects/plugin/npruntime.html>
            if (pluginScriptObject)
                _NPN_RetainObject(pluginScriptObject);

            void** v = (void**)value;
            *v = pluginScriptObject;
  		    PRINT("PG :  PluginView::getValue EXITS \n");
            return NPERR_NO_ERROR;
        }

        case NPNVnetscapeWindow: {
			notImplemented();
  		    PRINT("PG :  PluginView::getValue EXITS \n");
            return NPERR_NO_ERROR;
        }
		case NPNVgcHandle: {
			if (Page* page = m_parentFrame->page())
			{
				int hWin;
				int hGC;
				page->chrome()->client()->getWindowHandles(&hWin, &hGC);
				if(m_GCHandle != hGC)
				{
					m_GCHandle = hGC;
				}
				void** v = (void**)value;
				*v = (void*)hGC;
			}

		}
        default:
  		    PRINT("PG :  PluginView::getValue EXITS \n");
            return NPERR_GENERIC_ERROR;
    }
}

NPError PluginView::setValue(NPPVariable variable, void* value)
{
    PRINT("PG :  PluginView::setValue ENTERED \n");
    switch (variable) {
        case NPPVpluginWindowBool:
            m_isWindowed = value;
		    PRINT("PG :  PluginView::setValue EXITS \n");
            return NPERR_NO_ERROR;
        case NPPVpluginTransparentBool:
            m_isTransparent = value;
		    PRINT("PG :  PluginView::setValue EXITS \n");
			return NPERR_NO_ERROR;
        default:
            notImplemented();
		    PRINT("PG :  PluginView::setValue EXITS \n");
            return NPERR_GENERIC_ERROR;
    }
}

void PluginView::invalidateTimerFired(Timer<PluginView>* timer)
{
	PRINT("PG :  PluginView::invalidateTimerFired ENTERED \n");
    ASSERT(timer == &m_invalidateTimer);

    for (unsigned i = 0; i < m_invalidRects.size(); i++)
        Widget::invalidateRect(m_invalidRects[i]);
    m_invalidRects.clear();
	PRINT("PG :  PluginView::invalidateTimerFired EXITS \n");
}


void PluginView::invalidateRect(NPRect* rect)
{
	PRINT("PG :  PluginView::invalidateRect ENTERED \n");
	if (!rect) {
        invalidate();
		PRINT("PG :  PluginView::invalidateRect EXITS \n");
        return;
    }

    IntRect r(rect->left, rect->top, rect->right - rect->left, rect->bottom - rect->top);

    if (m_isWindowed) {
        //<TODO> : platform invalidateRect
    } else {
        if (m_quirks.contains(PluginQuirkThrottleInvalidate)) {
            m_invalidRects.append(r);
            if (!m_invalidateTimer.isActive())
                m_invalidateTimer.startOneShot(0.001);
        } else
            Widget::invalidateRect(r);
    }
	PRINT("PG :  PluginView::invalidateRect EXITS \n");
}

void PluginView::invalidateRegion(NPRegion region)
{
	PRINT("PG :  PluginView::invalidateRegion ENTERED \n");
    notImplemented();
	PRINT("PG :  PluginView::invalidateRegion EXITS \n");
}

void PluginView::forceRedraw()
{
	PRINT("PG :  PluginView::forceRedraw ENTERED \n");
	updateWindow();
	PRINT("PG :  PluginView::forceRedraw EXITS \n");
}

void PluginView::pushPopupsEnabledState(bool state)
{
	PRINT("PG :  PluginView::pushPopupsEnabledState ENTERED \n");
	m_popupStateStack.append(state);
	PRINT("PG :  PluginView::forceRedraw EXITS \n");
}

void PluginView::popPopupsEnabledState()
{
	PRINT("PG :  PluginView::popPopupsEnabledState ENTERED \n");
	m_popupStateStack.removeLast();
	PRINT("PG :  PluginView::popPopupsEnabledState EXITS \n");
}

bool PluginView::arePopupsAllowed() const
{
	PRINT("PG :  PluginView::arePopupsAllowed ENTERED \n");
	if (!m_popupStateStack.isEmpty())
        return m_popupStateStack.last();

    return false;
}

KJS::Bindings::Instance* PluginView::bindingInstance()
{
    NPObject* object = 0;
	PRINT("PG :  PluginView::bindingInstance ENTERED \n");
    if (!m_plugin || !m_plugin->pluginFuncs()->getvalue)
        return 0;

    NPError npErr;
    {
        KJS::JSLock::DropAllLocks dropAllLocks;
        setCallingPlugin(true);
        npErr = m_plugin->pluginFuncs()->getvalue(m_instance, NPPVpluginScriptableNPObject, &object);
        setCallingPlugin(false);
    }

    if (npErr != NPERR_NO_ERROR || !object)
        return 0;

    RefPtr<KJS::Bindings::RootObject> root = m_parentFrame->createRootObject(this, m_parentFrame->scriptProxy()->globalObject());
	KJS::Bindings::Instance *instance =NULL;

    _NPN_ReleaseObject(object);
	PRINT("PG :  PluginView::bindingInstance EXITS \n");
    return instance;
}

PluginView::~PluginView()
{
	PRINT("PG :  PluginView::~PluginView ENTERED \n");
	stop();

    deleteAllValues(m_requests);

    freeStringArray(m_paramNames, m_paramCount);
    freeStringArray(m_paramValues, m_paramCount);

	//<TODO> : shp destroy window

    m_parentFrame->cleanupScriptObjectsForPlugin(this);

    if (m_plugin && !m_quirks.contains(PluginQuirkDontUnloadPlugin))
        m_plugin->unload();
	PRINT("PG :  PluginView::~PluginView EXITS \n");
}

void PluginView::disconnectStream(PluginStream* stream)
{
	PRINT("PG :  PluginView::disconnectStream ENTERED \n");
    ASSERT(m_streams.contains(stream));

    m_streams.remove(stream);
	PRINT("PG :  PluginView::disconnectStream EXITS \n");
}

void PluginView::determineQuirks(const String& mimeType)
{
	PRINT("PG :  PluginView::determineQuirks ENTERED \n");
	notImplemented();
	PRINT("PG :  PluginView::determineQuirks EXITS \n");
}

void PluginView::setParameters(const Vector<String>& paramNames, const Vector<String>& paramValues)
{
	PRINT("PG :  PluginView::setParameters ENTERED \n");
	ASSERT(paramNames.size() == paramValues.size());

    unsigned size = paramNames.size();
    unsigned paramCount = 0;

    m_paramNames = reinterpret_cast<char**>(fastMalloc(sizeof(char*) * size));
    m_paramValues = reinterpret_cast<char**>(fastMalloc(sizeof(char*) * size));

    for (unsigned i = 0; i < size; i++) {
        if (m_quirks.contains(PluginQuirkRemoveWindowlessVideoParam) && equalIgnoringCase(paramNames[i], "windowlessvideo"))
            continue;

        m_paramNames[paramCount] = createUTF8String(paramNames[i]);
        m_paramValues[paramCount] = createUTF8String(paramValues[i]);

        paramCount++;
    }

    m_paramCount = paramCount;
	PRINT("PG :  PluginView::setParameters EXITS \n");
}

PluginView::PluginView(Frame* parentFrame, const IntSize& size, PluginPackage* plugin, Element* element, const KURL& url, const Vector<String>& paramNames, const Vector<String>& paramValues, const String& mimeType, bool loadManually)
    : m_parentFrame(parentFrame)
    , m_plugin(plugin)
    , m_element(element)
    , m_isStarted(false)
    , m_url(url)
    , m_baseURL(m_parentFrame->loader()->completeURL(m_parentFrame->document()->baseURL()))
    , m_status(PluginStatusLoadedSuccessfully)
    , m_requestTimer(this, &PluginView::requestTimerFired)
    , m_invalidateTimer(this, &PluginView::invalidateTimerFired)
    , m_popPopupsStateTimer(this, &PluginView::popPopupsStateTimerFired)
    , m_paramNames(0)
    , m_paramValues(0)
    , m_isWindowed(true) // mostly flash plugins 
    , m_isTransparent(false)
    , m_isVisible(false)
    , m_attachedToWindow(false)
    , m_haveInitialized(false)
    , m_lastMessage(0)
    , m_isCallingPluginWndProc(false)
    , m_loadManually(loadManually)
    , m_manualStream(0)
	, m_GCHandle(INVALID_HOBJ)
{

	PRINT("PG :  PluginView::PluginView ENTERED \n");
    m_instance = &m_instanceStruct;
    m_instance->ndata = this;

    m_mimeType = mimeType.utf8();
    determineQuirks(mimeType);

    setParameters(paramNames, paramValues);

    m_mode = m_loadManually ? NP_FULL : NP_EMBED;
	m_protoAccountHandle = parentFrame->loader()->client()->getProtoAccountHandle();
    resize(size);
	PRINT("PG :  PluginView::PluginView EXITS \n");
}

void PluginView::init()
{
	PRINT("PG :  PluginView::init entered \n");

    if (m_haveInitialized)
        return;
    m_haveInitialized = true;

   // Moved load request from start() to here as we are not able to know the mimetype
   // until we get headers from URL request
	if (!m_url.isEmpty() && !m_loadManually) {
		PRINT("PG :  PluginView::init Request for header from Network \n");
        FrameLoadRequest frameLoadRequest;
        frameLoadRequest.resourceRequest().setHTTPMethod("GET");
        frameLoadRequest.resourceRequest().setURL(m_url);
        load(frameLoadRequest, false, 0);
    }

    if (m_isWindowed) {
        registerPluginView();
		//<TODO> : 

    } else {
        m_npWindow.type = NPWindowTypeDrawable;
        m_npWindow.window = 0;
    }

    if (!m_quirks.contains(PluginQuirkDeferFirstSetWindowCall))
        setNPWindowRect(frameGeometry());
	PRINT("PG :  PluginView::init EXITS \n");
}

void PluginView::didReceiveResponse(const ResourceResponse& response)
{
	PRINT("PG :  PluginView::didReceiveResponse ENTERED \n");
    ASSERT(m_loadManually);
    ASSERT(!m_manualStream);

    m_manualStream = new PluginStream(this, m_parentFrame, m_parentFrame->loader()->activeDocumentLoader()->request(), false, 0, plugin()->pluginFuncs(), instance(), m_quirks);
    m_manualStream->setLoadManually(true);

    m_manualStream->didReceiveResponse(0, response);
	PRINT("PG :  PluginView::didReceiveResponse EXITS \n");
}

void PluginView::didReceiveData(const char* data, int length)
{
	PRINT("PG :  PluginView::didReceiveData ENTERED \n");
    ASSERT(m_loadManually);
    ASSERT(m_manualStream);

    m_manualStream->didReceiveData(0, data, length);
	PRINT("PG :  PluginView::didReceiveData EXITS \n");
}

void PluginView::didFinishLoading()
{
	PRINT("PG :  PluginView::didFinishLoading ENTERED \n");
    ASSERT(m_loadManually);
    ASSERT(m_manualStream);

    m_manualStream->didFinishLoading(0);
	PRINT("PG :  PluginView::didFinishLoading EXITS \n");
}

void PluginView::didFail(const ResourceError& error)
{
	PRINT("PG :  PluginView::didFail ENTERED \n");
    ASSERT(m_loadManually);
    ASSERT(m_manualStream);

    m_manualStream->didFail(0, error);
	PRINT("PG :  PluginView::didFail EXITS \n");
}

void PluginView::setCallingPlugin(bool b) const
{
	//PRINT("PG :  PluginView::setCallingPlugin ENTETED \n");
    if (!m_quirks.contains(PluginQuirkHasModalMessageLoop))
        return;

    if (b)
        ++s_callingPlugin;
    else
        --s_callingPlugin;

    ASSERT(s_callingPlugin >= 0);

	PRINT("PG :  PluginView::setCallingPlugin EXITS \n");
}

bool PluginView::isCallingPlugin()
{
	PRINT("PG :  PluginView::isCallingPlugin ENTERED \n");
	PRINT("PG :  PluginView::isCallingPlugin EXITS \n");
    return s_callingPlugin > 0;
}

PluginView* PluginView::create(Frame* parentFrame, const IntSize& size, Element* element, const KURL& url, const Vector<String>& paramNames, const Vector<String>& paramValues, const String& mimeType, bool loadManually)
{
	// if we fail to find a plugin for this MIME type, findPlugin will search for
	// a plugin by the file extension and update the MIME type, so pass a mutable String
	PRINT("PG PluginView::create entered \n");   
	String mimeTypeCopy = mimeType;
	PluginPackage* plugin = NULL;
	plugin = PluginDatabase::installedPlugins()->findPlugin(url, mimeTypeCopy);

	// No plugin was found, try refreshing the database and searching again
	if (!plugin && PluginDatabase::installedPlugins()->refresh()) {
		mimeTypeCopy = mimeType;
		plugin = PluginDatabase::installedPlugins()->findPlugin(url, mimeTypeCopy);
	} 
	PluginView* pPluginView = NULL;
	if(plugin)
	{
		pPluginView = new PluginView(parentFrame, size, plugin, element, url, paramNames, paramValues, mimeTypeCopy, loadManually);
	}
	else
	{
		PRINT("PG PluginView::create MIME not supported by PLATFORM \n"); 
	}
	PRINT("PG PluginView::create EXITS \n"); 
	return pPluginView;
}

void PluginView::streamDidFinishLoading(PluginStream* stream)
{
	disconnectStream(stream);
	// check whether the page has completely loaded
	if(TRUE == m_parentFrame->loader()->client()->isLoadingComplete())
	{
		if(m_isWindowed)
		{
			updateWindow();
		}
		else // for non windowed plugins, they do not need the GC handle
		{
			NPEvent npEvent;
			npEvent.event  = PLUGIN_UI_EVENT_CREATE;
			npEvent.lParam = m_protoAccountHandle;
			if(FALSE == dispatchNPEvent(npEvent))
			{
				// Destroy the plugin if creation fails
				NPSavedData* savedData = 0;
				setCallingPlugin(true);
				NPError npErr = m_plugin->pluginFuncs()->destroy(m_instance, &savedData);
				setCallingPlugin(false);
				LOG_NPERROR(npErr);
			}				
		}
	}
}
#ifdef SFB_GWES_TRANSFORM
bool PluginView::relocatePluginWindow(const IntPoint newlocation, IntPoint oldlocation) const
{
	if(oldlocation == newlocation)
		return TRUE;
	NPEvent npEvent;
	NPWindow windowPosition;
	memset(&windowPosition, 0, sizeof(windowPosition));

	windowPosition.x		 = newlocation.x();
	windowPosition.y		 = newlocation.y();

	npEvent.event	 = PLUGIN_UI_EVENT_WINPOS_CHANGED;
	npEvent.lParam   = reinterpret_cast<uint32>(&windowPosition);

	setCallingPlugin(true);
	bool result = m_plugin->pluginFuncs()->event(m_instance, &npEvent);
	setCallingPlugin(false);
	return result;
}
// resizes the plugin window to the new size if the 
bool PluginView::resizePluginWindow(const IntSize newSize, IntSize oldSize) const
{
	//for windowless plugins (embeded audio)
	// size param is not used, but the plugins must be created 
	if(newSize.isEmpty() && m_isWindowed)
		return FALSE;

	NPEvent npEvent;
	NPWindow windowSize;
	memset(&windowSize, 0, sizeof(windowSize));

	// limit the max size of the new rect to the platform limited plugin window 
	IntSize clippedSize = newSize.shrunkTo(m_maxPluginWindowSize);

	if(FALSE == CheckAvailableMemory(clippedSize.height(), clippedSize.width(), m_GCHandle))
	{
		if(!m_optimumSizeSaved.isEmpty())
		{
			if(FALSE == CheckAvailableMemory(m_optimumSizeSaved.height(), m_optimumSizeSaved.width(), m_GCHandle))
			{
				npEvent.event	 = PLUGIN_UI_EVENT_HIDE;
				setCallingPlugin(true);
				bool result = m_plugin->pluginFuncs()->event(m_instance, &npEvent);
				setCallingPlugin(false);
				return FALSE;
			}
			else
			{
				windowSize.width  = m_optimumSizeSaved.width();
				windowSize.height = m_optimumSizeSaved.height();
			}
		}
		else
		{
			return FALSE;
		}
	}
	else
	{
		windowSize.width  = clippedSize.width();
		windowSize.height = clippedSize.height();
	}
	npEvent.event	 = PLUGIN_UI_EVENT_WINSIZE_CHANGED;
	npEvent.lParam   = reinterpret_cast<uint32>(&windowSize);

	setCallingPlugin(true);
	bool result = m_plugin->pluginFuncs()->event(m_instance, &npEvent);
	setCallingPlugin(false);
	return result; 
}
#endif

} // namespace WebCore

#endif //SUPPORT_NP_PLUGIN
