/**
 * This file is part of the KDE project.
 *
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 2000 Simon Hausmann <hausmann@kde.org>
 *           (C) 2000 Stefan Schimanski (1Stein@gmx.de)
 * Copyright (C) 2004, 2005, 2006 Apple Computer, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */
#include "config.h"
#include "RenderPart.h"

#include "Document.h"
#include "Frame.h"
#include "FrameTree.h"
#include "FrameView.h"
#include "HTMLFrameOwnerElement.h"
#include "HTMLNames.h"
#include "Page.h"
#if PLATFORM(SHP) && defined(SFB_SSR_RENDERING)
#include "HTMLIFrameElement.h"
#endif

namespace WebCore {

using namespace HTMLNames;

RenderPart::RenderPart(HTMLFrameOwnerElement* node)
    : RenderWidget(node)
{
    // init RenderObject attributes
    setInline(false);
}

RenderPart::~RenderPart()
{
    // Since deref ends up calling setWidget back on us, need to make sure
    // that widget is already 0 so it won't do any work.
    Widget* widget = m_widget;
    m_widget = 0;
    if (widget && widget->isFrameView())
        static_cast<FrameView*>(widget)->deref();
    else
        delete widget;
}

void RenderPart::setWidget(Widget* widget)
{
    if (widget != m_widget) {
        if (widget && widget->isFrameView())
            static_cast<FrameView*>(widget)->ref();
        RenderWidget::setWidget(widget);

        // make sure the scrollbars are set correctly for restore
        // ### find better fix
        viewCleared();
    }
}

void RenderPart::viewCleared()
{
}

void RenderPart::deleteWidget()
{
    if (m_widget && m_widget->isFrameView())
        static_cast<FrameView*>(m_widget)->deref();
    else
        delete m_widget;
}

// FIXME: This should not be necessary.  Remove this once WebKit knows to properly schedule
// layouts using WebCore when objects resize.
void RenderPart::updateWidgetPosition()
{
    if (!m_widget)
        return;
    
    int x, y, width, height;
	bool requestRelayout = false;

    absolutePosition(x, y);
    x += borderLeft() + paddingLeft();
    y += borderTop() + paddingTop();
	
	width = m_width - borderLeft() - borderRight() - paddingLeft() - paddingRight();

#if PLATFORM(SHP) && defined(SFB_SSR_RENDERING)
	if( SSR_MODE_ON == document()->getSSRMode())
	{
		if(element() && element()->hasTagName(iframeTag))
		{
			ScrollView* view = static_cast<ScrollView*>(m_widget);
			if( m_height >0 && view->contentsHeight()>0)
				m_height = view->contentsHeight();
		}
	}
#endif
    height = m_height - borderTop() - borderBottom() - paddingTop() - paddingBottom();
    IntRect newBounds(x,y,width,height);
    bool boundsChanged = newBounds != m_widget->frameGeometry();
    if (boundsChanged) {
        // The widget changed positions.  Update the frame geometry.
        RenderArena *arena = ref();
        element()->ref();
        m_widget->setFrameGeometry(newBounds);
        element()->deref();
        deref(arena);
    }

    // if the frame bounds got changed, or if view needs layout (possibly indicating
    // content size is wrong) we have to do a layout to set the right widget size
    if (m_widget && m_widget->isFrameView()) {
        FrameView* frameView = static_cast<FrameView*>(m_widget);
        if (boundsChanged || frameView->needsLayout())
		{
	        frameView->layout();
			
#if PLATFORM(SHP) && defined(SFB_SSR_RENDERING)
// CELUNITE_SSR_Change_Begin
			if( SSR_MODE_ON == document()->getSSRMode())
			{
				if(element() && element()->hasTagName(frameTag))
				{
					ScrollView* view = static_cast<ScrollView*>(m_widget);
					m_height= view->contentsHeight();
					setNeedsLayout(true);
				}
				if(element() && element()->hasTagName(iframeTag))
				{
					HTMLIFrameElement *iFrame = static_cast<HTMLIFrameElement *>(node()) ;
					AtomicString m_url=iFrame->url();
					if((!m_url.contains("http://")) || m_url.contains(document()->domain()))
					{
						ScrollView* view = static_cast<ScrollView*>(m_widget);
						m_height = view->contentsHeight();
						this->setNeedsLayout(true);
					}
					else
					{
						m_height = 0;
						m_width = 0 ;
					}
				}
			}
// CELUNITE_SSR_Change_End
#endif
		}
	}
}

#if PLATFORM(SHP) && defined(USE_SMALL_SCREEN_LAYOUT)
void RenderPart::expandWidgetSize()
{
	if(!m_widget)
        return;

	//for small screen we set the visible size to the content size
	if(element() && element()->hasTagName(iframeTag))
	{
		FrameView* view = static_cast<FrameView*>(m_widget);
		
		bool requestRelayout = false;

		int height = view->contentsHeight();
		if(view->visibleHeight() > height)
			height = view->visibleHeight();

		int extendedHeight = height + borderTop() + borderBottom() + paddingTop() + paddingBottom();
		if(extendedHeight > m_height)
		{
			m_height = extendedHeight;
			requestRelayout = true;
		}

		int width = view->contentsWidth();
		if(view->visibleWidth() > width)
			width = view->visibleWidth();

		int extendedWidth = width + borderLeft() + borderRight() + paddingLeft() + paddingRight();
		if(extendedWidth > m_width)
		{
			m_width = extendedWidth;
			requestRelayout = true;
		}

		if(requestRelayout)
		{
			view->layout();
			//view->setNeedsLayout();
			//view->scheduleLayout();

			//when iframe's visible size changes, make it and its parent
			//(recursively) dirty and set them as layout needed.
			this->setPrefWidthsDirty(true);

			RenderObject* parent = this->parent();
			while(parent)
			{
				parent->setNeedsLayout(true);
				parent->setPrefWidthsDirty(true);
				parent = parent->parent();
			}

			(static_cast<FrameView*>(view->parent()))->setNeedsLayout();
		}
	}
}
#endif //PLATFORM(SHP) && USE_SMALL_SCREEN_LAYOUT

}
