#ifndef URLLOADER_H
#define URLLOADER_H

//  INCLUDES
#include "ContentLoader.h"
#include "BaseTransaction.h"
#include "CMMDefs.h"

// CONSTANTS

// MACROS

// DATA TYPES

// FUNCTION PROTOTYPES

// FORWARD DECLARATIONS
class ContentDispatcher;
//class UrlRequestInfo;

// CLASS DECLARATION

/**
*  This is the base class for all type of loaders.
*  @lib resLoader.lib
*  @since 3.1
*/
class UrlLoader : public IContentLoader, public ITransactionObserver CMM_MODIFIED_ADD
    {
    public:  // Constructors and destructor

        /**
        * Constructor.
        * @param _contentDispatcher This dispatcher takes the incoming chunks
        * @return
        */
        UrlLoader( ContentDispatcher* _contentDispatcher );

        /**
        * Destructor.
        */
        virtual ~UrlLoader();

    public:  // new functions

        /**
        * Lock url loader
        */
        void Ref() { ++m_refCount; }

        /**
        * Unlock url loader
        */
        void Deref() { if( --m_refCount == 0 ) delete this; }


    public: // from IContentLoader

        /**
        * Request a url.
        * @param _requestInfo Url request info created by the resloader client.
        * @return OS wide error code.
        */
        virtual eErrorCode Request( UrlRequestInfo* _requestInfo );

        /**
        * Cancel a transaction by id.
        * @param _transactionId Transaction id to cancel
        * @param _error This error code is propagated up to
        * the resource loader client through Complete()
        * @return KErrNone/KErrNotFound
        */
        virtual eErrorCode Cancel( int _transactionId, eErrorCode _error = ERR_OK );

        /**
        * Cancell all outstanding transactions.
        * @param _error This error code is propagated up to
        * the resource loader client through Complete()
        * @return ErrNotFound if there is not outstanding transaction.
        */
        virtual eErrorCode CancelAll( eErrorCode _error = ERR_OK );

    public: // from ITransactionObserver

        /**
        * This function is called when the header information is available
        * @param _transaction Observed transaction.
        * @param _headers Header information
        * @return ErrNotSupported if it is not a native browser content
        */
        virtual eErrorCode Headers( BaseTransaction* _transaction, UrlResponseInfo* _response );

        /**
        * This function is called when a new partial body is available
        * @param _transaction Observed transaction.
        * @param _response Response information
        * @return void
        */
        virtual void Response( BaseTransaction* _transaction, UrlResponseInfo* _response );

        /**
        * This function is called when the transaction is complete
        * @param _transaction Observed transaction.
        * @param _error HTTP error (404) / cancel error
        * @return void
        */
        virtual void Complete( BaseTransaction* _transaction, UrlResponseInfo* _response, eErrorCode _error );

        /**
        * This function is called when an error happens to the transaction.
        * @param _transaction Observed transaction.
        * @param _error HTTP error
        * @return 
        */
        virtual bool HandleError( BaseTransaction* _transaction, int _error );

        /**
        * Get the url loader so that the transaction can have the session.
        * @return Url loader object
        */
        UrlLoader* GetUrlLoader();

        BaseTransaction* Transaction(int _transactionId );

        BaseTransaction* FindTransactionByTid(const int _transactionId );

    protected: 

        /**
        * Create a transaction.
        * @param _urlRequestInfo Url request info created by the resource loader client.
        * @return Transaction object.
        */
        virtual BaseTransaction* CreateTransaction( UrlRequestInfo* _urlRequestInfo ) = 0;

        /**
        * Cancel a transaction by transaction object
        * @param _transaction Transaction to cancel
        * @param _error This error code is propagated up to
        * the resource loader client through Complete()
        * @return
        */
        void Cancel( BaseTransaction* _transaction, eErrorCode _error = ERR_OK );
        
    private: // New functions

        /**
        * Close the url loader/session.
        * @return OS wide error code
        */
        eErrorCode CloseLoader();

        /**
        * Call submit on the transaction
        * @param aTrans trans to submit
        * @return
        */
        eErrorCode DoSubmitTransaction( BaseTransaction* _trans );

    
	public: // to redirect stream
        /**
        * Removes the transaction from the active transaction list
        * @param _transactionId Id of the transaction to be removed
        * @return BaseTransaction that was removed from the list
        */
        BaseTransaction* RemoveTransaction( int _transactionId);

    protected:    // Data

        // content handler
        ContentDispatcher* m_contentDispatcher; // not owned

        // The load queue.
        Vector<BaseTransaction*> m_activeTransList;

        //
        unsigned int  m_refCount;

    };

#endif      // CURLLOADER_H

// End of File
