/*
     File:       CFNetwork/CFURLProtectionSpacePriv.h
     Copyright:  (c) 2007 by Apple Inc., all rights reserved
*/
#ifndef __CFURLPROTECTIONSPACEPRIV__
#define __CFURLPROTECTIONSPACEPRIV__

#ifndef __CFNETWORKDEFS__
#include <CFNetwork/CFNetworkDefs.h>
#endif

#ifndef __COREFOUNDATION__
#include <CoreFoundation/CoreFoundation.h>
#endif



#include <AvailabilityMacros.h>

#if PRAGMA_ONCE
#pragma once
#endif

#ifdef __cplusplus
extern "C" {
#endif

#if PRAGMA_ENUM_ALWAYSINT
    #pragma enumsalwaysint on
#endif


/*
 *  CFURLProtectionSpaceRef
 *  
 *  Discussion:
 *    This is the type of a reference CFURLProtectionSpace.
 */
typedef struct _CFURLProtectionSpace*   CFURLProtectionSpaceRef;

/*
 *  CFURLProtectionSpaceServerType
 *  
 *  Discussion:
 *    The different kinds of protection spaces that can be passed in to
 *    CFURLProtectionSpaceCreate()
 */
enum CFURLProtectionSpaceServerType {

  /*
   * This protection space is for an HTTP server.
   */
  kCFURLProtectionSpaceServerHTTP = 1,

  /*
   * This protection space is for an HTTPS server (HTTP server over a
   * secure connection).
   */
  kCFURLProtectionSpaceServerHTTPS = 2,

  /*
   * This protection space is for an FTP server.
   */
  kCFURLProtectionSpaceServerFTP = 3,

  /*
   * This protection space is for an FTPS server (FTP server over a
   * secure connection).
   */
  kCFURLProtectionSpaceServerFTPS = 4,

  /*
   * This protection space is for an HTTP proxy.
   */
  kCFURLProtectionSpaceProxyHTTP = 5,

  /*
   * This protection space is for an HTTPS proxy (HTTP proxy over a
   * secure connection) .
   */
  kCFURLProtectionSpaceProxyHTTPS = 6,

  /*
   * This protection space is for an FTP proxy.
   */
  kCFURLProtectionSpaceProxyFTP = 7,

  /*
   * This protection space is for an SOCKS proxy.
   */
  kCFURLProtectionSpaceProxySOCKS = 8
};
typedef enum CFURLProtectionSpaceServerType CFURLProtectionSpaceServerType;



/*
 *  CFURLProtectionSpaceAuthenticationScheme
 *  
 *  Discussion:
 *    The different kinds of authentication schmes that can be passed
 *    in to CFURLProtectionSpaceCreate()
 */
enum CFURLProtectionSpaceAuthenticationScheme {

  /*
   * Use the default authentication scheme for a protocol. For
   * protection spaces created using
   * kCFURLProtectionSpaceAuthenticationSchemeDefault, the
   * CFURLProtectionSpaceCopyAuthenticationScheme function will return
   * kCFURLProtectionSpaceAuthenticationSchemeDefault until the actual
   * authentication scheme is determined.
   */
  kCFURLProtectionSpaceAuthenticationSchemeDefault = 1,

  /*
   * Use the Basic HTTP authentication scheme.
   */
  kCFURLProtectionSpaceAuthenticationSchemeHTTPBasic = 2,

  /*
   * Use the Digest Access HTTP authentication scheme.
   */
  kCFURLProtectionSpaceAuthenticationSchemeHTTPDigest = 3,

  /*
   * Use HTML Forms authentication.
   */
  kCFURLProtectionSpaceAuthenticationSchemeHTMLForm = 4,

  /*
   * Use the NTLM authentication scheme.
   */
  kCFURLProtectionSpaceAuthenticationSchemeNTLM = 5,

  /*
   * Negotiate for the authentication scheme - Kerberos or NTLM.
   */
  kCFURLProtectionSpaceAuthenticationSchemeNegotiate = 6
};
typedef enum CFURLProtectionSpaceAuthenticationScheme CFURLProtectionSpaceAuthenticationScheme;


/*
 *  CFURLProtectionSpaceGetTypeID()
 *  
 *  Discussion:
 *    Returns the type identifier of all CFURLProtectionSpace instances.
 *  
 */
APPLE_WIN_API CFTypeID 
CFURLProtectionSpaceGetTypeID(void)                           AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;



/*
 *  CFURLProtectionSpaceCreate()
 *  
 *  Discussion:
 *    Creates a new CFURLProtectionSpace object.
 *  
 *  Parameters:
 *    
 *    allocator:
 *      The CFAllocator which should be used to allocate memory for the
 *      CFURLProtectionSpace. If this reference is not a valid
 *      CFAllocator, the behavior is undefined.
 *    
 *    host:
 *      A CFString containing the host name for the protection space.
 *    
 *    port:
 *      A SInt32 value containing the port number for the protection
 *      space.
 *    
 *    serverType:
 *      A CFURLProtectionSpaceServerType constant defining the server
 *      type or proxy type for the protection space.
 *    
 *    realm:
 *      A CFString containing realm string for the protection space, or
 *      NULL if there is no realm.
 *    
 *    authenticationScheme:
 *      A CFURLProtectionSpaceAuthenticationScheme constant defining
 *      the authentication scheme for the protection space.
 *  
 *  Result:
 *    A valid CFURLProtectionSpaceRef, or NULL if unsuccessful.
 *  
 */
APPLE_WIN_API CFURLProtectionSpaceRef 
CFURLProtectionSpaceCreate(
  CFAllocatorRef                             allocator,
  CFStringRef                                host,
  SInt32                                     port,
  CFURLProtectionSpaceServerType             serverType,
  CFStringRef                                realm,                      /* can be NULL */
  CFURLProtectionSpaceAuthenticationScheme   authenticationScheme) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;



/*
 *  CFURLProtectionSpaceCreateCopy()
 *  
 *  Discussion:
 *    Creates a new copy of a CFURLProtectionSpace object.
 *  
 *  Parameters:
 *    
 *    allocator:
 *      The CFAllocator which should be used to allocate memory for the
 *      CFURLProtectionSpace. If this reference is not a valid
 *      CFAllocator, the behavior is undefined.
 *    
 *    theProtectionSpace:
 *      The CFURLProtectionSpace object to be copied.
 *  
 *  Result:
 *    A valid CFURLProtectionSpaceRef, or NULL if unsuccessful.
 *  
 */
APPLE_WIN_API CFURLProtectionSpaceRef 
CFURLProtectionSpaceCreateCopy(
  CFAllocatorRef            allocator,
  CFURLProtectionSpaceRef   theProtectionSpace)               AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;



/*
 *  CFURLProtectionSpaceGetHost()
 *  
 *  Discussion:
 *    Gets the host name of a CFURLProtectionSpace object.
 *  
 *  Parameters:
 *    
 *    protectionSpace:
 *      The CFURLProtectionSpace object.
 *  
 *  Result:
 *    A CFString containing the host name, or NULL if unsuccessful.
 *  
 */
APPLE_WIN_API CFStringRef 
CFURLProtectionSpaceGetHost(CFURLProtectionSpaceRef protectionSpace) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;



/*
 *  CFURLProtectionSpaceGetPort()
 *  
 *  Discussion:
 *    Gets the port number of a CFURLProtectionSpace object.
 *  
 *  Parameters:
 *    
 *    protectionSpace:
 *      The CFURLProtectionSpace object.
 *  
 *  Result:
 *    A SInt32 containing the port number, or 0 if unsuccessful.
 *  
 */
APPLE_WIN_API SInt32 
CFURLProtectionSpaceGetPort(CFURLProtectionSpaceRef protectionSpace) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;



/*
 *  CFURLProtectionSpaceGetServerType()
 *  
 *  Discussion:
 *    Gets the server or proxy type of a CFURLProtectionSpace object.
 *  
 *  Parameters:
 *    
 *    protectionSpace:
 *      The CFURLProtectionSpace object.
 *  
 *  Result:
 *    A CFURLProtectionSpaceServerType containing the server or proxy
 *    type, or 0 if unsuccessful.
 *  
 */
APPLE_WIN_API CFURLProtectionSpaceServerType 
CFURLProtectionSpaceGetServerType(CFURLProtectionSpaceRef protectionSpace) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;



/*
 *  CFURLProtectionSpaceGetRealm()
 *  
 *  Discussion:
 *    Gets the realm of a CFURLProtectionSpace object.
 *  
 *  Parameters:
 *    
 *    protectionSpace:
 *      The CFURLProtectionSpace object.
 *  
 *  Result:
 *    A CFString containing the realm, or NULL if there is no realm.
 *  
 */
APPLE_WIN_API CFStringRef 
CFURLProtectionSpaceGetRealm(CFURLProtectionSpaceRef protectionSpace) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;



/*
 *  CFURLProtectionSpaceGetAuthenticationScheme()
 *  
 *  Discussion:
 *    Gets the authentication scheme of a CFURLProtectionSpace object.
 *  
 *  Parameters:
 *    
 *    protectionSpace:
 *      The CFURLProtectionSpace object.
 *  
 *  Result:
 *    A CFURLProtectionSpaceAuthenticationScheme containing the
 *    authentication scheme, or 0 if unsuccessful.
 *  
 */
APPLE_WIN_API CFURLProtectionSpaceAuthenticationScheme 
CFURLProtectionSpaceGetAuthenticationScheme(CFURLProtectionSpaceRef protectionSpace) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;



/*
 *  CFURLProtectionSpaceIsProxy()
 *  
 *  Discussion:
 *    Determine if this CFURLProtectionSpace is a proxy server.
 *  
 *  Parameters:
 *    
 *    protectionSpace:
 *      The CFURLProtectionSpace object.
 *  
 *  Result:
 *    TRUE if this CFURLProtectionSpace is a proxy server; otherwise,
 *    FALSE.
 *  
 */
APPLE_WIN_API Boolean 
CFURLProtectionSpaceIsProxy(CFURLProtectionSpaceRef protectionSpace) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;



/*
 *  CFURLProtectionSpaceReceivesCredentialSecurely()
 *  
 *  Discussion:
 *    Determine if the credentials for this CFURLProtectionSpace can be
 *    sent securely.
 *  
 *  Parameters:
 *    
 *    protectionSpace:
 *      The CFURLProtectionSpace object.
 *  
 *  Result:
 *    TRUE if a secure authentication scheme or protocol will be used;
 *    otherwise, FALSE.
 *  
 */
APPLE_WIN_API Boolean 
CFURLProtectionSpaceReceivesCredentialSecurely(CFURLProtectionSpaceRef protectionSpace) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


#if PRAGMA_ENUM_ALWAYSINT
    #pragma enumsalwaysint reset
#endif

#ifdef __cplusplus
}
#endif

#endif /* __CFURLPROTECTIONSPACEPRIV__ */

