/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics.
 */
/**
 * @file		IContents.h
 * @brief		Define interface ISetting of WebControl
 * @author	Chaehwan Li (chaehwan.li@samsung.com)
 * @version	1.0
 * @date		2008.03.14	v1.0		Created
 */

#ifndef __IContents_Defined
#define __IContents_Defined

#include "WebDefs.h"
#include "IIMEListener.h"
#include "IContentsListener.h"
#include "IWindowListener.h"
//#include "IPDPContext.h"
#include "IDOMCore.h"
#include "IUnknown.h"
#include "IPropertyMap.h"
#include "IStreamListener.h"
#include "IDecisionListener.h"

namespace	WebControl
{

class IPageInfo : public IUnknown
{
public:
	/*
	 * @desc gets the title of page
	 * @param out_pszBuffer : pointer to buffer
	 * @param in_nBufferLen : length of buffer
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getTitle( char* out_pszBuffer, int in_nBufferLen ) = 0;

	/*
	 * @desc gets the url of page
	 * @param out_pszBuffer : pointer to buffer
	 * @param in_nBufferLen : length of buffer
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getURL( char* out_pszBuffer, int in_nBufferLen ) = 0;

	/*
	 * @desc gets the size of page
	 * @param out_pszBuffer : pointer to buffer
	 * @param in_nBufferLen : length of buffer
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getSize( int *out_size ) = 0;

	/*
	 * @desc gets last modified date of page
	 * @param out_pszBuffer : pointer to buffer
	 * @param in_nBufferLen : length of buffer
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getLastModified( char* out_pszBuffer, int in_nBufferLen ) = 0;

	/*
	 * @desc gets the mime type of page
	 * @param out_pszBuffer : pointer to buffer
	 * @param in_nBufferLen : length of buffer
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getMimetype( char* out_pszBuffer, int in_nBufferLen ) = 0;

};

class IImage : public IUnknown
{
public:
	/*
	 * @desc gets codec type
	 * @param out_pszBuffer : pointer to buffer
	 * @param in_nBufferLen : length of buffer
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getCodecType( ImgCodecType *out_codec ) = 0;

	virtual HRESULT getURL( char* out_pszBuffer, int in_pnBufferLen ) = 0;

	/*
	 * @desc gets raw data
	 * @param out_pszBuffer : pointer to buffer
	 * @param in_nBufferLen : buffer length
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getRawData( char* out_pszBuffer, int in_nBufferLen ) = 0;

	/*
	 * @desc gets Raw data size
	 * @param
	 * @return int total size
	 */
	virtual HRESULT getRawDataSize( int *out_size ) = 0;
};


/*
 * @desc
 */
class IContents
{
public:
	/*
	 * @desc registers with Content's listner to a Browser Engine.
	 * @param in_pIContentsListener : pointer to contents listeners
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT setContentListener( IContentsListener* in_pIContentsListener ) = 0;
	virtual HRESULT getContentListener( IContentsListener** out_pIContentsListener ) = 0;

	virtual HRESULT setDownloadListener( URILoader::IStreamListener *in_pIStreamListener ) = 0;
	virtual HRESULT setDecisionListener( IDecisionListener *in_pIDecisionListener ) = 0;

	/*
	 * @desc registers with IME's listner to a Browser Engine.
	 * @param in_pIIMEListener : pointer to a IME listeners
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT setIMEListener( IIMEListener* in_pIIMEListener ) = 0;

	/*
	 * @desc registers with Window's listner to a Browser Engine.
	 * @param in_pIWindowListner : pointer to a Window listeners
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT setWindowListener( IWindowListener* in_pIWindowListener ) = 0;

	/*
	 * @desc registers with PDPContext's listner to a Browser Engine.
	 * @param in_pIPDPContextListener : pointer to a PDPContext listeners
	 * @return HRESULT TRUE/FALSE
	 */
	//virtual HRESULT setPDPContextListner( IPDPContextListener* in_pIPDPContextListener ) = 0;

	/*
	 * @desc suspends a Browser Engine
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT suspend( void ) = 0;

	/*
	 * @desc resumes a Browser Engine
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT resume( void ) = 0;

	/*
	 * @desc aborts page request and page actions of a Browser Engine
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT abort( void ) = 0;

	/*
	 * @desc creates a thumbnail image for displayed page currently.
	 * @param
	 * @return IImage pointer to IImage
	 */
	virtual HRESULT createPageThumbnail( IImage** out_Image ) = 0;

	/*
	 * @desc gets an image informatioin on focused.
	 * @param out_pImage : pointer to IImage
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getCurrentImageInfo( IImage** out_pImage ) = 0;

	/*
	 * @desc gets an image informatioin with position.
	 * @param out_pImage : pointer to IMAGEINFO
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getCurrentImageInfoByPoint( IImage** out_pImage, int x, int y ) = 0;

	/*
	 * @desc gets a current page informatioin.
	 * @param out_pImageInfo : pointer to IPageInfo
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getCurrentPageInfo( IPageInfo** out_pPageInfo ) = 0;

//#ifdef SFB_DRAG_DROP
//	/*
//	 * @desc gets a current drag element rect.
//	 * @return HRESULT TRUE/FALSE
//	 */
//        virtual HRESULT getCurrentDragElementRect(int*height, int*width,int* actualX,int* actualY, int x, int y,  int* ctElement )= 0;
//	
//        /*
//	 * @desc gets a current drag element Info.
//	 * @return HRESULT TRUE/FALSE
//	 */
//	virtual HRESULT getCurrentDragElementInfo(void* ctElement, char* srcUrl, char* hrefUrl, char** textString )= 0;
//#endif
	/*
	 * @desc checks to exist a displayed contents currently.
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT isExistDisplayedContents() = 0;

	/*
	 * @desc gets state whether plugin is enabled or not with given mime type.
	 * @param in_PlugInType : plugin types
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getPlugInEnable( char* in_pszPlugInMime ) = 0;

	/*
	 * @desc gets a position of virtual pointer
	 * @param out_x, out_y : x, y
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getVirtualPointerCurrentPos( int* out_x, int* out_y ) = 0;

	/*
	 * @desc loads URL with POST method
	 * @param in_pszUrl : posted url
	 * @param in_pszPostData : posted data
	 * @param in_PostDataLen : length of the posted data
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT postURL( char* in_pszUrl, char* in_pszPostData, int in_PostDataLen ) = 0;

	/*
	 * @desc loads URL
	 * @param In_pszUrl : URL
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT loadURL(const char* in_pszUrl ) = 0;

	/*
	 * @desc gets current document URL
	 * @param none
	 * @return const char* docUrl
	 */
	virtual const char* getCurrentDocURL() = 0;

	/*
	 * @desc check mode that a browser engine is image save mode or not
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */

	virtual HRESULT isImageSaveMode( void ) = 0;

	/*
	 * @desc check mode that a browser engine is map view mode or not
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */

//	#ifdef _SFB_PAGE_PREVIEW
	virtual HRESULT isMapViewMode( void ) = 0;
//	#endif

	/*
	 * @desc check image is on the point or not
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT isPointOnImage( int x, int y ) = 0;

	/*
	 * @desc check mode that a browser engine is search text mode or not
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT isSearchTextMode( void ) = 0;

	/*
	 * @desc check mode that a browser engine is virtual pointer mode or not
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT isVirtualPointerMode( void ) = 0;

	/*
	 * @desc passes key pressed event to browser engine
	 * @param in_KeyValue : key value is based on platform
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT KeyPressed( int in_KeyValue ) = 0;

	/*
	 * @desc passes key released event to browser engine
	 * @param in_KeyValue : key value is based on platform
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT KeyReleased( int in_KeyValue ) = 0;

	/*
	 * @desc loads contents with contents data
	 * @param in_pszUrl : URL
	 * @param in_pszData : contents
	 * @param in_DataLen : length
	 * @param in_pszMimeType : mimetype of contents
	 * @param in_pszCharset : charset of contents
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT loadData( char* in_pszUrl, char* in_pszData, int in_DataLen, char* in_pszMimeType, char* in_pszCharset ) = 0;

	/*
	 * @desc loads contents with file name
	 * @param in_pszPathFileName : File name included path. ( without filescheme )
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT loadFile( char* in_pszPathFileName ) = 0;

	/*
	 * @desc loads contents with file name that is saved by a browser engine and is located in saved page directory.
	 * @param in_pszPathFileName : File Name included path
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT loadSavedPage( char* in_pszPathFileName ) = 0;

	/*
	 * @desc moves focus to next text
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT nextSearchText( char* in_pszSearchText ) = 0;

	/*
	 * @desc gets a title of current page
	 * @param out_pszTitle : pointer to title
	 * @param in_TitleLen : length of title
	 * @return HRESULT TRUE/FALSE
	 */
	// removed
	//virtual HRESULT getPageInfoTitle( char* out_pszTitle, int in_TitleLen ) = 0;

	/*
	 * @desc gets a URL of current page
	 * @param out_pszUrl : pointer to URL
	 * @param in_UrlLen : length of URL
	 * @return HRESULT TRUE/FALSE
	 */
	// removed
	//virtual HRESULT getPageInfoURL( char* out_pszUrl, int in_UrlLen ) = 0;

	/*
	 * @desc scrolls by given x, y
	 * @param x,y :
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT scrollPane( int dx, int dy ) = 0;

	/*
	 * @desc flick scrolls by given x, y
	 * @param x,y :
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT flickScrollPane( int dx, int dy )=0;

	/*
	 * @desc moves focus to previous text
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT prevSearchText( char* in_pszSearchText ) = 0;

	/*
	 * @desc reloads current contents
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT reloadPage( void ) = 0;

	/*
	 * @desc saves a current contents to given path
	 * @param in_pszPath : path to indicate for saved page.
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT saveCurrentPage( char* in_pszPath ) = 0;

	/*
	 * @desc sets page rendering mode to browser engine
	 * @param in_nRenderingMode : page rendering mode
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT setPageRenderingMode( int in_nRenderingMode ) = 0;

	/*
	 * @desc get page rendering mode to browser engine
	 * @param in_nRenderingMode : page rendering mode
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getPageRenderingMode( int *out_nRenderingMode ) = 0;

	/*
	 * @desc sets virtual pointer mode to browser engine
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT setVirtualPointerMode( BOOL in_bOnOff ) = 0;

	/*
	 * @desc sets save image mode to browser engine
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT setImageSaveMode( BOOL in_bOnOff ) = 0;

	/*
	 * @desc sets page Map View mode to browser engine
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT isSearchMode( BOOL bMode )=0;

	/*
	 * @desc sets one click mode to browser engine
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */

	virtual HRESULT setOneClickMode( BOOL bMode )=0;

	/**
	 * @desc to set Smart Touch Enabled.
	 */
       virtual void setSmartTouchEnabled(bool enabled) = 0;

	   /**
	 * @desc to set Smart Touch Focus Range.
	 */
       virtual  void setSmartTouchFocusRange(int range) = 0;

	   /**
	 * @desc to get real content height.
	 */
	virtual HRESULT getContentsSize( int *width, int *heigth) = 0; 
	
	
//	#ifdef _SFB_PAGE_PREVIEW
	virtual HRESULT setMapViewMode( BOOL in_bOnOff ) = 0;

	virtual bool pagePilotScroll( int pos_X, int pos_Y) = 0;
//	#endif
	/*
	 * @desc sets zoom value of contents to browser engine
	 * @param in_Zoom : Zoom Value ( 0 ~ 100 : 50 means 100% , 100 means 200% )
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT setZoom( unsigned int in_Zoom ) = 0;


	/*
	 * @desc 
	 * @param in_Zoom : Zoom Value ( 0 ~ 100 : 50 means 100% , 100 means 200% )
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT zooming( unsigned int in_Zoom ) = 0;

	/*
	 * @desc gets zoom value of contents to browser engine
	 * @param in_Zoom : Zoom Value ( 0 ~ 100 : 50 means 100% , 100 means 200% )
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getZoom( unsigned int *out_Zoom ) = 0;

	/*
	 * @desc passes click event by touch to browser engine
	 * @param x, y  : position
	 * @return HRESULT TRUE/FALSE
	 */
//	virtual HRESULT TouchClick( int x, int y ) = 0;

	/*
	 * @desc passes focus move event by touch to browser engine
	 * @param x, y  : position
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT TouchFocusMove( int x, int y ) = 0;

	/*
	 * @desc passes touch pressed event to browser engine
	 * @param x, y  : position
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT TouchPressed( int x, int y ) = 0;

	/*
	 * @desc passes touch released event to browser engine
	 * @param x, y  : position
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT TouchReleased( int x, int y, bool bHandleDoubleTab = false ) = 0;

	/*
	 * @desc passes touch Long Pressed event to browser engine
	 * @param x, y  : position
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT TouchLongPressed( int x, int y ) = 0;

	/*
	 * @desc gets total count of images in current contents
	 * @param
	 * @return int Count of images
	 */
	virtual HRESULT getCountImages( int *out_count  ) = 0;

	/*
	 * @desc searches text on current contents and if the text is found then enter the search mode
	 * @param in_pszSearchText : pointer to text to search
	 * @param out_pCount : pointer to the searched text if is found.
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT beginSearchText( char* in_pszSearchText, int* out_pCount ) = 0;

	/*
	 * @desc finishes the search mode
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT endSearchText( void ) = 0;

	/*
	 * @desc draws Browser Engin's screen
	 * @param in_x, in_y, in_w, in_h : x, y, w, h
	 * @return
	 */
	virtual HRESULT drawScreen( int in_x, int in_y, int in_w, int in_h ) = 0;

	/*
	 * @desc changes Browser screen's area
	 * @param w, h : weight, height
	 * @return
	 */
	virtual HRESULT changeScreenSize( int w, int h ) = 0;

	/*
	 * Signifies the end of page load
	*/
	

	/*
	 * @desc rotates Browser screen's area
	 * @param in_x, in_y, in_w, in_h : x, y, w, h
	 * @return
	 */
	virtual HRESULT rotateScreen( int in_x, int in_y, int in_w, int in_h ) = 0;

	/*
	 * @desc shows Browser Engin's screen
	 * @param in_x, in_y, in_w, in_h : x, y, w, h
	 * @return
	 */
	virtual HRESULT showScreen( int in_x, int in_y, int in_w, int in_h ) = 0;


	/*
	 * @desc copy the content of screen to destination gc
	 *
	 */
	virtual HRESULT copyScreen( int destGC, int destx, int desty, int srcx, int srcy, int srcw, int srch ) = 0;


	/*
	* @desc get the graphic context which WebControl is using
	*/
	virtual HRESULT getGC( int* out_gc )=0;

	/*
	 * @desc updates page without reload.
	 * @param
	 * @return
	 */
	virtual HRESULT updatePage( void ) = 0;

	/*
	 * @desc checks whether the focus is assigned on the point or not.
	 * @param x, y : point
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT	isFocusedByPoint( int x, int y ) = 0;

	/*
	 * @desc checks whether the focus is possible on the point or not.
	 * @param x, y : point
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT isFocusableByPoint( int x, int y ) = 0;

	/*
	 * @desc checks whether the anchor tag is on the point or not.
	 * @param x, y : point
	 * @return HRESULT TRUE/FALSE
	 */
	// removed
	//virtual HRESULT isAnchorByPoint( int x, int y ) = 0;

	/*
	 * @desc gets DOM Node with point.
	 * @param out_pIDomNode : pointer to IDOMNode class
	 * @param in_x : x
	 * @param in_y : y
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getElementByPoint( IPropertyMap** out_propertyMap, int x, int y ) = 0;

	/*
	 * @desc gets DOM Node with focus
	 * @param out_pIDomNode : pointer to IDOMNode class
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getElementByFocus( IPropertyMap** out_propertyMap ) = 0;

	virtual HRESULT getWmlDoCount( int *out_count ) = 0;
	virtual HRESULT getWmlDoName( int in_index, char *inout_buffer, int in_bufsize ) = 0;
	virtual HRESULT executeWmlDo( int in_index ) = 0;

	/*
     *@param drawsBackround YES to cause the receiver to draw a default white background, NO otherwise.
    *@desc Sets whether the receiver draws a default white background when the loaded page has no background specified.
	*/
	virtual HRESULT setDrawBackground( BOOL in_bDraw ) = 0;

	/*
	* @desc Specifiy maximum zoom factor
	*/
	virtual HRESULT setMaxZoomFactor( float factor ) = 0;
	/*
	* @desc Specifiy minimum zoom factor
	*/
	virtual HRESULT setMinZoomFactor( float factor ) = 0;


	/*
	* @desc minimize the size of backingstore
	* backingstore means the memory gc used by WebControl
	* WebControl use bigger memmory gc than actual window size, 
	* So If you want to save memory when the webcontrol is hided or unfocused, 
	* use this funtion to shrink the size of memorygc same as window size .
	* The actual minimum backingstore size is calculated by WebContorl implementation.
	*/
	virtual HRESULT minimizeBackingStore() = 0;

	/*
	* @desc maximize the size of backingstore
	* Use this funtion to restore the size of backingstore after calling minimizeBackingStore().
	* The actual maximum backingstore size is calculated by WebControl implementation.
	*/
	virtual HRESULT maximizeBackingStore() = 0;

	virtual HRESULT setInitialOverviewMode( bool bEnable ) = 0;

	virtual void setJSPromptResultString(const char *inStrData) = 0 ;
	virtual void setJSConfirmResult(bool bResult) = 0 ;
	 /*
	 * @desc Sends Event for engine destruct.
	 * @return none
         * gnana for end key performance SFB_IMPROVE_END_KEY
	 */
	
	virtual void cancelLingerTimer() = 0;//Oct 21 Anshul
	virtual void InstanceExit() = 0;
	virtual void setEndKeyPressed(bool status) = 0;

	//for clients of webControl who cant override the default preferences eg widget. default is false ie can override.
	virtual void setoverrideDefaultPreferences(bool set) = 0 ;

// [todo]
// need to check if the following funtions are required
#if 0
	/***************************************************************************
	* For ACCESS
	****************************************************************************/
	/*
	 * @desc loads URL for ex-method
	 * @param In_pszUrl : URL
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT LoadURLEx( char* in_pszUrl ) = 0; // for ACCESS

	/*
	 * @desc shows a touch effect on indicated position.
	 * @param x, y : position
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT CreateTouchEffect( int x, int y ) = 0; // for ACCESS

	/*
	 * @desc moves a focus from focused Iframe to parent page.
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT EscapeIFrameFocused( void ) = 0; // for ACCESS

	/*
	 * @desc moves a focus from multi selection to parent page.
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT EscapeMultiSelection( void ) = 0; // for ACCESS

	/*
	 * @desc gets a current page informatioin.
	 * @param out_pImageInfo : pointer to IPageInfo
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT GetCurrentPageInfoEx( IPageInfo* out_pPageInfo ) = 0;  // for ACCESS

	/*
	 * @desc check focus is in a Frame
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT IsFrameSelected( void ) = 0; // for ACCESS

	/*
	 * @desc check focusing is in a Frame
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT IsFrameSelecting( void ) = 0; // for ACCESS

	/*
	 * @desc check focus is in a IFrame
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT IsIFrameFocused( void ) = 0; // for ACCESS

	/*
	 * @desc check whether browser engine is loading contents or not.
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT IsLoadingPage( void ) = 0; // for ACCESS ( ?? )

	/*
	 * @desc check focus is on a Multi selection
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT IsMultiSelection( void ) = 0; // for ACCESS ( ?? )

	/*
	 * @desc check select box is opened or not
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT IsSelectBoxOpened( void ) = 0; // for Access ( ?? )

	/*
	 * @desc
	 * @param
	 * @return
	 */
	virtual HRESULT ScrollSelectBox( int y ) = 0; // for ACCESS

	/*
	 * @desc
	 * @param
	 * @return
	 */
	virtual HRESULT SetIFrameFocused( void ) = 0; // for ACCESS

	/*
	 * @desc sets key is repeated for virtual pointer mode to browser engine
	 * @param in_OnOff :  repeated
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT SetVirtualPointerKeyRepeated( bool in_OnOff ) = 0; // for ACCESS ( ?? )

	/*
	 * @desc clears Browser Engine's GC
	 * @param
	 * @return
	 */
	virtual void ClearGC() = 0; // for ACCESS
#endif
};


} // namespace WebControl

#endif // __IContents_Defined