/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		WebImage.cpp
 * @brief		implement WebImage
 * @author	Sangheon Kim (sangheon77.kim@samsung.com)
 * @version	1.0
 * @date		2008.04.01	v1.0		Created
 */

#include "ImgCODEC.h"

#include "WebImage.h"

#include "HitTestResult.h"
#include "Image.h"
#include "DeprecatedString.h"
#include "KURL.h"


using namespace WebCore;

namespace WebControl {

WebImage::WebImage( WebCore::HitTestResult& result)
{
	m_pHitResult = new HitTestResult( result );
}

WebImage::~WebImage()
{
	if( m_pHitResult )
		delete m_pHitResult;
}

/*
 * @desc gets codec type
 * @param out_pszBuffer : pointer to buffer
 * @param in_nBufferLen : length of buffer
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebImage::getCodecType( ImgCodecType *out_codec )
{
	WebCore::Image* pImage = m_pHitResult->image();

	if( pImage == NULL )
	{
		SysDebug(( MID_EXCEPTION, "WebImage::getCodecType : pImage == NULL\n" ));
		return HRESULT_NOK;
	}

	if (pImage->data() == NULL)
	{
        SysDebug(( MID_EXCEPTION, "WebImage::getCodecType : SharedBuffer is Empty \n" ));
		return HRESULT_NOK;  
	}
  
	ImgCodecType codec = ImgGetDataCodecType( (unsigned char*)(pImage->data()->data()) ); 

	switch( codec )
	{
	case ::IMG_DEC_GIF:
		*out_codec = IMG_CODEC_GIF;
		break;

	case ::IMG_DEC_PNG:
		*out_codec = IMG_CODEC_PNG;
		break;

	case ::IMG_DEC_WBMP:
		*out_codec = IMG_CODEC_WBMP;
		break;

	case ::IMG_DEC_JPEG:
		*out_codec = IMG_CODEC_JPEG;
		break;

	case ::IMG_DEC_BMP:
		*out_codec = IMG_CODEC_BMP;
		break;

	case ::IMG_DEC_TIF:
		*out_codec = IMG_CODEC_TIF;
		break;

	case ::IMG_DEC_AGIF:
		*out_codec = IMG_CODEC_AGIF;
		break;

	default:
		*out_codec = IMG_CODEC_UNKNOWN;
		break;
	}

	return HRESULT_OK;

}

HRESULT WebImage::getURL( char* out_pszBuffer, int in_pnBufferLen )
{
	int length;

	DeprecatedString url = m_pHitResult->absoluteImageURL().deprecatedString();
	length = url.length();

	if( in_pnBufferLen < length )
	{
		SysDebug(( MID_EXCEPTION, "WebImage::getURL : buffer size is too small\n" ));
		return HRESULT_NO_MEMORY;
	}

	AcStrcpy( out_pszBuffer, url.latin1() );

	return HRESULT_OK;
}

/*
 * @desc gets raw data
 * @param out_pszBuffer : pointer to buffer
 * @param in_nBufferLen : buffer length
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebImage::getRawData( char* out_pszBuffer, int in_nBufferLen )
{
	WebCore::Image* pImage = m_pHitResult->image();

	if( pImage == NULL )
	{
		SysDebug(( MID_EXCEPTION, "WebImage::getRawDataSize : pImage == NULL\n" ));
		return HRESULT_NOK;
	}

	if( in_nBufferLen < pImage->data()->size() )
	{
		SysDebug(( MID_EXCEPTION, "WebImage::getRawDataSize : buffer size is too small\n" ));
		return HRESULT_NO_MEMORY;
	}

	AcMemcpy( out_pszBuffer, pImage->data()->data() ,pImage->data()->size() );

	return HRESULT_OK;
}

/*
 * @desc gets Raw data size
 * @param 
 * @return int total size
 */
HRESULT WebImage::getRawDataSize( int *out_size )
{
	WebCore::Image* pImage = m_pHitResult->image();

	if( pImage == NULL )
	{
		SysDebug(( MID_EXCEPTION, "WebImage::getRawDataSize : pImage == NULL\n" ));
		return HRESULT_NOK;
	}

	*out_size = pImage->data()->size();

	return HRESULT_OK;
}



} // namespace WebFrame